@@ -23,6 +23,7 @@
 #include <lv2/lv2plug.in/ns/ext/port-props/port-props.h>
 #include <lv2/lv2plug.in/ns/ext/port-groups/port-groups.h>
 #include <lv2/lv2plug.in/ns/ext/resize-port/resize-port.h>
+#include <lv2/lv2plug.in/ns/ext/buf-size/buf-size.h>
 #include <lv2/lv2plug.in/ns/extensions/units/units.h>
 #include <lv2/lv2plug.in/ns/extensions/ui/ui.h>
 #include <lv2/lv2plug.in/ns/ext/instance-access/instance-access.h>
@@ -34,10 +35,15 @@
 // Include common definitions
 #include <container/const.h>
 
+// Some definitions that may be lacking in LV2
 #ifndef LV2_ATOM__Object
-    #define LV2_ATOM__Object        LV2_ATOM_PREFIX "Object"
+    #define LV2_ATOM__Object            LV2_ATOM_PREFIX "Object"
 #endif /* LV2_ATOM__Object */
 
+#ifndef LV2_STATE__StateChanged
+    #define LV2_STATE__StateChanged     LV2_STATE_PREFIX "StateChanged"
+#endif /* LV2_STATE__StateChanged */
+
 #pragma pack(push, 1)
 typedef struct LV2_Atom_Midi
 {
@@ -46,6 +52,8 @@ typedef struct LV2_Atom_Midi
 } LV2_Atom_Midi;
 #pragma pack(pop)
 
+#define LV2PORT_MAX_BLOCK_LENGTH        8192
+
 namespace lsp
 {
     #define LSP_LV2_ATOM_KEY_SIZE       (sizeof(uint32_t) * 2)
@@ -90,6 +98,7 @@ namespace lsp
             LV2_URID_Unmap         *unmap;
             LV2_Worker_Schedule    *sched;
             LV2_Inline_Display     *iDisplay;
+            LV2_State_Map_Path     *mapPath;
             LV2UI_Resize           *ui_resize;
             LV2Wrapper             *pWrapper;
 
@@ -107,13 +116,12 @@ namespace lsp
             LV2_URID                uridEventTransfer;
             LV2_URID                uridObject;
             LV2_URID                uridBlank;
-            LV2_URID                uridState;
-            LV2_URID                uridStateChange;
-            LV2_URID                uridStateFlags;
+            LV2_URID                uridStateChanged;
             LV2_URID                uridStateRequest;
-            LV2_URID                uridConnectUI;
             LV2_URID                uridUINotification;
+            LV2_URID                uridConnectUI;
             LV2_URID                uridDisconnectUI;
+            LV2_URID                uridDumpState;
             LV2_URID                uridPathType;
             LV2_URID                uridMidiEventType;
             LV2_URID                uridKvtKeys;
@@ -131,7 +139,6 @@ namespace lsp
             LV2_URID                uridPatchGet;
             LV2_URID                uridPatchSet;
             LV2_URID                uridPatchMessage;
-            LV2_URID                uridPatchResponse;
             LV2_URID                uridPatchProperty;
             LV2_URID                uridPatchValue;
 
@@ -149,6 +156,8 @@ namespace lsp
             LV2_URID                uridTimeBeatsPerBar;
             LV2_URID                uridTimeBeatsPerMinute;
 
+            LV2_URID                uridMaxBlockLength;
+
             // OSC-related URIDs
             LV2_URID                uridOscBundle;
             LV2_URID                uridOscBundleTimetag;
@@ -179,12 +188,13 @@ namespace lsp
 
             LV2UI_Controller        ctl;
             LV2UI_Write_Function    wf;
-            ssize_t                 nAtomIn;        // Atom input port identifier
-            ssize_t                 nAtomOut;       // Atom output port identifier
-            uint8_t                *pBuffer;        // Atom serialization buffer
-            size_t                  nBufSize;       // Atom serialization buffer size
-            float                   fUIRefreshRate; // UI refresh rate
-            void                   *pParentWindow;  // Parent window handle
+            ssize_t                 nAtomIn;            // Atom input port identifier
+            ssize_t                 nAtomOut;           // Atom output port identifier
+            size_t                  nMaxBlockLength;    // Maximum size of audio block passed to plugin
+            uint8_t                *pBuffer;            // Atom serialization buffer
+            size_t                  nBufSize;           // Atom serialization buffer size
+            float                   fUIRefreshRate;     // UI refresh rate
+            void                   *pParentWindow;      // Parent window handle
 
         public:
             inline LV2Extensions(const LV2_Feature* const* feat, const char *uri, LV2UI_Controller lv2_ctl, LV2UI_Write_Function lv2_write)
@@ -194,9 +204,11 @@ namespace lsp
                 ui_resize           = NULL;
                 sched               = NULL;
                 iDisplay            = NULL;
+                mapPath             = NULL;
                 pParentWindow       = NULL;
                 pWrapper            = NULL;
                 fUIRefreshRate      = MESH_REFRESH_RATE;
+                nMaxBlockLength     = LV2PORT_MAX_BLOCK_LENGTH;
 
                 const LV2_Options_Option *opts = NULL;
 
@@ -221,8 +233,10 @@ namespace lsp
                             ui_resize = reinterpret_cast<LV2UI_Resize *>(f->data);
                         else if (!strcmp(f->URI, LV2_INLINEDISPLAY__queue_draw))
                             iDisplay = reinterpret_cast<LV2_Inline_Display *>(f->data);
+                    #if LSP_LV2_NO_INSTANCE_ACCESS != 1
                         else if (!strcmp(f->URI, LV2_INSTANCE_ACCESS_URI))
                             pWrapper = reinterpret_cast<LV2Wrapper *>(f->data);
+                    #endif
                         else if (!strcmp(f->URI, LV2_OPTIONS__options))
                         {
                             lsp_trace("Received options from host");
@@ -251,13 +265,11 @@ namespace lsp
                 uridEventTransfer           = map_uri(LV2_ATOM__eventTransfer);
                 uridObject                  = forge.Object;
                 uridBlank                   = map_uri(LV2_ATOM__Blank);
-                uridState                   = map_primitive("state");
-                uridConnectUI               = map_primitive("ui_connect");
+                uridStateChanged            = map_uri(LV2_STATE__StateChanged);
                 uridUINotification          = map_type("UINotification");
+                uridConnectUI               = map_primitive("ui_connect");
                 uridDisconnectUI            = map_primitive("ui_disconnect");
-                uridStateRequest            = map_type("StateRequest");
-                uridStateChange             = map_type("StateChange");
-                uridStateFlags              = map_type("StateFlags");
+                uridDumpState               = map_primitive("dumpState");
                 uridPathType                = forge.Path;
                 uridMidiEventType           = map_uri(LV2_MIDI__MidiEvent);
                 uridKvtObject               = map_primitive("KVT");
@@ -275,7 +287,6 @@ namespace lsp
                 uridPatchGet                = map_uri(LV2_PATCH__Get);
                 uridPatchSet                = map_uri(LV2_PATCH__Set);
                 uridPatchMessage            = map_uri(LV2_PATCH__Message);
-                uridPatchResponse           = map_uri(LV2_PATCH__Response);
                 uridPatchProperty           = map_uri(LV2_PATCH__property);
                 uridPatchValue              = map_uri(LV2_PATCH__value);
                 uridAtomUrid                = forge.URID;
@@ -292,6 +303,8 @@ namespace lsp
                 uridTimeBeatsPerBar         = map_uri(LV2_TIME__beatsPerBar);
                 uridTimeBeatsPerMinute      = map_uri(LV2_TIME__beatsPerMinute);
 
+                uridMaxBlockLength          = map_uri(LV2_BUF_SIZE__maxBlockLength);
+
                 // OSC-related URIDs
                 uridOscBundle               = map_uri(LV2_OSC__Bundle);
                 uridOscBundleTimetag        = map_uri(LV2_OSC__bundleTimetag);
@@ -340,7 +353,7 @@ namespace lsp
                                 opts->value
                         );
 
-                        // Move to next option
+                        // Update Rate for the UI
                         if ((opts->context == LV2_OPTIONS_INSTANCE) &&
                             (opts->key == uridUpdateRate) &&
                             (opts->value != NULL))
@@ -355,7 +368,21 @@ namespace lsp
                                 fUIRefreshRate  = *reinterpret_cast<const int64_t *>(opts->value);
                             if (fUIRefreshRate < 0)
                                 fUIRefreshRate = MESH_REFRESH_RATE;
-                            lsp_trace("UI refresh rate was set to %f", fUIRefreshRate);
+                            lsp_trace("UI refresh rate has been set to %f", fUIRefreshRate);
+                        }
+
+                        if ((opts->context == LV2_OPTIONS_INSTANCE) &&
+                            (opts->key == uridMaxBlockLength) &&
+                            (opts->value != NULL))
+                        {
+                            ssize_t blk_len = nMaxBlockLength;
+                            if ((opts->type == forge.Int) && (opts->size == sizeof(int32_t)))
+                                blk_len = *reinterpret_cast<const int32_t *>(opts->value);
+                            else if ((opts->type == forge.Long) && (opts->size == sizeof(int64_t)))
+                                blk_len = *reinterpret_cast<const int64_t *>(opts->value);
+                            if (blk_len > 0)
+                                nMaxBlockLength = blk_len;
+                            lsp_trace("MaxBlockLength has been set to %d", int(nMaxBlockLength));
                         }
 
                         opts++;
@@ -570,6 +597,15 @@ namespace lsp
                 hStore          = store;
                 hRetrieve       = retrieve;
                 hHandle         = handle;
+
+                for (size_t i=0; features[i]; ++i)
+                {
+                    const LV2_Feature *f = features[i];
+                    lsp_trace("Host reported state extension uri=%s, data=%p", f->URI, f->data);
+
+                    if (!::strcmp(f->URI, LV2_STATE__mapPath))
+                        mapPath = reinterpret_cast<LV2_State_Map_Path *>(f->data);
+                }
             }
 
             inline void store_value(LV2_URID urid, LV2_URID type, const void *data, size_t size)
@@ -610,6 +646,7 @@ namespace lsp
                 hStore          = NULL;
                 hRetrieve       = NULL;
                 hHandle         = NULL;
+                mapPath         = NULL;
             }
 
             inline bool ui_create_atom_transport(size_t port, size_t buf_size)
@@ -645,19 +682,24 @@ namespace lsp
                 forge_pop(&frame);
                 write_data(nAtomOut, lv2_atom_total_size(msg), uridEventTransfer, msg);
 
-                // Send PATCH GET message
-                lsp_trace("Sending PATCH GET message");
-                msg = forge_object(&frame, uridChunk, uridPatchGet);
+                return true;
+            }
+
+            inline bool request_state_dump()
+            {
+                if (map == NULL)
+                    return false;
+
+                // Prepare forge for transfer
+                LV2_Atom_Forge_Frame    frame;
+                forge_set_buffer(pBuffer, nBufSize);
+
+                // Send DUMP STATE message
+                lsp_trace("Sending DUMP STATE message");
+                LV2_Atom *msg = forge_object(&frame, uridDumpState, uridUINotification);
                 forge_pop(&frame);
                 write_data(nAtomOut, lv2_atom_total_size(msg), uridEventTransfer, msg);
 
-                // Sent STATE REQUEST message
-                lsp_trace("Sending STATE REQUEST message");
-                msg = forge_object(&frame, uridState, uridStateRequest);
-                forge_pop(&frame);
-                write_data(nAtomOut, lv2_atom_total_size(msg), uridEventTransfer, msg);
-
-                lsp_trace("patch request has been written");
                 return true;
             }
 
@@ -679,7 +721,7 @@ namespace lsp
                 write_data(nAtomOut, lv2_atom_total_size(msg), uridEventTransfer, msg);
             }
 
-            inline bool ui_write_patch(LV2Serializable *p)
+            bool ui_write_patch(LV2Serializable *p)
             {
                 if ((map == NULL) || (p->get_urid() <= 0))
                     return false;
@@ -699,30 +741,6 @@ namespace lsp
                 write_data(nAtomOut, lv2_atom_total_size(msg), uridEventTransfer, msg);
                 return true;
             }
-
-            inline bool ui_write_state(LV2Serializable *p, size_t flags = 0)
-            {
-                if ((map == NULL) || (p->get_urid() <= 0))
-                    return false;
-
-                // Forge PATCH SET message
-                LV2_Atom_Forge_Frame    frame;
-                forge_set_buffer(pBuffer, nBufSize);
-
-                forge_frame_time(0);
-                LV2_Atom *msg = forge_object(&frame, uridState, uridStateChange);
-                if (flags != 0)
-                {
-                    forge_key(uridStateFlags);
-                    forge_int(int(flags));
-                }
-                forge_key(p->get_urid());
-                p->serialize();
-                forge_pop(&frame);
-
-                write_data(nAtomOut, lv2_atom_total_size(msg), uridEventTransfer, msg);
-                return true;
-            }
     };
 
     typedef struct LV2Mesh
@@ -800,7 +818,6 @@ namespace lsp
     } LV2Mesh;
 
     #define PATCH_OVERHEAD  (sizeof(LV2_Atom_Property) + sizeof(LV2_Atom_URID) + sizeof(LV2_Atom) + 0x20)
-    #define STATE_OVERHEAD  (sizeof(LV2_Atom_Object) + sizeof(LV2_Atom_Property) + sizeof(LV2_Atom_Float) + 0x20)
 
     inline long lv2_all_port_sizes(const port_t *ports, bool in, bool out)
     {
@@ -820,7 +837,7 @@ namespace lsp
             {
                 case R_CONTROL:
                 case R_METER:
-                    size            += STATE_OVERHEAD + sizeof(LV2_Atom_Float);
+                    size            += PATCH_OVERHEAD + sizeof(LV2_Atom_Float);
                     break;
                 case R_MESH:
                     if (IS_OUT_PORT(p) && (!out))
