@@ -17,13 +17,15 @@ namespace lsp
             LV2Extensions          *pExt;
             LV2_URID                urid;
             ssize_t                 nID;
+            bool                    bVirtual;
 
         public:
-            explicit LV2Port(const port_t *meta, LV2Extensions *ext): IPort(meta)
+            explicit LV2Port(const port_t *meta, LV2Extensions *ext, bool virt): IPort(meta)
             {
                 pExt            =   ext;
                 urid            =   (meta != NULL) ? pExt->map_port(meta->id) : -1;
                 nID             =   -1;
+                bVirtual        =   virt;
             }
             virtual ~LV2Port()
             {
@@ -56,8 +58,9 @@ namespace lsp
 
             /** Deserialize state of the port from LV2 Atom
              * @param flags additional flags
+             * @return true if internal state of the port has changed
              */
-            virtual void deserialize(const void *data, size_t flags)  { };
+            virtual bool deserialize(const void *data, size_t flags)  { return false; };
 
             /** Get type of the LV2 port in terms of Atom
              *
@@ -99,17 +102,23 @@ namespace lsp
              * @param id port ID
              */
             inline void             set_id(size_t id)   { nID = id;     }
+
+            /**
+             * Check that port is virtual
+             * @return true if port is virtual (non controlled by DAW and stored in PLUGIN STATE)
+             */
+            inline bool            is_virtual() const  { return bVirtual; }
     };
 
     class LV2PortGroup: public LV2Port
     {
-        private:
+        protected:
             float                   nCurrRow;
             size_t                  nCols;
             size_t                  nRows;
 
         public:
-            explicit LV2PortGroup(const port_t *meta, LV2Extensions *ext) : LV2Port(meta, ext)
+            explicit LV2PortGroup(const port_t *meta, LV2Extensions *ext, bool virt) : LV2Port(meta, ext, virt)
             {
                 nCurrRow            = meta->start;
                 nCols               = port_list_size(meta->members);
@@ -140,11 +149,16 @@ namespace lsp
                 pExt->forge_int(nCurrRow);
             }
 
-            virtual void deserialize(const void *data, size_t flags)
+            virtual bool deserialize(const void *data, size_t flags)
             {
                 const LV2_Atom_Int *atom = reinterpret_cast<const LV2_Atom_Int *>(data);
-                if ((atom->body >= 0) && (atom->body < int32_t(nRows)))
+                if ((atom->body >= 0) && (atom->body < int32_t(nRows)) && (nCurrRow != atom->body))
+                {
                     nCurrRow        = atom->body;
+                    return true;
+                }
+
+                return false;
             }
 
             virtual void save()
@@ -183,10 +197,10 @@ namespace lsp
     class LV2RawPort: public LV2Port
     {
         protected:
-            void *pBuffer;
+            void       *pBuffer;
 
         public:
-            explicit LV2RawPort(const port_t *meta, LV2Extensions *ext) : LV2Port(meta, ext), pBuffer(NULL) { }
+            explicit LV2RawPort(const port_t *meta, LV2Extensions *ext, bool virt) : LV2Port(meta, ext, virt), pBuffer(NULL) { }
             virtual ~LV2RawPort() { pBuffer = NULL; };
 
         public:
@@ -200,20 +214,59 @@ namespace lsp
 
     class LV2AudioPort: public LV2RawPort
     {
+        protected:
+            float      *pSanitized;
+            float      *pFrame;
+
         public:
-            LV2AudioPort(const port_t *meta, LV2Extensions *ext) : LV2RawPort(meta, ext) { }
-            virtual ~LV2AudioPort() { };
+            explicit LV2AudioPort(const port_t *meta, LV2Extensions *ext) : LV2RawPort(meta, ext, false)
+            {
+                pSanitized  = NULL;
+                pFrame      = NULL;
+
+                if (IS_IN_PORT(pMetadata))
+                {
+                    size_t length = pExt->nMaxBlockLength;
+                    pSanitized = reinterpret_cast<float *>(::malloc(sizeof(float) * length));
+                    if (pSanitized != NULL)
+                        dsp::fill_zero(pSanitized, length);
+                    else
+                        lsp_warn("Failed to allocate sanitize buffer for port %s", pMetadata->id);
+                }
+            }
+
+            virtual ~LV2AudioPort()
+            {
+                if (pSanitized != NULL)
+                {
+                    ::free(pSanitized);
+                    pSanitized = NULL;
+                }
+            };
+
+            virtual void *getBuffer() { return pFrame; };
+
+            // Should be always called at least once after bind() and before processing
+            void sanitize(size_t off, size_t samples)
+            {
+                pFrame  = reinterpret_cast<float *>(pBuffer) + off;
+                if (pSanitized != NULL)
+                {
+                    dsp::sanitize2(pSanitized, reinterpret_cast<float *>(pFrame), samples);
+                    pFrame      = pSanitized;
+                }
+            }
     };
 
     class LV2InputPort: public LV2Port
     {
-        private:
+        protected:
             const float    *pData;
             float           fValue;
             float           fPrev;
 
         public:
-            explicit LV2InputPort(const port_t *meta, LV2Extensions *ext) : LV2Port(meta, ext)
+            explicit LV2InputPort(const port_t *meta, LV2Extensions *ext, bool virt) : LV2Port(meta, ext, virt)
             {
                 pData       = NULL;
                 fValue      = meta->start;
@@ -280,10 +333,14 @@ namespace lsp
                     fValue      = limit_value(pMetadata, *(reinterpret_cast<const float *>(data)));
             }
 
-            virtual void deserialize(const void *data, size_t flags)
+            virtual bool deserialize(const void *data, size_t flags)
             {
                 const LV2_Atom_Float *atom = reinterpret_cast<const LV2_Atom_Float *>(data);
+                if (fValue == atom->body)
+                    return false;
+
                 fValue      = atom->body;
+                return true;
             }
 
             virtual void serialize()
@@ -299,15 +356,65 @@ namespace lsp
             }
     };
 
+    class LV2BypassPort: public LV2InputPort
+    {
+        public:
+            explicit LV2BypassPort(const port_t *meta, LV2Extensions *ext) : LV2InputPort(meta, ext, false) { }
+
+            virtual ~LV2BypassPort() {}
+
+        public:
+            virtual float getValue()
+            {
+                return pMetadata->max - fValue;
+            }
+
+            virtual void setValue(float value)
+            {
+                fValue      = pMetadata->max - value;
+            }
+
+            virtual void save()
+            {
+                if (nID >= 0)
+                    return;
+                float value = pMetadata->max - fValue;
+                lsp_trace("save port id=%s, urid=%d (%s), value=%f", pMetadata->id, urid, get_uri(), value);
+                pExt->store_value(urid, pExt->forge.Float, &value, sizeof(float));
+            }
+
+            virtual void restore()
+            {
+                if (nID >= 0)
+                    return;
+                lsp_trace("restore port id=%s, urid=%d (%s)", pMetadata->id, urid, get_uri());
+                size_t count            = 0;
+                const void *data        = pExt->restore_value(urid, pExt->forge.Float, &count);
+                if ((count == sizeof(float)) && (data != NULL))
+                    fValue      = limit_value(pMetadata, pMetadata->max - *(reinterpret_cast<const float *>(data)));
+            }
+
+            virtual bool deserialize(const void *data, size_t flags)
+            {
+                const LV2_Atom_Float *atom = reinterpret_cast<const LV2_Atom_Float *>(data);
+                float v = pMetadata->max - atom->body;
+                if (fValue == v)
+                    return false;
+
+                fValue      = v;
+                return true;
+            }
+    };
+
     class LV2OutputPort: public LV2Port
     {
-        private:
+        protected:
             float  *pData;
             float   fPrev;
             float   fValue;
 
         public:
-            explicit LV2OutputPort(const port_t *meta, LV2Extensions *ext) : LV2Port(meta, ext)
+            explicit LV2OutputPort(const port_t *meta, LV2Extensions *ext) : LV2Port(meta, ext, false)
             {
                 pData       = NULL;
                 fPrev       = meta->start;
@@ -388,11 +495,11 @@ namespace lsp
 
     class LV2MeshPort: public LV2Port
     {
-        private:
+        protected:
             LV2Mesh                 sMesh;
 
         public:
-            explicit LV2MeshPort(const port_t *meta, LV2Extensions *ext): LV2Port(meta, ext)
+            explicit LV2MeshPort(const port_t *meta, LV2Extensions *ext): LV2Port(meta, ext, false)
             {
                 sMesh.init(meta, ext);
             }
@@ -445,12 +552,12 @@ namespace lsp
 
     class LV2FrameBufferPort: public LV2Port
     {
-        private:
+        protected:
             frame_buffer_t      sFB;
             size_t              nRowID;
 
         public:
-            explicit LV2FrameBufferPort(const port_t *meta, LV2Extensions *ext): LV2Port(meta, ext)
+            explicit LV2FrameBufferPort(const port_t *meta, LV2Extensions *ext): LV2Port(meta, ext, false)
             {
                 sFB.init(meta->start, meta->step);
                 nRowID = 0;
@@ -515,7 +622,7 @@ namespace lsp
 
     class LV2PathPort: public LV2Port
     {
-        private:
+        protected:
             lv2_path_t          sPath;
             atomic_t            nLastChange;
 
@@ -526,7 +633,7 @@ namespace lsp
             }
 
         public:
-            explicit LV2PathPort(const port_t *meta, LV2Extensions *ext): LV2Port(meta, ext)
+            explicit LV2PathPort(const port_t *meta, LV2Extensions *ext): LV2Port(meta, ext, true)
             {
                 sPath.init();
                 nLastChange = sPath.nChanges;
@@ -540,9 +647,31 @@ namespace lsp
 
             virtual void save()
             {
-                lsp_trace("save port id=%s, urid=%d (%s), value=%s", pMetadata->id, urid, get_uri(), sPath.sPath);
-                if (strlen(sPath.sPath) > 0)
-                    pExt->store_value(urid, pExt->uridPathType, sPath.sPath, strlen(sPath.sPath) + sizeof(char));
+                const char *path = sPath.sPath;
+
+                lsp_trace("save port id=%s, urid=%d (%s), value=%s", pMetadata->id, urid, get_uri(), path);
+
+                if (::strlen(path) > 0)
+                {
+                    char *mapped = NULL;
+
+                    // We need to translate absolute path to relative path?
+                    if ((pExt->mapPath != NULL) && (::strstr(path, LSP_BUILTIN_PREFIX) != path))
+                    {
+                        mapped = pExt->mapPath->abstract_path(pExt->mapPath->handle, path);
+                        if (mapped != NULL)
+                        {
+                            lsp_trace("mapped path: %s -> %s", path, mapped);
+                            path = mapped;
+                        }
+                    }
+
+                    // Store the actual value of the path
+                    pExt->store_value(urid, pExt->uridPathType, path, ::strlen(path) + sizeof(char));
+
+                    if (mapped != NULL)
+                        ::free(mapped);
+                }
             }
 
             void tx_request()
@@ -558,6 +687,7 @@ namespace lsp
                 uint32_t type           = -1;
 
                 const char *path        = reinterpret_cast<const char *>(pExt->retrieve_value(urid, &type, &count));
+                char *mapped            = NULL;
                 if (path != NULL)
                 {
                     if (type == pExt->forge.URID)
@@ -565,7 +695,7 @@ namespace lsp
                         const LV2_URID *urid    = reinterpret_cast<const LV2_URID *>(path);
                         path                = pExt->unmap_urid(*urid);
                         if (path != NULL)
-                            count               = strnlen(path, PATH_MAX);
+                            count               = ::strnlen(path, PATH_MAX-1);
                     }
                     else if ((type != pExt->uridPathType) && (type != pExt->forge.String))
                     {
@@ -576,10 +706,34 @@ namespace lsp
                 }
 
                 if ((path != NULL) && (count > 0))
+                {
+                    // Save path as temporary variable
+                    char tmp_path[PATH_MAX];
+                    ::strncpy(tmp_path, path, count);
+                    tmp_path[count] = '\0';
+                    path        = tmp_path;
+
+                    // We need to translate relative path to absolute path?
+                    if ((pExt->mapPath != NULL) && (::strstr(path, LSP_BUILTIN_PREFIX) != path))
+                    {
+                        mapped = pExt->mapPath->absolute_path(pExt->mapPath->handle, path);
+                        if (mapped != NULL)
+                        {
+                            lsp_trace("unmapped path: %s -> %s", path, mapped);
+                            path  = mapped;
+                            count = ::strnlen(path, PATH_MAX-1);
+                        }
+                    }
+
+                    // Restore the actual value of the path
                     set_string(path, count, PF_STATE_IMPORT);
+                }
                 else
                     set_string("", 0, PF_STATE_IMPORT);
                 tx_request();
+
+                if (mapped != NULL)
+                    ::free(mapped);
             }
 
             virtual bool tx_pending()
@@ -599,12 +753,14 @@ namespace lsp
                 reset_tx_pending();
             }
 
-            virtual void deserialize(const void *data, size_t flags)
+            virtual bool deserialize(const void *data, size_t flags)
             {
-                const LV2_Atom *atom = reinterpret_cast<const LV2_Atom *>(data);
+                const LV2_Atom *atom = static_cast<const LV2_Atom *>(data);
                 if (atom->type != pExt->uridPathType)
-                    return;
+                    return false;
+
                 set_string(reinterpret_cast<const char *>(atom + 1), atom->size, flags);
+                return true;
             }
 
             virtual LV2_URID get_type_urid()    { return pExt->uridPathType; }
@@ -617,11 +773,11 @@ namespace lsp
 
     class LV2MidiPort: public LV2Port
     {
-        private:
+        protected:
             midi_t      sQueue;
 
         public:
-            explicit LV2MidiPort(const port_t *meta, LV2Extensions *ext): LV2Port(meta, ext)
+            explicit LV2MidiPort(const port_t *meta, LV2Extensions *ext): LV2Port(meta, ext, false)
             {
                 sQueue.clear();
             }
@@ -635,11 +791,11 @@ namespace lsp
 
     class LV2OscPort: public LV2Port
     {
-        private:
+        protected:
             osc_buffer_t     *pFB;
 
         public:
-            explicit LV2OscPort(const port_t *meta, LV2Extensions *ext) : LV2Port(meta, ext)
+            explicit LV2OscPort(const port_t *meta, LV2Extensions *ext) : LV2Port(meta, ext, false)
             {
                 pFB     = NULL;
             }
