@@ -56,9 +56,10 @@ typedef struct vst_state_header
 #define VST_GET_VERSION_STRNAME         "vst_get_lsp_build_version"
 
 #define LSP_VST_USER_MAGIC              CCONST('L', 'S', 'P', 'U')
-#define VST_PROGRAM_HDR_SIZE            (sizeof(fxProgram) - 2 * sizeof(VstInt32))
 #define VST_BANK_HDR_SKIP               (2*sizeof(VstInt32))
+#define VST_PROGRAM_HDR_SKIP            (2*sizeof(VstInt32))
 #define VST_BANK_HDR_SIZE               (sizeof(fxBank) - VST_BANK_HDR_SKIP)
+#define VST_PROGRAM_HDR_SIZE            (sizeof(fxProgram) - VST_PROGRAM_HDR_SKIP)
 #define VST_STATE_BUFFER_SIZE           (VST_BANK_HDR_SIZE + sizeof(vst_state))
 
 enum
