@@ -138,6 +138,7 @@ namespace lsp
             float                   nCurrRow;
             size_t                  nCols;
             size_t                  nRows;
+            volatile vst_serial_t   nSID; // Serial ID of the parameter
 
         public:
             explicit VSTPortGroup(const port_t *meta, AEffect *effect, audioMasterCallback callback) : VSTPort(meta, effect, callback)
@@ -167,6 +168,11 @@ namespace lsp
                 return nCurrRow;
             }
 
+            inline vst_serial_t getSID()
+            {
+                return nSID;
+            }
+
             virtual bool serializable() const { return true; }
 
             virtual void serialize(vst_chunk_t *chunk)
@@ -181,7 +187,10 @@ namespace lsp
                     return -1;
                 int32_t value   = BE_TO_CPU(*(reinterpret_cast<const int32_t *>(data)));
                 if ((value >= 0) && (value < ssize_t(nRows)))
+                {
                     nCurrRow        = value;
+                    ++nSID;
+                }
                 return sizeof(int32_t);
             }
 
@@ -192,7 +201,10 @@ namespace lsp
 
                 int32_t v       = BE_TO_CPU(*(reinterpret_cast<const int32_t *>(data)));
                 if ((v >= 0) && (v < ssize_t(nRows)))
+                {
                     nCurrRow        = v;
+                    ++nSID;
+                }
 
                 return true;
             }
@@ -206,26 +218,71 @@ namespace lsp
     class VSTAudioPort: public VSTPort
     {
         private:
-            float *pBuffer;
+            float          *pBuffer;
+            float          *pSanitized;
+            size_t          nBufSize;
 
         public:
             explicit VSTAudioPort(const port_t *meta, AEffect *effect, audioMasterCallback callback) : VSTPort(meta, effect, callback)
             {
                 pBuffer     = NULL;
+                pSanitized  = NULL;
+                nBufSize    = 0;
             }
 
             virtual ~VSTAudioPort()
             {
                 pBuffer     = NULL;
+
+                if (pSanitized != NULL)
+                {
+                    ::free(pSanitized);
+                    pSanitized  = NULL;
+                    nBufSize    = 0;
+                }
             };
 
         public:
             virtual void *getBuffer() { return pBuffer; };
 
-            void bind(float *data)
+            void bind(float *data, size_t samples)
             {
-                pBuffer = data;
+                pBuffer     = data;
+                if (pSanitized == NULL)
+                    return;
+
+                // Perform sanitize() if possible
+                if (samples <= nBufSize)
+                {
+                    dsp::sanitize2(pSanitized, reinterpret_cast<float *>(pBuffer), samples);
+                    pBuffer = pSanitized;
+                }
+                else
+                {
+                    lsp_warn("Could not sanitize buffer data for port %s, not enough buffer size (required: %d, actual: %d)",
+                            pMetadata->id, int(samples), int(nBufSize));
+                }
             };
+
+            void set_blk_size(size_t size)
+            {
+                if (!IS_IN_PORT(pMetadata))
+                    return;
+                if (nBufSize == size)
+                    return;
+
+                float *buf  = reinterpret_cast<float *>(::realloc(pSanitized, sizeof(float) * size));
+                if (buf == NULL)
+                {
+                    ::free(pSanitized);
+                    pSanitized = NULL;
+                    return;
+                }
+
+                nBufSize    = size;
+                pSanitized  = buf;
+                dsp::fill_zero(pSanitized, nBufSize);
+            }
     };
 
     class VSTParameterPort: public VSTPort
@@ -267,12 +324,10 @@ namespace lsp
 
             virtual bool pre_process(size_t samples)
             {
-                return fVstValue != fVstPrev;
-            }
-
-            virtual void post_process(size_t samples)
-            {
-                fVstPrev        = fVstValue;
+                if (fVstValue == fVstPrev)
+                    return false;
+                fVstPrev = fVstValue;
+                return true;
             }
 
             virtual void writeValue(float value)
@@ -319,6 +374,7 @@ namespace lsp
                     return -1;
                 float value     = BE_TO_CPU(*(reinterpret_cast<const float *>(data)));
                 writeValue(value);
+                ++nSID;
                 return sizeof(float);
             }
 
@@ -329,6 +385,7 @@ namespace lsp
 
                 float v         = BE_TO_CPU(*(reinterpret_cast<const float *>(data)));
                 writeValue(v);
+                ++nSID;
                 return true;
             }
     };
@@ -476,8 +533,8 @@ namespace lsp
                     const uint8_t *bytes        = reinterpret_cast<const uint8_t *>(vme->midiData);
 
                     // Decode MIDI event
-                    midi_event_t me;
-                    if (!decode_midi_message(&me, bytes))
+                    midi::event_t me;
+                    if (midi::decode(&me, bytes) <= 0)
                         return;
 
                     // Put the event to the queue
@@ -485,7 +542,7 @@ namespace lsp
 
                     // Debug
                     #ifdef LSP_TRACE
-                        #define TRACE_KEY(x)    case MIDI_MSG_ ## x: evt_type = #x; break;
+                        #define TRACE_KEY(x)    case midi::MIDI_MSG_ ## x: evt_type = #x; break;
                         lsp_trace("midi dump: %02x %02x %02x", int(bytes[0]) & 0xff, int(bytes[1]) & 0xff, int(bytes[2]) & 0xff);
 
                         char tmp_evt_type[32];
@@ -574,12 +631,12 @@ namespace lsp
 
                 for (size_t i=0; i<sQueue.nEvents; ++i)
                 {
-                    const midi_event_t *me  = &sQueue.vEvents[i];
-                    VstMidiEvent       *dst = &vEvents[pEvents->numEvents];
+                    const midi::event_t    *me  = &sQueue.vEvents[i];
+                    VstMidiEvent           *dst = &vEvents[pEvents->numEvents];
 
                     // Debug
                     #ifdef LSP_TRACE
-                        #define TRACE_KEY(x)    case MIDI_MSG_ ## x: evt_type = #x; break;
+                        #define TRACE_KEY(x)    case midi::MIDI_MSG_ ## x: evt_type = #x; break;
 
                         char tmp_evt_type[32];
                         const char *evt_type = NULL;
@@ -616,7 +673,7 @@ namespace lsp
 
                     #endif /* LSP_TRACE */
 
-                    size_t bytes = encode_midi_message(me, reinterpret_cast<uint8_t *>(dst->midiData));
+                    ssize_t bytes = midi::encode(reinterpret_cast<uint8_t *>(dst->midiData), me);
                     if (bytes <= 0)
                     {
                         lsp_error("Tried to serialize invalid MIDI event");
@@ -626,6 +683,11 @@ namespace lsp
                     dst->type           = kVstMidiType;
                     dst->byteSize       = sizeof(VstMidiEvent);
                     dst->deltaFrames    = me->timestamp;
+                    dst->flags          = (me->type >= midi::MIDI_MSG_CLOCK) ? kVstMidiEventIsRealtime : 0;
+                    dst->noteLength     = 0;
+                    dst->noteOffset     = 0;
+                    dst->detune         = 0;
+                    dst->noteOffVelocity= (me->type == midi::MIDI_MSG_NOTE_OFF) ? me->note.velocity : 0;
 
                     lsp_trace("midi dump: %02x %02x %02x",
                         int(dst->midiData[0]) & 0xff, int(dst->midiData[1]) & 0xff, int(dst->midiData[2]) & 0xff);
