@@ -28,13 +28,14 @@ namespace lsp
     class VSTWrapper: public IWrapper IF_VST_UI_ON(, public IUIWrapper)
     {
         private:
-            plugin_t                   *pPlugin;
             AEffect                    *pEffect;
             audioMasterCallback         pMaster;
             ipc::IExecutor             *pExecutor;
             vst_chunk_t                 sChunk;
             bool                        bUpdateSettings;
             float                       fLatency;
+            volatile atomic_t           nDumpReq;
+            atomic_t                    nDumpResp;
             VSTPort                    *pBypass;
 
             cvector<VSTAudioPort>       vInputs;        // List of input audio ports
@@ -69,12 +70,13 @@ namespace lsp
                 static status_t slot_ui_resize(LSPWidget *sender, void *ptr, void *data);
             )
 
-            status_t check_vst_header(const fxBank *bank, size_t size);
+            status_t check_vst_bank_header(const fxBank *bank, size_t size);
+            status_t check_vst_program_header(const fxProgram *prog, size_t size);
             void deserialize_v1(const fxBank *bank);
             void deserialize_v2_v3(const uint8_t *data, size_t bytes);
             void deserialize_new_chunk_format(const uint8_t *data, size_t bytes);
-
             void sync_position();
+            status_t serialize_port_data();
 
         public:
             VSTWrapper(
@@ -82,7 +84,7 @@ namespace lsp
                     plugin_t *plugin,
                     const char *name,
                     audioMasterCallback callback
-            )
+            ): IWrapper(plugin)
             {
                 pPlugin         = plugin;
                 pEffect         = effect;
@@ -91,6 +93,8 @@ namespace lsp
                 pExecutor       = NULL;
 
                 fLatency        = 0.0f;
+                nDumpReq        = 0;
+                nDumpResp       = 0;
                 pBypass         = NULL;
                 bUpdateSettings = true;
 
@@ -189,7 +193,7 @@ namespace lsp
 
             virtual ICanvas *create_canvas(ICanvas *&cv, size_t width, size_t height);
 
-            size_t serialize_state(const void **dst);
+            size_t serialize_state(const void **dst, bool program);
             void deserialize_state(const void *data, size_t size);
 
             /**
@@ -209,6 +213,11 @@ namespace lsp
              * @return true on success
              */
             virtual bool kvt_release();
+
+            /**
+             * Request for state dump
+             */
+            virtual void dump_state_request();
     };
 }
 
@@ -597,6 +606,14 @@ namespace lsp
             bUpdateSettings     = false;
         }
 
+        // Need to dump state?
+        atomic_t dump_req   = nDumpReq;
+        if (dump_req != nDumpResp)
+        {
+            dump_plugin_state();
+            nDumpResp           = dump_req;
+        }
+
         // Call the main processing unit
         pPlugin->process(samples);
 
@@ -907,7 +924,7 @@ namespace lsp
                     if ((offset + i) < ck_size)
                     {
                         uint8_t c   = ddump[i];
-                        if ((c < 0x20) || (c >= 0x80))
+                        if ((c < 0x20) || (c >= 0x7f))
                             c           = '.';
                         lsp_nprintf("%c", c);
                     }
@@ -924,35 +941,8 @@ namespace lsp
         #define dump_vst_bank(...)
     #endif /* LSP_TRACE */
 
-    size_t VSTWrapper::serialize_state(const void **dst)
+    status_t VSTWrapper::serialize_port_data()
     {
-        // Clear chunk
-        sChunk.clear();
-
-        // Write the bank header
-        fxBank bank;
-        ::bzero(&bank, sizeof(bank));
-
-        bank.chunkMagic     = CPU_TO_BE(VstInt32(cMagic));
-        bank.byteSize       = 0;
-        bank.fxMagic        = CPU_TO_BE(VstInt32(chunkBankMagic));
-        bank.version        = CPU_TO_BE(VstInt32(1));
-        bank.fxID           = CPU_TO_BE(VstInt32(pEffect->uniqueID));
-        bank.fxVersion      = CPU_TO_BE(VstInt32(2000)); // Version 2.0.0 of the bank
-        bank.numPrograms    = 0;
-        bank.currentProgram = 0;
-
-        size_t bank_off     = sChunk.write(&bank, offsetof(fxBank, content.data.chunk));
-
-        vst_state_header hdr;
-        ::bzero(&hdr, sizeof(hdr));
-        hdr.nMagic1         = CPU_TO_BE(VstInt32(LSP_VST_USER_MAGIC));
-        hdr.nSize           = 0;
-        hdr.nVersion        = CPU_TO_BE(VstInt32(VST_FX_VERSION_JUCE_FIX));
-        hdr.nMagic2         = CPU_TO_BE(VstInt32(LSP_VST_USER_MAGIC));
-        size_t hdr_off      = sChunk.write(&hdr, sizeof(hdr));
-
-        size_t data_off     = sChunk.offset;
         size_t param_off    = 0;
 
         // Serialize all regular ports
@@ -980,8 +970,7 @@ namespace lsp
             if (sChunk.res != STATUS_OK)
             {
                 lsp_warn("Error serializing parameter is=%s, code=%d", p->id, int(sChunk.res));
-                *dst = NULL;
-                return 0;
+                return sChunk.res;
             }
         }
 
@@ -1104,33 +1093,112 @@ namespace lsp
             sKVTMutex.unlock();
         }
 
-        if (res != STATUS_OK)
+        return res;
+    }
+
+    size_t VSTWrapper::serialize_state(const void **dst, bool program)
+    {
+        // Clear chunk
+        status_t res;
+        sChunk.clear();
+
+        // Write the bank header
+        if (program)
         {
-            *dst    = NULL;
-            return 0;
+            fxProgram prog;
+            ::bzero(&prog, sizeof(prog));
+
+            prog.chunkMagic     = CPU_TO_BE(VstInt32(cMagic));
+            prog.byteSize       = 0;
+            prog.fxMagic        = CPU_TO_BE(VstInt32(chunkPresetMagic));
+            prog.version        = CPU_TO_BE(VstInt32(1));    // Version 1.0.0 of the program
+            prog.fxID           = CPU_TO_BE(VstInt32(pEffect->uniqueID));
+            prog.fxVersion      = CPU_TO_BE(VstInt32(VST_FX_VERSION_JUCE_FIX));
+            prog.numParams      = 0;
+            prog.prgName[0]     = '\0';
+
+            size_t prog_off     = sChunk.write(&prog, offsetof(fxProgram, content.data.chunk));
+
+            vst_state_header hdr;
+            ::bzero(&hdr, sizeof(hdr));
+            hdr.nMagic1         = CPU_TO_BE(VstInt32(LSP_VST_USER_MAGIC));
+            hdr.nSize           = 0;
+            hdr.nVersion        = CPU_TO_BE(VstInt32(VST_FX_VERSION_JUCE_FIX));
+            hdr.nMagic2         = CPU_TO_BE(VstInt32(LSP_VST_USER_MAGIC));
+            size_t hdr_off      = sChunk.write(&hdr, sizeof(hdr));
+            size_t data_off     = sChunk.offset;
+
+            if ((res = serialize_port_data()) != STATUS_OK)
+            {
+                *dst    = NULL;
+                return 0;
+            }
+
+            // Write the size of chunk
+            fxProgram *pprog            = sChunk.fetch<fxProgram>(prog_off);
+            VstInt32 size               = sChunk.offset - VST_PROGRAM_HDR_SKIP;
+            pprog->content.data.size    = CPU_TO_BE(VstInt32(sChunk.offset - hdr_off));
+            pprog->byteSize             = CPU_TO_BE(size);
+
+            vst_state_header *phdr      = sChunk.fetch<vst_state_header>(hdr_off);
+            phdr->nSize                 = CPU_TO_BE(VstInt32(sChunk.offset - data_off));
+
+            dump_vst_bank(pprog, sChunk.offset);
+            *dst                = pprog;
+        }
+        else
+        {
+            fxBank bank;
+            ::bzero(&bank, sizeof(bank));
+
+            bank.chunkMagic     = CPU_TO_BE(VstInt32(cMagic));
+            bank.byteSize       = 0;
+            bank.fxMagic        = CPU_TO_BE(VstInt32(chunkBankMagic));
+            bank.version        = CPU_TO_BE(VstInt32(1)); // Version 2.0.0 of the bank
+            bank.fxID           = CPU_TO_BE(VstInt32(pEffect->uniqueID));
+            bank.fxVersion      = CPU_TO_BE(VstInt32(VST_FX_VERSION_JUCE_FIX)); // Version 2.0.0 of the bank
+            bank.numPrograms    = 0;
+            bank.currentProgram = 0;
+
+            size_t bank_off     = sChunk.write(&bank, offsetof(fxBank, content.data.chunk));
+
+            vst_state_header hdr;
+            ::bzero(&hdr, sizeof(hdr));
+            hdr.nMagic1         = CPU_TO_BE(VstInt32(LSP_VST_USER_MAGIC));
+            hdr.nSize           = 0;
+            hdr.nVersion        = CPU_TO_BE(VstInt32(VST_FX_VERSION_JUCE_FIX));
+            hdr.nMagic2         = CPU_TO_BE(VstInt32(LSP_VST_USER_MAGIC));
+            size_t hdr_off      = sChunk.write(&hdr, sizeof(hdr));
+            size_t data_off     = sChunk.offset;
+
+            if ((res = serialize_port_data()) != STATUS_OK)
+            {
+                *dst    = NULL;
+                return 0;
+            }
+
+            // Write the size of chunk
+            fxBank *pbank               = sChunk.fetch<fxBank>(bank_off);
+            VstInt32 size               = sChunk.offset - VST_BANK_HDR_SKIP;
+            pbank->content.data.size    = CPU_TO_BE(VstInt32(sChunk.offset - hdr_off));
+            pbank->byteSize             = CPU_TO_BE(size);
+
+            vst_state_header *phdr      = sChunk.fetch<vst_state_header>(hdr_off);
+            phdr->nSize                 = CPU_TO_BE(VstInt32(sChunk.offset - data_off));
+
+            dump_vst_bank(pbank, sChunk.offset);
+            *dst                = pbank;
         }
 
         // Issue callback
         pPlugin->state_saved();
-
-        // Write the size of chunk
-        fxBank *pbank               = sChunk.fetch<fxBank>(bank_off);
-        VstInt32 size               = sChunk.offset - VST_BANK_HDR_SKIP;
-        pbank->content.data.size    = CPU_TO_BE(VstInt32(sChunk.offset - hdr_off));
-        pbank->byteSize             = CPU_TO_BE(size);
-
-        vst_state_header *phdr      = sChunk.fetch<vst_state_header>(hdr_off);
-        phdr->nSize                 = CPU_TO_BE(VstInt32(sChunk.offset - data_off));
-
-        dump_vst_bank(pbank, sChunk.offset);
         lsp_trace("Plugin state has been saved");
 
         // Return result
-        *dst = pbank;
         return sChunk.offset;
     }
 
-    status_t VSTWrapper::check_vst_header(const fxBank *bank, size_t size)
+    status_t VSTWrapper::check_vst_bank_header(const fxBank *bank, size_t size)
     {
         // Validate size
         if (size < size_t(offsetof(fxBank, content.data.chunk)))
@@ -1160,14 +1228,6 @@ namespace lsp
             return STATUS_UNSUPPORTED_FORMAT;
         }
 
-        // Validate the version
-        VstInt32 version  = BE_TO_CPU(bank->version);
-        if (version > pEffect->version)
-        {
-            lsp_warn("Unsupported effect version (%d)", version);
-            return STATUS_UNSUPPORTED_FORMAT;
-        }
-
         // Validate the numParams
         if (bank->numPrograms != 0)
         {
@@ -1178,21 +1238,54 @@ namespace lsp
         return STATUS_OK;
     }
 
+    status_t VSTWrapper::check_vst_program_header(const fxProgram *prog, size_t size)
+    {
+        // Validate size
+        if (size < size_t(offsetof(fxProgram, content.data.chunk)))
+        {
+            lsp_warn("block size too small (0x%08x bytes)", int(size));
+            return STATUS_NOT_FOUND;
+        }
+
+        // Validate chunkMagic
+        if (prog->chunkMagic != BE_TO_CPU(cMagic))
+        {
+            lsp_warn("prog->chunkMagic (%08x) != BE_DATA(VST_CHUNK_MAGIC) (%08x)", int(prog->chunkMagic), int(BE_TO_CPU(cMagic)));
+            return STATUS_NOT_FOUND;
+        }
+
+        // Validate fxMagic
+        if (prog->fxMagic != BE_TO_CPU(chunkPresetMagic))
+        {
+            lsp_warn("prog->fxMagic (%08x) != BE_DATA(VST_OPAQUE_PRESET_MAGIC) (%08x)", int(prog->fxMagic), int(BE_TO_CPU(chunkPresetMagic)));
+            return STATUS_UNSUPPORTED_FORMAT;
+        }
+
+        // Validate fxID
+        if (prog->fxID != BE_TO_CPU(VstInt32(pEffect->uniqueID)))
+        {
+            lsp_warn("prog->fxID (%08x) != BE_DATA(VstInt32(pEffect->uniqueID)) (%08x)", int(prog->fxID), int(BE_TO_CPU(VstInt32(pEffect->uniqueID))));
+            return STATUS_UNSUPPORTED_FORMAT;
+        }
+
+        return STATUS_OK;
+    }
+
     void VSTWrapper::deserialize_state(const void *data, size_t size)
     {
-        const fxBank *bank          = reinterpret_cast<const fxBank *>(data);
-        const uint8_t *head         = reinterpret_cast<const uint8_t *>(data);
+        const fxBank *bank          = static_cast<const fxBank *>(data);
+        const fxProgram *prog       = static_cast<const fxProgram *>(data);
+        const uint8_t *head         = static_cast<const uint8_t *>(data);
 
-        status_t res                = check_vst_header(bank, size);
-
-        if (res == STATUS_OK)
+        status_t res;
+        if ((res = check_vst_bank_header(bank, size)) == STATUS_OK)
         {
-            lsp_warn("Found standard VST 2.x chunk header");
+            lsp_warn("Found standard VST 2.x chunk header (bank)");
             dump_vst_bank(bank, (BE_TO_CPU(bank->byteSize) + 2 * sizeof(VstInt32)));
 
             // Check the version
             VstInt32 fxVersion = BE_TO_CPU(bank->fxVersion);
-            if (fxVersion != VST_FX_VERSION_KVT_SUPPORT)
+            if (fxVersion < VST_FX_VERSION_KVT_SUPPORT)
                 deserialize_v1(bank);       // Load V1 bank for legacy support
             else
             {
@@ -1216,6 +1309,31 @@ namespace lsp
                 deserialize_new_chunk_format(head, bytes);
             }
         }
+        else if ((res = check_vst_program_header(prog, size)) == STATUS_OK)
+        {
+            // VST Programs do support only new chunk format
+            lsp_warn("Found standard VST 2.x chunk header (program)");
+            dump_vst_bank(prog, (BE_TO_CPU(prog->byteSize) + 2 * sizeof(VstInt32)));
+
+            // Get size of chunk
+            size_t bytes                    = BE_TO_CPU(VstInt32(prog->byteSize));
+            if (bytes < offsetof(fxProgram, content.data.chunk))
+            {
+                lsp_trace("byte_size (%d) < VST_STATE_BUFFER_SIZE (%d)", int(bytes), int(VST_STATE_BUFFER_SIZE));
+                return;
+            }
+
+            const uint8_t *tail = &head[bytes + VST_PROGRAM_HDR_SKIP];
+            head               += offsetof(fxProgram, content.data.chunk);
+            bytes               = BE_TO_CPU(prog->content.data.size);
+            if (size_t(tail - head) != bytes)
+            {
+                lsp_trace("Content size=0x%x does not match specified=0x%x", int(tail-head), int(bytes));
+                return;
+            }
+
+            deserialize_new_chunk_format(head, bytes);
+        }
         else if (res == STATUS_NOT_FOUND)
         {
             // Do stuff considering that there is NO chunk headers, just raw data
@@ -1546,6 +1664,11 @@ namespace lsp
     {
         return sKVTMutex.unlock();
     }
+
+    void VSTWrapper::dump_state_request()
+    {
+        atomic_add(&nDumpReq, 1);
+    }
 }
 
 #endif /* CONTAINER_VST_WRAPPER_H_ */
