@@ -97,6 +97,10 @@ namespace lsp
                 status_t    append_child(const LSPString *path);
                 status_t    append_child(const Path *path);
 
+                status_t    append(const char *path);
+                status_t    append(const LSPString *path);
+                status_t    append(const Path *path);
+
                 status_t    remove_last();
                 status_t    remove_last(char *path, size_t maxlen);
                 status_t    remove_last(LSPString *path);
@@ -106,6 +110,11 @@ namespace lsp
                 status_t    remove_base(const LSPString *path);
                 status_t    remove_base(const Path *path);
 
+                ssize_t     fmt(const char *fmt...);
+                ssize_t     fmt(const LSPString *fmt...);
+                ssize_t     vfmt(const char *fmt, va_list args);
+                ssize_t     vfmt(const LSPString *fmt, va_list args);
+
                 bool        is_absolute() const;
                 bool        is_relative() const;
                 bool        is_canonical() const;
@@ -136,7 +145,8 @@ namespace lsp
                 bool        equals(const LSPString *path) const;
                 bool        equals(const char *path) const;
 
-                inline const LSPString *as_string() const { return &sPath; }
+                inline const LSPString *as_string() const           { return &sPath; }
+                inline const char *as_utf8() const                  { return sPath.get_utf8(); }
                 inline const char *as_native(const char *charset = NULL) const { return sPath.get_native(charset); }
                 inline void take(Path *src) { sPath.take(&src->sPath); }
                 void take(LSPString *src);
