@@ -10,6 +10,7 @@
 
 #include <core/types.h>
 #include <math.h>
+#include <core/IStateDumper.h>
 #include <core/util/Oversampler.h>
 
 namespace lsp
@@ -465,6 +466,12 @@ namespace lsp
              */
             void process_overwrite(float *dst, size_t count);
 
+            /**
+             * Dump the state
+             * @param dumper dumper
+             */
+            void dump(IStateDumper *v) const;
+
         protected:
 
             /** Synthesize the required wave and write its sample to internal
