@@ -20,7 +20,7 @@ namespace asimd
 #define OP_K2_CORE(DST, OP, ORDER) \
     __ASM_EMIT("subs        %[count], %[count], #32") \
     __ASM_EMIT("mov         v17.16b, v16.16b") \
-    __ASM_EMIT("blo         2f") \
+    __ASM_EMIT("b.lo        2f") \
     /* 32x blocks */ \
     __ASM_EMIT("1:") \
     __ASM_EMIT("ldp         q0, q1, [%[" DST "], #0x00]") \
@@ -41,11 +41,11 @@ namespace asimd
     __ASM_EMIT("stp         q6, q7, [%[" DST "], #0x60]") \
     __ASM_EMIT("subs        %[count], %[count], #32") \
     __ASM_EMIT("add         %[dst], %[dst], #0x80") \
-    __ASM_EMIT("bhs         1b") \
+    __ASM_EMIT("b.hs        1b") \
     /* 16x block */ \
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds        %[count], %[count], #16") /* 32 - 16 */ \
-    __ASM_EMIT("blt         4f") \
+    __ASM_EMIT("b.lt        4f") \
     __ASM_EMIT("ldp         q0, q1, [%[" DST "], #0x00]") \
     __ASM_EMIT("ldp         q2, q3, [%[" DST "], #0x20]") \
     __ASM_EMIT(OP "         v0.4s, " ORDER("v0.4s", "v16.4s")) \
@@ -59,7 +59,7 @@ namespace asimd
     /* 8x block */ \
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds        %[count], %[count], #8") /* 16 - 8 */ \
-    __ASM_EMIT("blt         6f") \
+    __ASM_EMIT("b.lt        6f") \
     __ASM_EMIT("ldp         q0, q1, [%[" DST "], #0x00]") \
     __ASM_EMIT(OP "         v0.4s, " ORDER("v0.4s", "v16.4s")) \
     __ASM_EMIT(OP "         v1.4s, " ORDER("v1.4s", "v17.4s")) \
@@ -69,7 +69,7 @@ namespace asimd
     /* 4x block */ \
     __ASM_EMIT("6:") \
     __ASM_EMIT("adds        %[count], %[count], #4") /* 8 - 4 */ \
-    __ASM_EMIT("blt         8f") \
+    __ASM_EMIT("b.lt        8f") \
     __ASM_EMIT("ldr         q0, [%[" DST "], #0x00]") \
     __ASM_EMIT(OP "         v0.4s, " ORDER("v0.4s", "v16.4s")) \
     __ASM_EMIT("str         q0, [%[" DST "], #0x00]") \
@@ -78,7 +78,7 @@ namespace asimd
     /* 1x block */ \
     __ASM_EMIT("8:") \
     __ASM_EMIT("adds        %[count], %[count], #3") /* 4 - 3 */ \
-    __ASM_EMIT("blt         10f") \
+    __ASM_EMIT("b.lt        10f") \
     __ASM_EMIT("9:") \
     __ASM_EMIT("ld1r        {v0.4s}, [%[" DST "]]") \
     __ASM_EMIT(OP "         v0.4s, " ORDER("v0.4s", "v16.4s")) \
@@ -91,7 +91,7 @@ namespace asimd
 #define OP_K3_CORE(DST, SRC, OP, ORDER) \
     __ASM_EMIT("subs        %[count], %[count], #32") \
     __ASM_EMIT("mov         v17.16b, v16.16b") \
-    __ASM_EMIT("blo         2f") \
+    __ASM_EMIT("b.lo        2f") \
     /* 32x blocks */ \
     __ASM_EMIT("1:") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC "], #0x00]") \
@@ -113,11 +113,11 @@ namespace asimd
     __ASM_EMIT("subs        %[count], %[count], #32") \
     __ASM_EMIT("add         %[src], %[src], #0x80") \
     __ASM_EMIT("add         %[dst], %[dst], #0x80") \
-    __ASM_EMIT("bhs         1b") \
+    __ASM_EMIT("b.hs        1b") \
     /* 16x block */ \
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds        %[count], %[count], #16") /* 32 - 16 */ \
-    __ASM_EMIT("blt         4f") \
+    __ASM_EMIT("b.lt        4f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC "], #0x00]") \
     __ASM_EMIT("ldp         q2, q3, [%[" SRC "], #0x20]") \
     __ASM_EMIT(OP "         v0.4s, " ORDER("v0.4s", "v16.4s")) \
@@ -132,7 +132,7 @@ namespace asimd
     /* 8x block */ \
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds        %[count], %[count], #8") /* 16 - 8 */ \
-    __ASM_EMIT("blt         6f") \
+    __ASM_EMIT("b.lt        6f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC "], #0x00]") \
     __ASM_EMIT(OP "         v0.4s, " ORDER("v0.4s", "v16.4s")) \
     __ASM_EMIT(OP "         v1.4s, " ORDER("v1.4s", "v17.4s")) \
@@ -143,7 +143,7 @@ namespace asimd
     /* 4x block */ \
     __ASM_EMIT("6:") \
     __ASM_EMIT("adds        %[count], %[count], #4") /* 8 - 4 */ \
-    __ASM_EMIT("blt         8f") \
+    __ASM_EMIT("b.lt        8f") \
     __ASM_EMIT("ldr         q0, [%[" SRC "], #0x00]") \
     __ASM_EMIT(OP "         v0.4s, " ORDER("v0.4s", "v16.4s")) \
     __ASM_EMIT("str         q0, [%[" DST "], #0x00]") \
@@ -153,7 +153,7 @@ namespace asimd
     /* 1x block */ \
     __ASM_EMIT("8:") \
     __ASM_EMIT("adds        %[count], %[count], #3") /* 4 - 3 */ \
-    __ASM_EMIT("blt         10f") \
+    __ASM_EMIT("b.lt        10f") \
     __ASM_EMIT("9:") \
     __ASM_EMIT("ld1r        {v0.4s}, [%[" SRC "]]") \
     __ASM_EMIT(OP "         v0.4s, " ORDER("v0.4s", "v16.4s")) \
@@ -167,7 +167,7 @@ namespace asimd
 #define OP_DIV2_CORE(DST) \
     __ASM_EMIT("subs        %[count], %[count], #32") \
     __ASM_EMIT("mov         v9.16b, v8.16b") \
-    __ASM_EMIT("blo         2f") \
+    __ASM_EMIT("b.lo        2f") \
     /* 32x blocks */ \
     __ASM_EMIT("1:") \
     __ASM_EMIT("ldp         q0, q1, [%[" DST "], #0x00]") \
@@ -228,11 +228,11 @@ namespace asimd
     __ASM_EMIT("stp         q6, q7, [%[" DST "], #0x60]") \
     __ASM_EMIT("subs        %[count], %[count], #32") \
     __ASM_EMIT("add         %[dst], %[dst], #0x80") \
-    __ASM_EMIT("bhs         1b") \
+    __ASM_EMIT("b.hs        1b") \
     /* 16x block */ \
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds        %[count], %[count], #16") /* 32 - 16 */ \
-    __ASM_EMIT("blt         4f") \
+    __ASM_EMIT("b.lt        4f") \
     __ASM_EMIT("ldp         q0, q1, [%[" DST "], #0x00]") \
     __ASM_EMIT("ldp         q2, q3, [%[" DST "], #0x20]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
@@ -266,7 +266,7 @@ namespace asimd
     /* 8x block */ \
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds        %[count], %[count], #8") /* 16 - 8 */ \
-    __ASM_EMIT("blt         6f") \
+    __ASM_EMIT("b.lt        6f") \
     __ASM_EMIT("ldp         q0, q1, [%[" DST "], #0x00]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
     __ASM_EMIT("frecpe      v17.4s, v1.4s") \
@@ -286,7 +286,7 @@ namespace asimd
     /* 4x block */ \
     __ASM_EMIT("6:") \
     __ASM_EMIT("adds        %[count], %[count], #4") /* 8 - 4 */ \
-    __ASM_EMIT("blt         8f") \
+    __ASM_EMIT("b.lt        8f") \
     __ASM_EMIT("ldr         q0, [%[" DST "], #0x00]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
     __ASM_EMIT("frecps      v24.4s, v16.4s, v0.4s")             /* v24 = (2 - R*s2) */ \
@@ -300,7 +300,7 @@ namespace asimd
     /* 1x block */ \
     __ASM_EMIT("8:") \
     __ASM_EMIT("adds        %[count], %[count], #3") /* 4 - 3 */ \
-    __ASM_EMIT("blt         10f") \
+    __ASM_EMIT("b.lt        10f") \
     __ASM_EMIT("9:") \
     __ASM_EMIT("ld1r        {v0.4s}, [%[" DST "]]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
@@ -318,7 +318,7 @@ namespace asimd
 #define OP_DIV3_CORE(DST, SRC) \
     __ASM_EMIT("subs        %[count], %[count], #32") \
     __ASM_EMIT("mov         v9.16b, v8.16b") \
-    __ASM_EMIT("blo         2f") \
+    __ASM_EMIT("b.lo        2f") \
     /* 32x blocks */ \
     __ASM_EMIT("1:") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC "], #0x00]") \
@@ -380,11 +380,11 @@ namespace asimd
     __ASM_EMIT("subs        %[count], %[count], #32") \
     __ASM_EMIT("add         %[src], %[src], #0x80") \
     __ASM_EMIT("add         %[dst], %[dst], #0x80") \
-    __ASM_EMIT("bhs         1b") \
+    __ASM_EMIT("b.hs        1b") \
     /* 16x block */ \
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds        %[count], %[count], #16") /* 32 - 16 */ \
-    __ASM_EMIT("blt         4f") \
+    __ASM_EMIT("b.lt        4f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC "], #0x00]") \
     __ASM_EMIT("ldp         q2, q3, [%[" SRC "], #0x20]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
@@ -419,7 +419,7 @@ namespace asimd
     /* 8x block */ \
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds        %[count], %[count], #8") /* 16 - 8 */ \
-    __ASM_EMIT("blt         6f") \
+    __ASM_EMIT("b.lt        6f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC "], #0x00]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
     __ASM_EMIT("frecpe      v17.4s, v1.4s") \
@@ -440,7 +440,7 @@ namespace asimd
     /* 4x block */ \
     __ASM_EMIT("6:") \
     __ASM_EMIT("adds        %[count], %[count], #4") /* 8 - 4 */ \
-    __ASM_EMIT("blt         8f") \
+    __ASM_EMIT("b.lt        8f") \
     __ASM_EMIT("ldr         q0, [%[" SRC "], #0x00]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
     __ASM_EMIT("frecps      v24.4s, v16.4s, v0.4s")             /* v24 = (2 - R*s2) */ \
@@ -455,7 +455,7 @@ namespace asimd
     /* 1x block */ \
     __ASM_EMIT("8:") \
     __ASM_EMIT("adds        %[count], %[count], #3") /* 4 - 3 */ \
-    __ASM_EMIT("blt         10f") \
+    __ASM_EMIT("b.lt        10f") \
     __ASM_EMIT("9:") \
     __ASM_EMIT("ld1r        {v0.4s}, [%[" SRC "]]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
@@ -677,12 +677,276 @@ namespace asimd
         );
     }
 
+#define INC_ON(x)           x
+#define INC_OFF(x)
+
+#define MOD_KX_CORE(DST, SRC, INC)   \
+    __ASM_EMIT("subs            %[count], %[count], #16") \
+    __ASM_EMIT("frecpe          v16.4s, v4.4s") \
+    __ASM_EMIT("mov             v5.16b, v4.16b")                    /* v5   = k */ \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v4.4s") \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s") \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v4.4s") \
+    __ASM_EMIT("fmul            v6.4s, v20.4s, v16.4s")             /* v6   = 1/k */ \
+    __ASM_EMIT("mov             v7.16b, v6.16b")                    /* v7   = 1/k */ \
+    __ASM_EMIT("b.lo            2f") \
+    /* 16x blocks */ \
+    __ASM_EMIT("1:") \
+    __ASM_EMIT("ldp             q0, q1, [%[" SRC "], #0x00]") \
+    __ASM_EMIT("ldp             q2, q3, [%[" SRC "], #0x20]") \
+    __ASM_EMIT("fmul            v16.4s, v0.4s, v6.4s")              /* v0   = x/k */  \
+    __ASM_EMIT("fmul            v17.4s, v1.4s, v7.4s") \
+    __ASM_EMIT("fmul            v18.4s, v2.4s, v6.4s") \
+    __ASM_EMIT("fmul            v19.4s, v3.4s, v7.4s") \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("frintz          v17.4s, v17.4s") \
+    __ASM_EMIT("frintz          v18.4s, v18.4s") \
+    __ASM_EMIT("frintz          v19.4s, v19.4s") \
+    __ASM_EMIT("fmls            v0.4s, v16.4s, v4.4s") \
+    __ASM_EMIT("fmls            v1.4s, v17.4s, v5.4s") \
+    __ASM_EMIT("fmls            v2.4s, v18.4s, v4.4s") \
+    __ASM_EMIT("fmls            v3.4s, v19.4s, v5.4s") \
+    __ASM_EMIT("stp             q0, q1, [%[" DST "], #0x00]") \
+    __ASM_EMIT("stp             q2, q3, [%[" DST "], #0x20]") \
+    __ASM_EMIT("subs            %[count], %[count], #16") \
+    __ASM_EMIT(INC("add         %[" SRC "], %[" SRC "], #0x40")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x40") \
+    __ASM_EMIT("b.hs            1b") \
+    /* 8x block */ \
+    __ASM_EMIT("2:") \
+    __ASM_EMIT("adds            %[count], %[count], #8") \
+    __ASM_EMIT("b.lt            4f") \
+    __ASM_EMIT("ldp             q0, q1, [%[" SRC "], #0x00]") \
+    __ASM_EMIT("fmul            v16.4s, v0.4s, v6.4s")              /* v0   = x/k */  \
+    __ASM_EMIT("fmul            v17.4s, v1.4s, v7.4s") \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("frintz          v17.4s, v17.4s") \
+    __ASM_EMIT("fmls            v0.4s, v16.4s, v4.4s") \
+    __ASM_EMIT("fmls            v1.4s, v17.4s, v5.4s") \
+    __ASM_EMIT("stp             q0, q1, [%[" DST "], #0x00]") \
+    __ASM_EMIT("sub             %[count], %[count], #8") \
+    __ASM_EMIT(INC("add         %[" SRC "], %[" SRC "], #0x20")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x20") \
+    /* 4x blocks */ \
+    __ASM_EMIT("4:") \
+    __ASM_EMIT("adds            %[count], %[count], #4") \
+    __ASM_EMIT("b.lt            6f") \
+    __ASM_EMIT("ldr             q0, [%[" SRC "]]") \
+    __ASM_EMIT("fmul            v16.4s, v0.4s, v6.4s")              /* v0   = x/k */  \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("fmls            v0.4s, v16.4s, v4.4s") \
+    __ASM_EMIT("str             q0, [%[" DST "]]") \
+    __ASM_EMIT("sub             %[count], %[count], #4") \
+    __ASM_EMIT(INC("add         %[" SRC "], %[" SRC "], #0x10")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x10") \
+    /* 1x blocks */ \
+    __ASM_EMIT("6:") \
+    __ASM_EMIT("adds            %[count], %[count], #3") \
+    __ASM_EMIT("b.lt            8f") \
+    __ASM_EMIT("7:") \
+    __ASM_EMIT("ld1r            {v0.4s}, [%[" SRC "]]") \
+    __ASM_EMIT("fmul            v16.4s, v0.4s, v6.4s")              /* v0   = x/k */  \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("fmls            v0.4s, v16.4s, v4.4s") \
+    __ASM_EMIT("st1             {v0.s}[0], [%[" DST "]]") \
+    __ASM_EMIT("subs            %[count], %[count], #1") \
+    __ASM_EMIT(INC("add         %[" SRC "], %[" SRC "], #0x04")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x04") \
+    __ASM_EMIT("bge             7b") \
+    __ASM_EMIT("8:")
+
+#define RMOD_KX_CORE(DST, SRC, INC)   \
+    __ASM_EMIT("subs            %[count], %[count], #16") \
+    __ASM_EMIT("mov             v5.16b, v4.16b")                    /* v5   = k */ \
+    __ASM_EMIT("b.lo            2f") \
+    /* 16x blocks */ \
+    __ASM_EMIT("1:") \
+    __ASM_EMIT("ldp             q0, q1, [%[" SRC "], #0x00]") \
+    __ASM_EMIT("ldp             q2, q3, [%[" SRC "], #0x20]") \
+    __ASM_EMIT("frecpe          v16.4s, v0.4s")                     /* v16  = s2 */ \
+    __ASM_EMIT("frecpe          v17.4s, v1.4s") \
+    __ASM_EMIT("frecpe          v18.4s, v2.4s") \
+    __ASM_EMIT("frecpe          v19.4s, v3.4s") \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v0.4s")             /* v20  = (2 - R*s2) */ \
+    __ASM_EMIT("frecps          v21.4s, v17.4s, v1.4s") \
+    __ASM_EMIT("frecps          v22.4s, v18.4s, v2.4s") \
+    __ASM_EMIT("frecps          v23.4s, v19.4s, v3.4s") \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")            /* v16  = s2' = s2 * (2 - R*s2) */ \
+    __ASM_EMIT("fmul            v17.4s, v21.4s, v17.4s") \
+    __ASM_EMIT("fmul            v18.4s, v22.4s, v18.4s") \
+    __ASM_EMIT("fmul            v19.4s, v23.4s, v19.4s") \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v0.4s")             /* v20  = (2 - R*s2') */ \
+    __ASM_EMIT("frecps          v21.4s, v17.4s, v1.4s") \
+    __ASM_EMIT("frecps          v22.4s, v18.4s, v2.4s") \
+    __ASM_EMIT("frecps          v23.4s, v19.4s, v3.4s") \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")            /* v16  = s2" = s2' * (2 - R*s2) = 1/x */  \
+    __ASM_EMIT("fmul            v17.4s, v21.4s, v17.4s") \
+    __ASM_EMIT("fmul            v18.4s, v22.4s, v18.4s") \
+    __ASM_EMIT("fmul            v19.4s, v23.4s, v19.4s") \
+    __ASM_EMIT("fmul            v16.4s, v16.4s, v4.4s")             /* v16  = k/x */ \
+    __ASM_EMIT("fmul            v17.4s, v17.4s, v5.4s") \
+    __ASM_EMIT("fmul            v18.4s, v18.4s, v4.4s") \
+    __ASM_EMIT("fmul            v19.4s, v19.4s, v5.4s") \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("frintz          v17.4s, v17.4s") \
+    __ASM_EMIT("frintz          v18.4s, v18.4s") \
+    __ASM_EMIT("frintz          v19.4s, v19.4s") \
+    __ASM_EMIT("fmul            v0.4s, v0.4s, v16.4s")              /* v0   = x * int(k/x) */ \
+    __ASM_EMIT("fmul            v1.4s, v1.4s, v17.4s") \
+    __ASM_EMIT("fmul            v2.4s, v2.4s, v18.4s") \
+    __ASM_EMIT("fmul            v3.4s, v3.4s, v19.4s") \
+    __ASM_EMIT("fsub            v0.4s, v4.4s, v0.4s")               /* v0   = k - x * int(k/x) */ \
+    __ASM_EMIT("fsub            v1.4s, v5.4s, v1.4s") \
+    __ASM_EMIT("fsub            v2.4s, v4.4s, v2.4s") \
+    __ASM_EMIT("fsub            v3.4s, v5.4s, v3.4s") \
+    __ASM_EMIT("stp             q0, q1, [%[" DST "], #0x00]") \
+    __ASM_EMIT("stp             q2, q3, [%[" DST "], #0x20]") \
+    __ASM_EMIT("subs            %[count], %[count], #16") \
+    __ASM_EMIT(INC("add         %[" SRC "], %[" SRC "], #0x40")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x40") \
+    __ASM_EMIT("b.hs            1b") \
+    /* 8x block */ \
+    __ASM_EMIT("2:") \
+    __ASM_EMIT("adds            %[count], %[count], #8") \
+    __ASM_EMIT("b.lt            4f") \
+    __ASM_EMIT("ldp             q0, q1, [%[" SRC "], #0x00]") \
+    __ASM_EMIT("frecpe          v16.4s, v0.4s")                     /* v16  = s2 */ \
+    __ASM_EMIT("frecpe          v17.4s, v1.4s") \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v0.4s")             /* v20  = (2 - R*s2) */ \
+    __ASM_EMIT("frecps          v21.4s, v17.4s, v1.4s") \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")            /* v16  = s2' = s2 * (2 - R*s2) */ \
+    __ASM_EMIT("fmul            v17.4s, v21.4s, v17.4s") \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v0.4s")             /* v20  = (2 - R*s2') */ \
+    __ASM_EMIT("frecps          v21.4s, v17.4s, v1.4s") \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")            /* v16  = s2" = s2' * (2 - R*s2) = 1/x */  \
+    __ASM_EMIT("fmul            v17.4s, v21.4s, v17.4s") \
+    __ASM_EMIT("fmul            v16.4s, v16.4s, v4.4s")             /* v16  = k/x */ \
+    __ASM_EMIT("fmul            v17.4s, v17.4s, v5.4s") \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("frintz          v17.4s, v17.4s") \
+    __ASM_EMIT("fmul            v0.4s, v0.4s, v16.4s")              /* v0   = x * int(k/x) */ \
+    __ASM_EMIT("fmul            v1.4s, v1.4s, v17.4s") \
+    __ASM_EMIT("fsub            v0.4s, v4.4s, v0.4s")               /* v0   = k - x * int(k/x) */ \
+    __ASM_EMIT("fsub            v1.4s, v5.4s, v1.4s") \
+    __ASM_EMIT("stp             q0, q1, [%[" DST "], #0x00]") \
+    __ASM_EMIT("sub             %[count], %[count], #8") \
+    __ASM_EMIT(INC("add         %[" SRC "], %[" SRC "], #0x20")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x20") \
+    /* 4x blocks */ \
+    __ASM_EMIT("4:") \
+    __ASM_EMIT("adds            %[count], %[count], #4") \
+    __ASM_EMIT("b.lt            6f") \
+    __ASM_EMIT("ldr             q0, [%[" SRC "]]") \
+    __ASM_EMIT("frecpe          v16.4s, v0.4s")                     /* v16  = s2 */ \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v0.4s")             /* v20  = (2 - R*s2) */ \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")            /* v16  = s2' = s2 * (2 - R*s2) */ \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v0.4s")             /* v20  = (2 - R*s2') */ \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")            /* v16  = s2" = s2' * (2 - R*s2) = 1/x */  \
+    __ASM_EMIT("fmul            v16.4s, v16.4s, v4.4s")             /* v16  = k/x */ \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("fmul            v0.4s, v0.4s, v16.4s")              /* v0   = x * int(k/x) */ \
+    __ASM_EMIT("fsub            v0.4s, v4.4s, v0.4s")               /* v0   = k - x * int(k/x) */ \
+    __ASM_EMIT("str             q0, [%[" DST "]]") \
+    __ASM_EMIT("sub             %[count], %[count], #4") \
+    __ASM_EMIT(INC("add         %[" SRC "], %[" SRC "], #0x10")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x10") \
+    /* 1x blocks */ \
+    __ASM_EMIT("6:") \
+    __ASM_EMIT("adds            %[count], %[count], #3") \
+    __ASM_EMIT("b.lt            8f") \
+    __ASM_EMIT("7:") \
+    __ASM_EMIT("ld1r            {v0.4s}, [%[" SRC "]]") \
+    __ASM_EMIT("frecpe          v16.4s, v0.4s")                     /* v16  = s2 */ \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v0.4s")             /* v20  = (2 - R*s2) */ \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")            /* v16  = s2' = s2 * (2 - R*s2) */ \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, v0.4s")             /* v20  = (2 - R*s2') */ \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")            /* v16  = s2" = s2' * (2 - R*s2) = 1/x */  \
+    __ASM_EMIT("fmul            v16.4s, v16.4s, v4.4s")             /* v16  = k/x */ \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("fmul            v0.4s, v0.4s, v16.4s")              /* v0   = x * int(k/x) */ \
+    __ASM_EMIT("fsub            v0.4s, v4.4s, v0.4s")               /* v0   = k - x * int(k/x) */ \
+    __ASM_EMIT("st1             {v0.s}[0], [%[" DST "]]") \
+    __ASM_EMIT("subs            %[count], %[count], #1") \
+    __ASM_EMIT(INC("add         %[" SRC "], %[" SRC "], #0x04")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x04") \
+    __ASM_EMIT("bge             7b") \
+    __ASM_EMIT("8:")
+
+    void mod_k2(float *dst, float k, size_t count)
+    {
+        IF_ARCH_AARCH64(float *pk = &k);
+        ARCH_AARCH64_ASM
+        (
+            __ASM_EMIT("ld1r       {v4.4s}, [%[k]]")
+            MOD_KX_CORE("dst", "dst", INC_OFF)
+            : [dst] "+r" (dst),
+              [count] "+r" (count)
+            : [k] "r" (pk)
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q16", "q17", "q18", "q19", "q20", "q21", "q22", "q23"
+        );
+    }
+
+    void rmod_k2(float *dst, float k, size_t count)
+    {
+        IF_ARCH_AARCH64(float *pk = &k);
+        ARCH_AARCH64_ASM
+        (
+            __ASM_EMIT("ld1r        {v4.4s}, [%[k]]")
+            RMOD_KX_CORE("dst", "dst", INC_OFF)
+            : [dst] "+r" (dst),
+              [count] "+r" (count)
+            : [k] "r" (pk)
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q16", "q17", "q18", "q19", "q20", "q21", "q22", "q23"
+        );
+    }
+
+    void mod_k3(float *dst, const float *src, float k, size_t count)
+    {
+        IF_ARCH_AARCH64(float *pk = &k);
+        ARCH_AARCH64_ASM
+        (
+            __ASM_EMIT("ld1r        {v4.4s}, [%[k]]")
+            MOD_KX_CORE("dst", "src", INC_ON)
+            : [dst] "+r" (dst), [src] "+r" (src),
+              [count] "+r" (count)
+            : [k] "r" (pk)
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q16", "q17", "q18", "q19", "q20", "q21", "q22", "q23"
+        );
+    }
+
+    void rmod_k3(float *dst, const float *src, float k, size_t count)
+    {
+        IF_ARCH_AARCH64(float *pk = &k);
+        ARCH_AARCH64_ASM
+        (
+            __ASM_EMIT("ld1r        {v4.4s}, [%[k]]")
+            RMOD_KX_CORE("dst", "src", INC_ON)
+            : [dst] "+r" (dst), [src] "+r" (src),
+              [count] "+r" (count)
+            : [k] "r" (pk)
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q16", "q17", "q18", "q19", "q20", "q21", "q22", "q23"
+        );
+    }
+
+#undef INC_ON
+#undef INC_OFF
+
 #undef OP_DORDER
 #undef OP_RORDER
 #undef OP_K2_CORE
 #undef OP_K3_CORE
 #undef OP_DIV2_CORE
 #undef OP_DIV3_CORE
+#undef MOD_KX_CORE
+#undef RMOD_KX_CORE
 }
 
 #endif /* DSP_ARCH_AARCH64_ASIMD_PMATH_OP_KX_H_ */
