@@ -19,7 +19,7 @@ namespace asimd
 
 #define OP_VV2_CORE(DST, SRC, OP, SEL) \
     __ASM_EMIT("subs        %[count], %[count], #32") \
-    __ASM_EMIT("blo         2f") \
+    __ASM_EMIT("b.lo        2f") \
     /* 32x blocks */ \
     __ASM_EMIT("1:") \
     __ASM_EMIT("ldp         q0, q1, [%[" SEL(DST, SRC) "], #0x00]") \
@@ -45,11 +45,11 @@ namespace asimd
     __ASM_EMIT("subs        %[count], %[count], #32") \
     __ASM_EMIT("add         %[" DST "], %[" DST "], #0x80") \
     __ASM_EMIT("add         %[" SRC "], %[" SRC "], #0x80") \
-    __ASM_EMIT("bhs         1b") \
+    __ASM_EMIT("b.hs        1b") \
     /* 16x block */ \
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds        %[count], %[count], #16") /* 32 - 16 */ \
-    __ASM_EMIT("blt         4f") \
+    __ASM_EMIT("b.lt        4f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SEL(DST, SRC) "], #0x00]") \
     __ASM_EMIT("ldp         q2, q3, [%[" SEL(DST, SRC) "], #0x20]") \
     __ASM_EMIT("ldp         q16, q17, [%[" SEL(SRC, DST) "], #0x00]") \
@@ -66,7 +66,7 @@ namespace asimd
     /* 8x block */ \
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds        %[count], %[count], #8") /* 16 - 8 */ \
-    __ASM_EMIT("blt         6f") \
+    __ASM_EMIT("b.lt        6f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SEL(DST, SRC) "], #0x00]") \
     __ASM_EMIT("ldp         q2, q3, [%[" SEL(DST, SRC) "], #0x20]") \
     __ASM_EMIT("ldp         q16, q17, [%[" SEL(SRC, DST) "], #0x00]") \
@@ -80,7 +80,7 @@ namespace asimd
     /* 4x block */ \
     __ASM_EMIT("6:") \
     __ASM_EMIT("adds        %[count], %[count], #4") /* 8 - 4 */ \
-    __ASM_EMIT("blt         8f") \
+    __ASM_EMIT("b.lt        8f") \
     __ASM_EMIT("ldr         q0, [%[" SEL(DST, SRC) "], #0x00]") \
     __ASM_EMIT("ldr         q16, [%[" SEL(SRC, DST) "], #0x00]") \
     __ASM_EMIT(OP "         v0.4s, v0.4s, v16.4s") \
@@ -91,7 +91,7 @@ namespace asimd
     /* 1x block */ \
     __ASM_EMIT("8:") \
     __ASM_EMIT("adds        %[count], %[count], #3") /* 4 - 3 */ \
-    __ASM_EMIT("blt         10f") \
+    __ASM_EMIT("b.lt        10f") \
     __ASM_EMIT("9:") \
     __ASM_EMIT("ld1r        {v0.4s}, [%[" SEL(DST, SRC) "]]") \
     __ASM_EMIT("ld1r        {v16.4s}, [%[" SEL(SRC, DST) "]]") \
@@ -163,7 +163,7 @@ namespace asimd
 
 #define OP_DIV2_CORE(DST, SRC, SEL) \
     __ASM_EMIT("subs        %[count], %[count], #16") \
-    __ASM_EMIT("blo         2f") \
+    __ASM_EMIT("b.lo        2f") \
     /* 16x blocks */ \
     __ASM_EMIT("1:") \
     __ASM_EMIT("ldp         q0, q1, [%[" SEL(SRC, DST) "], #0x00]") \
@@ -199,11 +199,11 @@ namespace asimd
     __ASM_EMIT("subs        %[count], %[count], #16") \
     __ASM_EMIT("add         %[" DST "], %[" DST "], #0x40") \
     __ASM_EMIT("add         %[" SRC "], %[" SRC "], #0x40") \
-    __ASM_EMIT("bhs         1b") \
+    __ASM_EMIT("b.hs        1b") \
     /* 8x block */ \
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds        %[count], %[count], #8")            /* 16 - 8 */ \
-    __ASM_EMIT("blt         4f") \
+    __ASM_EMIT("b.lt        4f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SEL(SRC, DST) "], #0x00]") \
     __ASM_EMIT("ldp         q4, q5, [%[" SEL(DST, SRC) "], #0x00]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
@@ -225,7 +225,7 @@ namespace asimd
     /* 4x block */ \
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds        %[count], %[count], #4")            /* 8 - 4 */ \
-    __ASM_EMIT("blt         6f") \
+    __ASM_EMIT("b.lt        6f") \
     __ASM_EMIT("ldr         q0, [%[" SEL(SRC, DST) "], #0x00]") \
     __ASM_EMIT("ldr         q4, [%[" SEL(DST, SRC) "], #0x00]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
@@ -241,7 +241,7 @@ namespace asimd
     /* 1x block */ \
     __ASM_EMIT("6:") \
     __ASM_EMIT("adds        %[count], %[count], #3")            /* 4 - 3 */ \
-    __ASM_EMIT("blt         8f") \
+    __ASM_EMIT("b.lt        8f") \
     __ASM_EMIT("7:") \
     __ASM_EMIT("ld1r        {v0.4s}, [%[" SEL(SRC, DST) "]]") \
     __ASM_EMIT("ld1r        {v4.4s}, [%[" SEL(DST, SRC) "]]") \
@@ -290,12 +290,9 @@ namespace asimd
 
 #undef OP_DIV2_CORE
 
-#undef OP_SDSEL
-#undef OP_SRSEL
-
 #define OP_VV3_CORE(DST, SRC1, SRC2, OP) \
     __ASM_EMIT("subs        %[count], %[count], #32") \
-    __ASM_EMIT("blo         2f") \
+    __ASM_EMIT("b.lo        2f") \
     /* 32x blocks */ \
     __ASM_EMIT("1:") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC1 "], #0x00]") \
@@ -322,11 +319,11 @@ namespace asimd
     __ASM_EMIT("add         %[" DST "], %[" DST "], #0x80") \
     __ASM_EMIT("add         %[" SRC1 "], %[" SRC1 "], #0x80") \
     __ASM_EMIT("add         %[" SRC2 "], %[" SRC2 "], #0x80") \
-    __ASM_EMIT("bhs         1b") \
+    __ASM_EMIT("b.hs        1b") \
     /* 16x block */ \
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds        %[count], %[count], #16") /* 32 - 16 */ \
-    __ASM_EMIT("blt         4f") \
+    __ASM_EMIT("b.lt        4f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC1 "], #0x00]") \
     __ASM_EMIT("ldp         q2, q3, [%[" SRC1 "], #0x20]") \
     __ASM_EMIT("ldp         q16, q17, [%[" SRC2 "], #0x00]") \
@@ -344,7 +341,7 @@ namespace asimd
     /* 8x block */ \
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds        %[count], %[count], #8") /* 16 - 8 */ \
-    __ASM_EMIT("blt         6f") \
+    __ASM_EMIT("b.lt        6f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC1 "], #0x00]") \
     __ASM_EMIT("ldp         q16, q17, [%[" SRC2 "], #0x00]") \
     __ASM_EMIT(OP "         v0.4s, v0.4s, v16.4s") \
@@ -357,7 +354,7 @@ namespace asimd
     /* 4x block */ \
     __ASM_EMIT("6:") \
     __ASM_EMIT("adds        %[count], %[count], #4") /* 8 - 4 */ \
-    __ASM_EMIT("blt         8f") \
+    __ASM_EMIT("b.lt        8f") \
     __ASM_EMIT("ldr         q0, [%[" SRC1 "], #0x00]") \
     __ASM_EMIT("ldr         q16, [%[" SRC2 "], #0x00]") \
     __ASM_EMIT(OP "         v0.4s, v0.4s, v16.4s") \
@@ -369,7 +366,7 @@ namespace asimd
     /* 1x block */ \
     __ASM_EMIT("8:") \
     __ASM_EMIT("adds        %[count], %[count], #3") /* 4 - 3 */ \
-    __ASM_EMIT("blt         10f") \
+    __ASM_EMIT("b.lt        10f") \
     __ASM_EMIT("9:") \
     __ASM_EMIT("ld1r        {v0.4s}, [%[" SRC1 "]]") \
     __ASM_EMIT("ld1r        {v16.4s}, [%[" SRC2 "]]") \
@@ -428,7 +425,7 @@ namespace asimd
 
 #define OP_DIV3_CORE(DST, SRC1, SRC2) \
     __ASM_EMIT("subs        %[count], %[count], #16") \
-    __ASM_EMIT("blo         2f") \
+    __ASM_EMIT("b.lo        2f") \
     /* 16x blocks */ \
     __ASM_EMIT("1:") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC2 "], #0x00]") \
@@ -465,11 +462,11 @@ namespace asimd
     __ASM_EMIT("add         %[" DST "], %[" DST "], #0x40") \
     __ASM_EMIT("add         %[" SRC1 "], %[" SRC1 "], #0x40") \
     __ASM_EMIT("add         %[" SRC2 "], %[" SRC2 "], #0x40") \
-    __ASM_EMIT("bhs         1b") \
+    __ASM_EMIT("b.hs        1b") \
     /* 8x block */ \
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds        %[count], %[count], #8")            /* 16 - 8 */ \
-    __ASM_EMIT("blt         4f") \
+    __ASM_EMIT("b.lt        4f") \
     __ASM_EMIT("ldp         q0, q1, [%[" SRC2 "], #0x00]") \
     __ASM_EMIT("ldp         q4, q5, [%[" SRC1 "], #0x00]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
@@ -492,7 +489,7 @@ namespace asimd
     /* 4x block */ \
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds        %[count], %[count], #4")            /* 8 - 4 */ \
-    __ASM_EMIT("blt         6f") \
+    __ASM_EMIT("b.lt        6f") \
     __ASM_EMIT("ldr         q0, [%[" SRC2 "], #0x00]") \
     __ASM_EMIT("ldr         q4, [%[" SRC1 "], #0x00]") \
     __ASM_EMIT("frecpe      v16.4s, v0.4s")                     /* v16 = s2 */ \
@@ -509,7 +506,7 @@ namespace asimd
     /* 1x block */ \
     __ASM_EMIT("6:") \
     __ASM_EMIT("adds        %[count], %[count], #3")            /* 4 - 3 */ \
-    __ASM_EMIT("blt         8f") \
+    __ASM_EMIT("b.lt        8f") \
     __ASM_EMIT("7:") \
     __ASM_EMIT("ld1r        {v0.4s}, [%[" SRC2 "]]") \
     __ASM_EMIT("ld1r        {v4.4s}, [%[" SRC1 "]]") \
@@ -544,6 +541,175 @@ namespace asimd
 
 #undef OP_DIV3_CORE
 
+#define INC_ON(x)           x
+#define INC_OFF(x)
+
+#define FMOD_VV_CORE(DST, SRC1, INC1, SRC2, INC2, SEL)   \
+    __ASM_EMIT("subs            %[count], %[count], #16") \
+    __ASM_EMIT("b.lo            2f") \
+    /* 16x blocks */ \
+    __ASM_EMIT("1:") \
+    __ASM_EMIT("ldp             q0, q1, [%[" SRC1 "], #0x00]") \
+    __ASM_EMIT("ldp             q2, q3, [%[" SRC1 "], #0x20]") \
+    __ASM_EMIT("ldp             q4, q5, [%[" SRC2 "], #0x00]") \
+    __ASM_EMIT("ldp             q6, q7, [%[" SRC2 "], #0x20]") \
+    __ASM_EMIT("frecpe          v16.4s, " SEL("v4.4s", "v0.4s"))                    /* v16  = d */ \
+    __ASM_EMIT("frecpe          v17.4s, " SEL("v5.4s", "v1.4s")) \
+    __ASM_EMIT("frecpe          v18.4s, " SEL("v6.4s", "v2.4s")) \
+    __ASM_EMIT("frecpe          v19.4s, " SEL("v7.4s", "v3.4s")) \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, " SEL("v4.4s", "v0.4s"))            /* v20  = (2 - R*s2) */ \
+    __ASM_EMIT("frecps          v21.4s, v17.4s, " SEL("v5.4s", "v1.4s")) \
+    __ASM_EMIT("frecps          v22.4s, v18.4s, " SEL("v6.4s", "v2.4s")) \
+    __ASM_EMIT("frecps          v23.4s, v19.4s, " SEL("v7.4s", "v3.4s")) \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")                            /* v16  = s2' = s2 * (2 - R*s2) */ \
+    __ASM_EMIT("fmul            v17.4s, v21.4s, v17.4s") \
+    __ASM_EMIT("fmul            v18.4s, v22.4s, v18.4s") \
+    __ASM_EMIT("fmul            v19.4s, v23.4s, v19.4s") \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, " SEL("v4.4s", "v0.4s"))            /* v20  = (2 - R*s2') */ \
+    __ASM_EMIT("frecps          v21.4s, v17.4s, " SEL("v5.4s", "v1.4s")) \
+    __ASM_EMIT("frecps          v22.4s, v18.4s, " SEL("v6.4s", "v2.4s")) \
+    __ASM_EMIT("frecps          v23.4s, v19.4s, " SEL("v7.4s", "v3.4s")) \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")                            /* v16  = s2" = s2' * (2 - R*s2) = 1/d */  \
+    __ASM_EMIT("fmul            v17.4s, v21.4s, v17.4s") \
+    __ASM_EMIT("fmul            v18.4s, v22.4s, v18.4s") \
+    __ASM_EMIT("fmul            v19.4s, v23.4s, v19.4s") \
+    __ASM_EMIT("fmul            v16.4s, v16.4s, " SEL("v0.4s", "v4.4s"))            /* v16  = s/d */  \
+    __ASM_EMIT("fmul            v17.4s, v17.4s, " SEL("v1.4s", "v5.4s")) \
+    __ASM_EMIT("fmul            v18.4s, v18.4s, " SEL("v2.4s", "v6.4s")) \
+    __ASM_EMIT("fmul            v19.4s, v19.4s, " SEL("v3.4s", "v7.4s")) \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("frintz          v17.4s, v17.4s") \
+    __ASM_EMIT("frintz          v18.4s, v18.4s") \
+    __ASM_EMIT("frintz          v19.4s, v19.4s") \
+    __ASM_EMIT("fmls            " SEL("v0.4s", "v4.4s") ", v16.4s, " SEL("v4.4s", "v0.4s")) \
+    __ASM_EMIT("fmls            " SEL("v1.4s", "v5.4s") ", v17.4s, " SEL("v5.4s", "v1.4s")) \
+    __ASM_EMIT("fmls            " SEL("v2.4s", "v6.4s") ", v18.4s, " SEL("v6.4s", "v2.4s")) \
+    __ASM_EMIT("fmls            " SEL("v3.4s", "v7.4s") ", v19.4s, " SEL("v7.4s", "v3.4s")) \
+    __ASM_EMIT("stp             " SEL("q0, q1", "q4, q5") ", [%[" DST "], #0x00]") \
+    __ASM_EMIT("stp             " SEL("q2, q3", "q6, q7") ", [%[" DST "], #0x20]") \
+    __ASM_EMIT("subs            %[count], %[count], #16") \
+    __ASM_EMIT(INC1("add        %[" SRC1 "], %[" SRC1 "], #0x40")) \
+    __ASM_EMIT(INC2("add        %[" SRC2 "], %[" SRC2 "], #0x40")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x40") \
+    __ASM_EMIT("b.hs            1b") \
+    /* 8x block */ \
+    __ASM_EMIT("2:") \
+    __ASM_EMIT("adds            %[count], %[count], #8") \
+    __ASM_EMIT("b.lt            4f") \
+    __ASM_EMIT("ldp             q0, q1, [%[" SRC1 "], #0x00]") \
+    __ASM_EMIT("ldp             q4, q5, [%[" SRC2 "], #0x00]") \
+    __ASM_EMIT("frecpe          v16.4s, " SEL("v4.4s", "v0.4s"))                    /* v16  = d */ \
+    __ASM_EMIT("frecpe          v17.4s, " SEL("v5.4s", "v1.4s")) \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, " SEL("v4.4s", "v0.4s"))            /* v20  = (2 - R*s2) */ \
+    __ASM_EMIT("frecps          v21.4s, v17.4s, " SEL("v5.4s", "v1.4s")) \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")                            /* v16  = s2' = s2 * (2 - R*s2) */ \
+    __ASM_EMIT("fmul            v17.4s, v21.4s, v17.4s") \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, " SEL("v4.4s", "v0.4s"))            /* v20  = (2 - R*s2') */ \
+    __ASM_EMIT("frecps          v21.4s, v17.4s, " SEL("v5.4s", "v1.4s")) \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")                            /* v16  = s2" = s2' * (2 - R*s2) = 1/d */  \
+    __ASM_EMIT("fmul            v17.4s, v21.4s, v17.4s") \
+    __ASM_EMIT("fmul            v16.4s, v16.4s, " SEL("v0.4s", "v4.4s"))            /* v16  = s/d */  \
+    __ASM_EMIT("fmul            v17.4s, v17.4s, " SEL("v1.4s", "v5.4s")) \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("frintz          v17.4s, v17.4s") \
+    __ASM_EMIT("fmls            " SEL("v0.4s", "v4.4s") ", v16.4s, " SEL("v4.4s", "v0.4s")) \
+    __ASM_EMIT("fmls            " SEL("v1.4s", "v5.4s") ", v17.4s, " SEL("v5.4s", "v1.4s")) \
+    __ASM_EMIT("stp             " SEL("q0, q1", "q4, q5") ", [%[" DST "], #0x00]") \
+    __ASM_EMIT("sub             %[count], %[count], #8") \
+    __ASM_EMIT(INC1("add        %[" SRC1 "], %[" SRC1 "], #0x20")) \
+    __ASM_EMIT(INC2("add        %[" SRC2 "], %[" SRC2 "], #0x20")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x20") \
+    /* 4x blocks */ \
+    __ASM_EMIT("4:") \
+    __ASM_EMIT("adds            %[count], %[count], #4") \
+    __ASM_EMIT("b.lt            6f") \
+    __ASM_EMIT("ldr             q0, [%[" SRC1 "]]") \
+    __ASM_EMIT("ldr             q4, [%[" SRC2 "]]") \
+    __ASM_EMIT("frecpe          v16.4s, " SEL("v4.4s", "v0.4s"))                    /* v16  = d */ \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, " SEL("v4.4s", "v0.4s"))            /* v20  = (2 - R*s2) */ \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")                            /* v16  = s2' = s2 * (2 - R*s2) */ \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, " SEL("v4.4s", "v0.4s"))            /* v20  = (2 - R*s2') */ \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")                            /* v16  = s2" = s2' * (2 - R*s2) = 1/d */  \
+    __ASM_EMIT("fmul            v16.4s, v16.4s, " SEL("v0.4s", "v4.4s"))            /* v16  = s/d */  \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("fmls            " SEL("v0.4s", "v4.4s") ", v16.4s, " SEL("v4.4s", "v0.4s")) \
+    __ASM_EMIT("str             " SEL("q0", "q4") ", [%[" DST "]]") \
+    __ASM_EMIT("sub             %[count], %[count], #4") \
+    __ASM_EMIT(INC1("add        %[" SRC1 "], %[" SRC1 "], #0x10")) \
+    __ASM_EMIT(INC2("add        %[" SRC2 "], %[" SRC2 "], #0x10")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x10") \
+    /* 1x blocks */ \
+    __ASM_EMIT("6:") \
+    __ASM_EMIT("adds            %[count], %[count], #3") \
+    __ASM_EMIT("b.lt            8f") \
+    __ASM_EMIT("7:") \
+    __ASM_EMIT("ld1r            {v0.4s}, [%[" SRC1 "]]") \
+    __ASM_EMIT("ld1r            {v4.4s}, [%[" SRC2 "]]") \
+    __ASM_EMIT("frecpe          v16.4s, " SEL("v4.4s", "v0.4s"))                    /* v16  = d */ \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, " SEL("v4.4s", "v0.4s"))            /* v20  = (2 - R*s2) */ \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")                            /* v16  = s2' = s2 * (2 - R*s2) */ \
+    __ASM_EMIT("frecps          v20.4s, v16.4s, " SEL("v4.4s", "v0.4s"))            /* v20  = (2 - R*s2') */ \
+    __ASM_EMIT("fmul            v16.4s, v20.4s, v16.4s")                            /* v16  = s2" = s2' * (2 - R*s2) = 1/d */  \
+    __ASM_EMIT("fmul            v16.4s, v16.4s, " SEL("v0.4s", "v4.4s"))            /* v16  = s/d */  \
+    __ASM_EMIT("frintz          v16.4s, v16.4s") \
+    __ASM_EMIT("fmls            " SEL("v0.4s", "v4.4s") ", v16.4s, " SEL("v4.4s", "v0.4s")) \
+    __ASM_EMIT("st1             " SEL("{v0.s}[0]", "{v4.s}[0]") ", [%[" DST "]]") \
+    __ASM_EMIT("subs            %[count], %[count], #1") \
+    __ASM_EMIT(INC1("add        %[" SRC1 "], %[" SRC1 "], #0x04")) \
+    __ASM_EMIT(INC2("add        %[" SRC2 "], %[" SRC2 "], #0x04")) \
+    __ASM_EMIT("add             %[" DST "], %[" DST "], #0x04") \
+    __ASM_EMIT("bge             7b") \
+    __ASM_EMIT("8:")
+
+    void mod2(float *dst, const float *src, size_t count)
+    {
+        ARCH_AARCH64_ASM
+        (
+            FMOD_VV_CORE("dst", "dst", INC_OFF, "src", INC_ON, OP_SDSEL)
+            : [dst] "+r" (dst), [src] "+r" (src),
+              [count] "+r" (count)
+            :
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q16", "q17", "q18", "q19", "q20", "q21", "q22", "q23"
+        );
+    }
+
+    void rmod2(float *dst, const float *src, size_t count)
+    {
+        ARCH_AARCH64_ASM
+        (
+            FMOD_VV_CORE("dst", "dst", INC_OFF, "src", INC_ON, OP_SRSEL)
+            : [dst] "+r" (dst), [src] "+r" (src),
+              [count] "+r" (count)
+            :
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q16", "q17", "q18", "q19", "q20", "q21", "q22", "q23"
+        );
+    }
+
+    void mod3(float *dst, const float *src1, const float *src2, size_t count)
+    {
+        ARCH_AARCH64_ASM
+        (
+            FMOD_VV_CORE("dst", "src1", INC_ON, "src2", INC_ON, OP_SDSEL)
+            : [dst] "+r" (dst), [src1] "+r" (src1), [src2] "+r" (src2),
+              [count] "+r" (count)
+            :
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q16", "q17", "q18", "q19", "q20", "q21", "q22", "q23"
+        );
+    }
+
+#undef FMOD_VV_CORE
+#undef INC_ON
+#undef INC_OFF
+
+#undef OP_SDSEL
+#undef OP_SRSEL
+
 }
 
 #endif /* DSP_ARCH_AARCH64_ASIMD_PMATH_OP_VV_H_ */
