@@ -133,13 +133,13 @@ namespace neon_d32
     __ASM_EMIT("bge         9b") \
     __ASM_EMIT("10:")
 
-#define RDIV_K2_CORE(DST)   \
+#define RDIV_KX_CORE(DST, SRC, INC)   \
     __ASM_EMIT("subs            %[count], $16") \
     __ASM_EMIT("vmov            q5, q4") \
     __ASM_EMIT("blo             2f") \
     /* 16x blocks */ \
     __ASM_EMIT("1:") \
-    __ASM_EMIT("vldm            %[" DST "], {q0-q3}") \
+    __ASM_EMIT("vldm            %[" SRC "]" INC ", {q0-q3}") \
     __ASM_EMIT("vrecpe.f32      q8, q0")                    /* q8 = s2 */ \
     __ASM_EMIT("vrecpe.f32      q9, q1") \
     __ASM_EMIT("vrecpe.f32      q10, q2") \
@@ -171,7 +171,7 @@ namespace neon_d32
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds            %[count], $8") \
     __ASM_EMIT("blt             4f") \
-    __ASM_EMIT("vldm            %[" DST "], {q0-q1}") \
+    __ASM_EMIT("vldm            %[" SRC "]" INC ", {q0-q1}") \
     __ASM_EMIT("vrecpe.f32      q8, q0")                    /* q8 = s2 */ \
     __ASM_EMIT("vrecpe.f32      q9, q1") \
     __ASM_EMIT("vrecps.f32      q12, q8, q0")               /* q12 = (2 - R*s2) */ \
@@ -190,7 +190,7 @@ namespace neon_d32
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds            %[count], $4") \
     __ASM_EMIT("blt             6f") \
-    __ASM_EMIT("vldm            %[" DST "], {q0}") \
+    __ASM_EMIT("vldm            %[" SRC "]" INC ", {q0}") \
     __ASM_EMIT("vrecpe.f32      q8, q0")                    /* q8 = s2 */ \
     __ASM_EMIT("vrecps.f32      q12, q8, q0")               /* q12 = (2 - R*s2) */ \
     __ASM_EMIT("vmul.f32        q8, q12, q8")               /* q8 = s2' = s2 * (2 - R*s2) */ \
@@ -204,7 +204,7 @@ namespace neon_d32
     __ASM_EMIT("adds            %[count], $3") \
     __ASM_EMIT("blt             8f") \
     __ASM_EMIT("7:") \
-    __ASM_EMIT("vld1.32         {d0[], d1[]}, [%[" DST "]]") \
+    __ASM_EMIT("vld1.32         {d0[], d1[]}, [%[" SRC "]]" INC ) \
     __ASM_EMIT("vrecpe.f32      q8, q0")                    /* q8 = s2 */ \
     __ASM_EMIT("vrecps.f32      q12, q8, q0")               /* q12 = (2 - R*s2) */ \
     __ASM_EMIT("vmul.f32        q8, q12, q8")               /* q8 = s2' = s2 * (2 - R*s2) */ \
@@ -216,13 +216,87 @@ namespace neon_d32
     __ASM_EMIT("bge             7b") \
     __ASM_EMIT("8:")
 
-#define RDIV_K3_CORE(DST, SRC)   \
+
+#define MOD_KX_CORE(DST, SRC, INC)   \
     __ASM_EMIT("subs            %[count], $16") \
-    __ASM_EMIT("vmov            q5, q4") \
+    __ASM_EMIT("vrecpe.f32      q8, q4") \
+    __ASM_EMIT("vmov            q5, q4")                    /* q5 = k */ \
+    __ASM_EMIT("vrecps.f32      q12, q8, q4") \
+    __ASM_EMIT("vmul.f32        q8, q12, q8") \
+    __ASM_EMIT("vrecps.f32      q12, q8, q4") \
+    __ASM_EMIT("vmul.f32        q6, q12, q8")               /* q6 = 1/k */ \
+    __ASM_EMIT("vmov            q7, q6")                    /* q7 = 1/k */ \
     __ASM_EMIT("blo             2f") \
     /* 16x blocks */ \
     __ASM_EMIT("1:") \
-    __ASM_EMIT("vldm            %[" SRC "]!, {q0-q3}") \
+    __ASM_EMIT("vldm            %[" SRC "]" INC ", {q0-q3}") \
+    __ASM_EMIT("vmul.f32        q8, q0, q6")                /* q0 = x/k */  \
+    __ASM_EMIT("vmul.f32        q9, q1, q7") \
+    __ASM_EMIT("vmul.f32        q10, q2, q6") \
+    __ASM_EMIT("vmul.f32        q11, q3, q7") \
+    __ASM_EMIT("vcvt.s32.f32    q8, q8") \
+    __ASM_EMIT("vcvt.s32.f32    q9, q9") \
+    __ASM_EMIT("vcvt.s32.f32    q10, q10") \
+    __ASM_EMIT("vcvt.s32.f32    q11, q11") \
+    __ASM_EMIT("vcvt.f32.s32    q8, q8") \
+    __ASM_EMIT("vcvt.f32.s32    q9, q9") \
+    __ASM_EMIT("vcvt.f32.s32    q10, q10") \
+    __ASM_EMIT("vcvt.f32.s32    q11, q11") \
+    __ASM_EMIT("vmls.f32        q0, q8, q4") \
+    __ASM_EMIT("vmls.f32        q1, q9, q5") \
+    __ASM_EMIT("vmls.f32        q2, q10, q4") \
+    __ASM_EMIT("vmls.f32        q3, q11, q5") \
+    __ASM_EMIT("vstm            %[" DST "]!, {q0-q3}") \
+    __ASM_EMIT("subs            %[count], $16") \
+    __ASM_EMIT("bhs             1b") \
+    /* 8x block */ \
+    __ASM_EMIT("2:") \
+    __ASM_EMIT("adds            %[count], $8") \
+    __ASM_EMIT("blt             4f") \
+    __ASM_EMIT("vldm            %[" SRC "]" INC ", {q0-q1}") \
+    __ASM_EMIT("vmul.f32        q8, q0, q6")                /* q0 = x/k */  \
+    __ASM_EMIT("vmul.f32        q9, q1, q7") \
+    __ASM_EMIT("vcvt.s32.f32    q8, q8") \
+    __ASM_EMIT("vcvt.s32.f32    q9, q9") \
+    __ASM_EMIT("vcvt.f32.s32    q8, q8") \
+    __ASM_EMIT("vcvt.f32.s32    q9, q9") \
+    __ASM_EMIT("vmls.f32        q0, q8, q4") \
+    __ASM_EMIT("vmls.f32        q1, q9, q5") \
+    __ASM_EMIT("sub             %[count], $8") \
+    __ASM_EMIT("vstm            %[" DST "]!, {q0-q1}") \
+    /* 4x blocks */ \
+    __ASM_EMIT("4:") \
+    __ASM_EMIT("adds            %[count], $4") \
+    __ASM_EMIT("blt             6f") \
+    __ASM_EMIT("vldm            %[" SRC "]" INC ", {q0}") \
+    __ASM_EMIT("vmul.f32        q8, q0, q6")                /* q0 = x/k */  \
+    __ASM_EMIT("vcvt.s32.f32    q8, q8") \
+    __ASM_EMIT("vcvt.f32.s32    q8, q8") \
+    __ASM_EMIT("vmls.f32        q0, q8, q4") \
+    __ASM_EMIT("sub             %[count], $4") \
+    __ASM_EMIT("vstm            %[" DST "]!, {q0}") \
+    /* 1x blocks */ \
+    __ASM_EMIT("6:") \
+    __ASM_EMIT("adds            %[count], $3") \
+    __ASM_EMIT("blt             8f") \
+    __ASM_EMIT("7:") \
+    __ASM_EMIT("vld1.32         {d0[], d1[]}, [%[" SRC "]]" INC) \
+    __ASM_EMIT("vmul.f32        q8, q0, q6")                /* q0 = x/k */  \
+    __ASM_EMIT("vcvt.s32.f32    q8, q8") \
+    __ASM_EMIT("vcvt.f32.s32    q8, q8") \
+    __ASM_EMIT("vmls.f32        q0, q8, q4") \
+    __ASM_EMIT("vst1.32         {d0[0]}, [%[" DST "]]!") \
+    __ASM_EMIT("subs            %[count], $1") \
+    __ASM_EMIT("bge             7b") \
+    __ASM_EMIT("8:")
+
+#define RMOD_KX_CORE(DST, SRC, INC)   \
+    __ASM_EMIT("subs            %[count], $16") \
+    __ASM_EMIT("vmov            q5, q4")                    /* q5 = k */ \
+    __ASM_EMIT("blo             2f") \
+    /* 16x blocks */ \
+    __ASM_EMIT("1:") \
+    __ASM_EMIT("vldm            %[" SRC "]" INC ", {q0-q3}") \
     __ASM_EMIT("vrecpe.f32      q8, q0")                    /* q8 = s2 */ \
     __ASM_EMIT("vrecpe.f32      q9, q1") \
     __ASM_EMIT("vrecpe.f32      q10, q2") \
@@ -239,14 +313,30 @@ namespace neon_d32
     __ASM_EMIT("vrecps.f32      q13, q9, q1") \
     __ASM_EMIT("vrecps.f32      q14, q10, q2") \
     __ASM_EMIT("vrecps.f32      q15, q11, q3") \
-    __ASM_EMIT("vmul.f32        q0, q12, q8")               /* q0 = s2" = s2' * (2 - R*s2) = 1/s2 */  \
-    __ASM_EMIT("vmul.f32        q1, q13, q9") \
-    __ASM_EMIT("vmul.f32        q2, q14, q10") \
-    __ASM_EMIT("vmul.f32        q3, q15, q11") \
-    __ASM_EMIT("vmul.f32        q0, q0, q4")                /* k / s2 */ \
-    __ASM_EMIT("vmul.f32        q1, q1, q5") \
-    __ASM_EMIT("vmul.f32        q2, q2, q4") \
-    __ASM_EMIT("vmul.f32        q3, q3, q5") \
+    __ASM_EMIT("vmul.f32        q8, q12, q8")               /* q8 = s2" = s2' * (2 - R*s2) = 1/x */  \
+    __ASM_EMIT("vmul.f32        q9, q13, q9") \
+    __ASM_EMIT("vmul.f32        q10, q14, q10") \
+    __ASM_EMIT("vmul.f32        q11, q15, q11") \
+    __ASM_EMIT("vmul.f32        q8, q8, q4")                /* q8 = k/x */ \
+    __ASM_EMIT("vmul.f32        q9, q9, q5") \
+    __ASM_EMIT("vmul.f32        q10, q10, q4") \
+    __ASM_EMIT("vmul.f32        q11, q11, q5") \
+    __ASM_EMIT("vcvt.s32.f32    q8, q8") \
+    __ASM_EMIT("vcvt.s32.f32    q9, q9") \
+    __ASM_EMIT("vcvt.s32.f32    q10, q10") \
+    __ASM_EMIT("vcvt.s32.f32    q11, q11") \
+    __ASM_EMIT("vcvt.f32.s32    q8, q8") \
+    __ASM_EMIT("vcvt.f32.s32    q9, q9") \
+    __ASM_EMIT("vcvt.f32.s32    q10, q10") \
+    __ASM_EMIT("vcvt.f32.s32    q11, q11") \
+    __ASM_EMIT("vmul.f32        q0, q0, q8")                /* q0 = x * int(k/x) */ \
+    __ASM_EMIT("vmul.f32        q1, q1, q9") \
+    __ASM_EMIT("vmul.f32        q2, q2, q10") \
+    __ASM_EMIT("vmul.f32        q3, q3, q11") \
+    __ASM_EMIT("vsub.f32        q0, q4, q0")                /* q0 = k - x * int(k/x) */ \
+    __ASM_EMIT("vsub.f32        q1, q5, q1") \
+    __ASM_EMIT("vsub.f32        q2, q4, q2") \
+    __ASM_EMIT("vsub.f32        q3, q5, q3") \
     __ASM_EMIT("vstm            %[" DST "]!, {q0-q3}") \
     __ASM_EMIT("subs            %[count], $16") \
     __ASM_EMIT("bhs             1b") \
@@ -254,7 +344,7 @@ namespace neon_d32
     __ASM_EMIT("2:") \
     __ASM_EMIT("adds            %[count], $8") \
     __ASM_EMIT("blt             4f") \
-    __ASM_EMIT("vldm            %[" SRC "]!, {q0-q1}") \
+    __ASM_EMIT("vldm            %[" SRC "]" INC ", {q0-q1}") \
     __ASM_EMIT("vrecpe.f32      q8, q0")                    /* q8 = s2 */ \
     __ASM_EMIT("vrecpe.f32      q9, q1") \
     __ASM_EMIT("vrecps.f32      q12, q8, q0")               /* q12 = (2 - R*s2) */ \
@@ -263,23 +353,35 @@ namespace neon_d32
     __ASM_EMIT("vmul.f32        q9, q13, q9") \
     __ASM_EMIT("vrecps.f32      q12, q8, q0")               /* q12 = (2 - R*s2') */ \
     __ASM_EMIT("vrecps.f32      q13, q9, q1") \
-    __ASM_EMIT("vmul.f32        q0, q12, q8")               /* q0 = s2" = s2' * (2 - R*s2) = 1/s2 */  \
-    __ASM_EMIT("vmul.f32        q1, q13, q9") \
-    __ASM_EMIT("vmul.f32        q0, q0, q4")                /* k / s2 */ \
-    __ASM_EMIT("vmul.f32        q1, q1, q5") \
+    __ASM_EMIT("vmul.f32        q8, q12, q8")               /* q8 = s2" = s2' * (2 - R*s2) = 1/x */  \
+    __ASM_EMIT("vmul.f32        q9, q13, q9") \
+    __ASM_EMIT("vmul.f32        q8, q8, q4")                /* q8 = k/x */ \
+    __ASM_EMIT("vmul.f32        q9, q9, q5") \
+    __ASM_EMIT("vcvt.s32.f32    q8, q8") \
+    __ASM_EMIT("vcvt.s32.f32    q9, q9") \
+    __ASM_EMIT("vcvt.f32.s32    q8, q8") \
+    __ASM_EMIT("vcvt.f32.s32    q9, q9") \
+    __ASM_EMIT("vmul.f32        q0, q0, q8")                /* q0 = x * int(k/x) */ \
+    __ASM_EMIT("vmul.f32        q1, q1, q9") \
+    __ASM_EMIT("vsub.f32        q0, q4, q0")                /* q0 = k - x * int(k/x) */ \
+    __ASM_EMIT("vsub.f32        q1, q5, q1") \
     __ASM_EMIT("sub             %[count], $8") \
     __ASM_EMIT("vstm            %[" DST "]!, {q0-q1}") \
     /* 4x blocks */ \
     __ASM_EMIT("4:") \
     __ASM_EMIT("adds            %[count], $4") \
     __ASM_EMIT("blt             6f") \
-    __ASM_EMIT("vldm            %[" SRC "]!, {q0}") \
+    __ASM_EMIT("vldm            %[" SRC "]" INC ", {q0}") \
     __ASM_EMIT("vrecpe.f32      q8, q0")                    /* q8 = s2 */ \
     __ASM_EMIT("vrecps.f32      q12, q8, q0")               /* q12 = (2 - R*s2) */ \
     __ASM_EMIT("vmul.f32        q8, q12, q8")               /* q8 = s2' = s2 * (2 - R*s2) */ \
     __ASM_EMIT("vrecps.f32      q12, q8, q0")               /* q12 = (2 - R*s2') */ \
-    __ASM_EMIT("vmul.f32        q0, q12, q8")               /* q0 = s2" = s2' * (2 - R*s2) = 1/s2 */  \
-    __ASM_EMIT("vmul.f32        q0, q0, q4")                /* s1 / s2 */ \
+    __ASM_EMIT("vmul.f32        q8, q12, q8")               /* q8 = s2" = s2' * (2 - R*s2) = 1/x */  \
+    __ASM_EMIT("vmul.f32        q8, q8, q4")                /* q8 = k/x */ \
+    __ASM_EMIT("vcvt.s32.f32    q8, q8") \
+    __ASM_EMIT("vcvt.f32.s32    q8, q8") \
+    __ASM_EMIT("vmul.f32        q0, q0, q8")                /* q0 = x * int(k/x) */ \
+    __ASM_EMIT("vsub.f32        q0, q4, q0")                /* q0 = k - x * int(k/x) */ \
     __ASM_EMIT("sub             %[count], $4") \
     __ASM_EMIT("vstm            %[" DST "]!, {q0}") \
     /* 1x blocks */ \
@@ -287,13 +389,17 @@ namespace neon_d32
     __ASM_EMIT("adds            %[count], $3") \
     __ASM_EMIT("blt             8f") \
     __ASM_EMIT("7:") \
-    __ASM_EMIT("vld1.32         {d0[], d1[]}, [%[" SRC "]]!") \
+    __ASM_EMIT("vld1.32         {d0[], d1[]}, [%[" SRC "]]" INC) \
     __ASM_EMIT("vrecpe.f32      q8, q0")                    /* q8 = s2 */ \
     __ASM_EMIT("vrecps.f32      q12, q8, q0")               /* q12 = (2 - R*s2) */ \
     __ASM_EMIT("vmul.f32        q8, q12, q8")               /* q8 = s2' = s2 * (2 - R*s2) */ \
     __ASM_EMIT("vrecps.f32      q12, q8, q0")               /* q12 = (2 - R*s2') */ \
-    __ASM_EMIT("vmul.f32        q0, q12, q8")               /* q0 = s2" = s2' * (2 - R*s2) = 1/s2 */  \
-    __ASM_EMIT("vmul.f32        q0, q0, q4")                /* s1 / s2 */ \
+    __ASM_EMIT("vmul.f32        q8, q12, q8")               /* q8 = s2" = s2' * (2 - R*s2) = 1/x */  \
+    __ASM_EMIT("vmul.f32        q8, q8, q4")                /* q8 = k/x */ \
+    __ASM_EMIT("vcvt.s32.f32    q8, q8") \
+    __ASM_EMIT("vcvt.f32.s32    q8, q8") \
+    __ASM_EMIT("vmul.f32        q0, q0, q8")                /* q0 = x * int(k/x) */ \
+    __ASM_EMIT("vsub.f32        q0, q4, q0")                /* q0 = k - x * int(k/x) */ \
     __ASM_EMIT("vst1.32         {d0[0]}, [%[" DST "]]!") \
     __ASM_EMIT("subs            %[count], $1") \
     __ASM_EMIT("bge             7b") \
@@ -390,7 +496,39 @@ namespace neon_d32
         ARCH_ARM_ASM
         (
             __ASM_EMIT("vld1.f32    {d8[], d9[]}, [%[k]]")
-            RDIV_K2_CORE("dst")
+            RDIV_KX_CORE("dst", "dst", "")
+            : [dst] "+r" (dst),
+              [count] "+r" (count)
+            : [k] "r" (pk)
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q8", "q9", "q10", "q11", "q12", "q13", "q14", "q15"
+        );
+    }
+
+    void mod_k2(float *dst, float k, size_t count)
+    {
+        IF_ARCH_ARM(float *pk = &k);
+        ARCH_ARM_ASM
+        (
+            __ASM_EMIT("vld1.f32    {d8[], d9[]}, [%[k]]")
+            MOD_KX_CORE("dst", "dst", "")
+            : [dst] "+r" (dst),
+              [count] "+r" (count)
+            : [k] "r" (pk)
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q8", "q9", "q10", "q11", "q12", "q13", "q14", "q15"
+        );
+    }
+
+    void rmod_k2(float *dst, float k, size_t count)
+    {
+        IF_ARCH_ARM(float *pk = &k);
+        ARCH_ARM_ASM
+        (
+            __ASM_EMIT("vld1.f32    {d8[], d9[]}, [%[k]]")
+            RMOD_KX_CORE("dst", "dst", "")
             : [dst] "+r" (dst),
               [count] "+r" (count)
             : [k] "r" (pk)
@@ -491,7 +629,7 @@ namespace neon_d32
         ARCH_ARM_ASM
         (
             __ASM_EMIT("vld1.f32    {d8[], d9[]}, [%[k]]")
-            RDIV_K3_CORE("dst", "src")
+            RDIV_KX_CORE("dst", "src", "!")
             : [dst] "+r" (dst), [src] "+r" (src),
               [count] "+r" (count)
             : [k] "r" (pk)
@@ -501,12 +639,45 @@ namespace neon_d32
         );
     }
 
-#undef OP_DORDER
-#undef OP_RORDER
-#undef OP_K2_CORE
-#undef OP_K3_CORE
-#undef RDIV_K2_CORE
-#undef RDIV_K3_CORE
+    void mod_k3(float *dst, const float *src, float k, size_t count)
+    {
+        IF_ARCH_ARM(float *pk = &k);
+        ARCH_ARM_ASM
+        (
+            __ASM_EMIT("vld1.f32    {d8[], d9[]}, [%[k]]")
+            MOD_KX_CORE("dst", "src", "!")
+            : [dst] "+r" (dst), [src] "+r" (src),
+              [count] "+r" (count)
+            : [k] "r" (pk)
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q8", "q9", "q10", "q11", "q12", "q13", "q14", "q15"
+        );
+    }
+
+    void rmod_k3(float *dst, const float *src, float k, size_t count)
+    {
+        IF_ARCH_ARM(float *pk = &k);
+        ARCH_ARM_ASM
+        (
+            __ASM_EMIT("vld1.f32    {d8[], d9[]}, [%[k]]")
+            RMOD_KX_CORE("dst", "src", "!")
+            : [dst] "+r" (dst), [src] "+r" (src),
+              [count] "+r" (count)
+            : [k] "r" (pk)
+            : "cc", "memory",
+              "q0", "q1", "q2", "q3" , "q4", "q5", "q6", "q7",
+              "q8", "q9", "q10", "q11", "q12", "q13", "q14", "q15"
+        );
+    }
+
+    #undef OP_DORDER
+    #undef OP_RORDER
+    #undef OP_K2_CORE
+    #undef OP_K3_CORE
+    #undef RDIV_KX_CORE
+    #undef MOD_KX_CORE
+    #undef RMOD_KX_CORE
 }
 
 #endif /* DSP_ARCH_ARM_NEON_D32_PMATH_OP_KX_H_ */
