@@ -14,18 +14,20 @@
 
 namespace neon_d32
 {
-    static const float lanczos_2x2[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        -0.1664152316035080f,   // k1
-        +0.0000000000000000f,
-        +0.6203830132406946f,   // k0
+    IF_ARCH_ARM(
+        static const float lanczos_2x2[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            -0.0636843520278618f,   // k1
+            +0.0000000000000000f,
+            +0.5731591682507563f,   // k0
 
-        +1.0000000000000000f,
-        +0.6203830132406946f,   // k0
-        +0.0000000000000000f,
-        -0.1664152316035080f    // k1
-    };
+            +1.0000000000000000f,
+            +0.5731591682507563f,   // k0
+            +0.0000000000000000f,
+            -0.0636843520278618f    // k1
+        };
+    )
 
     void lanczos_resample_2x2(float *dst, const float *src, size_t count)
     {
@@ -44,14 +46,11 @@ namespace neon_d32
             __ASM_EMIT("1:")
             // Even cycle: prepare
             __ASM_EMIT("vld2.32         {q2-q3}, [%[src]]!")        // q2 = s0 s2 s4 s6, q3 = s1 s3 s5 s7
-            __ASM_EMIT("vmov            q4, q2")                    // q4 = s0 s2 s4 s6
-            __ASM_EMIT("vmov            q5, q2")                    // q5 = s0 s2 s4 s6
-            __ASM_EMIT("vmov            q6, q2")                    // q6 = s0 s2 s4 s6
             __ASM_EMIT("vldm            %[dst], {d14-d23}")
-            __ASM_EMIT("vtrn.32         q2, q4")                    // q2 = s0 s0 s4 s4, q4 = s2 s2 s6 s6
-            __ASM_EMIT("vtrn.32         q5, q6")                    // q5 = s0 s0 s4 s4, q6 = s2 s2 s6 s6
-            __ASM_EMIT("vswp            d5, d10")                   // q2 = s0, q4 = s2
-            __ASM_EMIT("vswp            d9, d12")                   // q5 = s4, q6 = s6
+            __ASM_EMIT("vdup.32         q6, d5[1]")
+            __ASM_EMIT("vdup.32         q5, d5[0]")
+            __ASM_EMIT("vdup.32         q4, d4[1]")
+            __ASM_EMIT("vdup.32         q2, d4[0]")
             // Even cycle: convolve
             __ASM_EMIT("vmla.f32        q7, q2, q0")
             __ASM_EMIT("vmla.f32        q8, q4, q0")
@@ -64,14 +63,11 @@ namespace neon_d32
             __ASM_EMIT("vstm            %[dst], {d14-d23}")
             __ASM_EMIT("add             %[dst], $0x08")
             // Odd cycle: prepare
-            __ASM_EMIT("vmov            q4, q3")
-            __ASM_EMIT("vmov            q5, q3")
-            __ASM_EMIT("vmov            q6, q4")
             __ASM_EMIT("vldm            %[dst], {d14-d23}")
-            __ASM_EMIT("vtrn.32         q3, q4")
-            __ASM_EMIT("vtrn.32         q5, q6")
-            __ASM_EMIT("vswp            d7, d10")
-            __ASM_EMIT("vswp            d9, d12")
+            __ASM_EMIT("vdup.32         q6, d7[1]")
+            __ASM_EMIT("vdup.32         q5, d7[0]")
+            __ASM_EMIT("vdup.32         q4, d6[1]")
+            __ASM_EMIT("vdup.32         q3, d6[0]")
             // Odd cycle: convolve
             __ASM_EMIT("vmla.f32        q7, q3, q0")
             __ASM_EMIT("vmla.f32        q8, q4, q0")
@@ -94,14 +90,11 @@ namespace neon_d32
 
             // Prepare
             __ASM_EMIT("vld1.32         {q2}, [%[src]]!")
-            __ASM_EMIT("vmov            q3, q2")
-            __ASM_EMIT("vmov            q4, q2")
-            __ASM_EMIT("vmov            q5, q2")
             __ASM_EMIT("vldm            %[dst], {d14-d19}")
-            __ASM_EMIT("vtrn.32         q2, q3")
-            __ASM_EMIT("vtrn.32         q4, q5")
-            __ASM_EMIT("vswp            d5, d8")
-            __ASM_EMIT("vswp            d7, d10") // q2 = s0, q3 = s1, q4 = s2, q5 = s3
+            __ASM_EMIT("vdup.32         q5, d5[1]") // q5 = s3
+            __ASM_EMIT("vdup.32         q4, d5[0]") // q4 = s2
+            __ASM_EMIT("vdup.32         q3, d4[1]") // q3 = s1
+            __ASM_EMIT("vdup.32         q2, d4[0]") // q2 = s0
             // Even cycle: convolve
             __ASM_EMIT("vmla.f32        q7, q2, q0")
             __ASM_EMIT("vmla.f32        q8, q4, q0")
@@ -143,23 +136,26 @@ namespace neon_d32
         );
     }
 
-    static const float lanczos_2x3[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        +0.0939539981090991f,   // k2
-        +0.0000000000000000f,
-        -0.1910530560835854f,   // k1
+    IF_ARCH_ARM
+    (
+        static const float lanczos_2x3[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            +0.0243170840741611f,   // k2
+            +0.0000000000000000f,
+            -0.1350949115231170f,   // k1
 
-        +0.0000000000000000f,
-        +0.6293724479752082f,   // k0
-        +1.0000000000000000f,
-        +0.6293724479752082f,   // k0
+            +0.0000000000000000f,
+            +0.6079271018540265f,   // k0
+            +1.0000000000000000f,
+            +0.6079271018540265f,   // k0
 
-        +0.0000000000000000f,
-        -0.1910530560835854f,   // k1
-        +0.0000000000000000f,
-        +0.0939539981090991f    // k2
-    };
+            +0.0000000000000000f,
+            -0.1350949115231170f,   // k1
+            +0.0000000000000000f,
+            +0.0243170840741611f    // k2
+        };
+    )
 
     void lanczos_resample_2x3(float *dst, const float *src, size_t count)
     {
@@ -178,14 +174,11 @@ namespace neon_d32
             __ASM_EMIT("1:")
             // Even cycle: prepare
             __ASM_EMIT("vld2.32         {q4-q5}, [%[src]]!")        // q4 = s0 s2 s4 s6, q5 = s1 s3 s5 s7
-            __ASM_EMIT("vmov            q6, q4")                    // q6 = s0 s2 s4 s6
-            __ASM_EMIT("vmov            q7, q4")                    // q7 = s0 s2 s4 s6
-            __ASM_EMIT("vmov            q8, q4")                    // q8 = s0 s2 s4 s6
             __ASM_EMIT("vldm            %[dst], {q10-q15}")
-            __ASM_EMIT("vtrn.32         q4, q6")                    // q4 = s0 s0 s4 s4, q6 = s2 s2 s6 s6
-            __ASM_EMIT("vtrn.32         q7, q8")                    // q7 = s0 s0 s4 s4, q8 = s2 s2 s6 s6
-            __ASM_EMIT("vswp            d9, d14")                   // q4 = s0, q6 = s2
-            __ASM_EMIT("vswp            d13, d16")                  // q7 = s4, q8 = s6
+            __ASM_EMIT("vdup.32         q8, d9[1]")                 // q8 = s6
+            __ASM_EMIT("vdup.32         q7, d9[0]")                 // q7 = s4
+            __ASM_EMIT("vdup.32         q6, d8[1]")                 // q6 = s2
+            __ASM_EMIT("vdup.32         q4, d8[0]")                 // q4 = s0
             // Even cycle: convolve
             __ASM_EMIT("vmla.f32        q10, q4, q0")
             __ASM_EMIT("vmla.f32        q11, q6, q0")
@@ -202,14 +195,12 @@ namespace neon_d32
             __ASM_EMIT("vstm            %[dst], {q10-q15}")
             __ASM_EMIT("add             %[dst], $0x08")
             // Odd cycle: prepare
-            __ASM_EMIT("vmov            q6, q5")                    // q6 = s0 s2 s4 s6
-            __ASM_EMIT("vmov            q7, q5")                    // q7 = s0 s2 s4 s6
-            __ASM_EMIT("vmov            q8, q5")                    // q8 = s0 s2 s4 s6
+
             __ASM_EMIT("vldm            %[dst], {q10-q15}")
-            __ASM_EMIT("vtrn.32         q5, q6")                    // q5 = s0 s0 s4 s4, q6 = s2 s2 s6 s6
-            __ASM_EMIT("vtrn.32         q7, q8")                    // q7 = s0 s0 s4 s4, q8 = s2 s2 s6 s6
-            __ASM_EMIT("vswp            d11, d14")                  // q5 = s0, q6 = s2
-            __ASM_EMIT("vswp            d13, d16")                  // q7 = s4, q8 = s6
+            __ASM_EMIT("vdup.32         q8, d11[1]")                // q8 = s7
+            __ASM_EMIT("vdup.32         q7, d11[0]")                // q7 = s5
+            __ASM_EMIT("vdup.32         q6, d10[1]")                // q6 = s3
+            __ASM_EMIT("vdup.32         q5, d10[0]")                // q5 = s1
             // Odd cycle: convolve
             __ASM_EMIT("vmla.f32        q10, q5, q0")
             __ASM_EMIT("vmla.f32        q11, q6, q0")
@@ -235,14 +226,11 @@ namespace neon_d32
 
             // Prepare
             __ASM_EMIT("vld1.32         {q4}, [%[src]]!")
-            __ASM_EMIT("vmov            q5, q4")
-            __ASM_EMIT("vmov            q6, q4")
-            __ASM_EMIT("vmov            q7, q4")
             __ASM_EMIT("vldm            %[dst], {q10-q13}")
-            __ASM_EMIT("vtrn.32         q4, q5")
-            __ASM_EMIT("vtrn.32         q6, q7")
-            __ASM_EMIT("vswp            d9, d12")
-            __ASM_EMIT("vswp            d11, d14") // q4 = s0, q5 = s1, q6 = s2, q7 = s3
+            __ASM_EMIT("vdup.32         q7, d9[1]")                 // q7 = s3
+            __ASM_EMIT("vdup.32         q6, d9[0]")                 // q6 = s2
+            __ASM_EMIT("vdup.32         q5, d8[1]")                 // q5 = s1
+            __ASM_EMIT("vdup.32         q4, d8[0]")                 // q4 = s0
             // Even cycle: convolve
             __ASM_EMIT("vmla.f32        q10, q4, q0")
             __ASM_EMIT("vmla.f32        q11, q6, q0")
@@ -289,39 +277,41 @@ namespace neon_d32
         );
     }
 
-    static const float lanczos_kernel_3x2[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        -0.1451906347823569f,
-        -0.1903584501504231f,
-        +0.0000000000000000f,
+    IF_ARCH_ARM(
+        static const float lanczos_kernel_3x2[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            -0.0315888188312782f,
+            -0.0854897486982225f,
+            +0.0000000000000000f,
 
-        +0.4051504629060886f,
-        +0.8228011237053413f,
-        +1.0000000000000000f,
-        +0.8228011237053413f,
+            +0.3419589947928900f,
+            +0.7897204707819555f,
+            +1.0000000000000000f,
+            +0.7897204707819555f,
 
-        +0.4051504629060886f,
-        +0.0000000000000000f,
-        -0.1903584501504231f,
-        -0.1451906347823569f,
+            +0.3419589947928900f,
+            +0.0000000000000000f,
+            -0.0854897486982225f,
+            -0.0315888188312782f,
 
-        // Shifted by 1 left
-        -0.1451906347823569f,
-        -0.1903584501504231f,
-        +0.0000000000000000f,
-        +0.4051504629060886f,
+            // Shifted by 1 left
+            -0.0315888188312782f,
+            -0.0854897486982225f,
+            +0.0000000000000000f,
+            +0.3419589947928900f,
 
-        +0.8228011237053413f,
-        +1.0000000000000000f,
-        +0.8228011237053413f,
-        +0.4051504629060886f,
+            +0.7897204707819555f,
+            +1.0000000000000000f,
+            +0.7897204707819555f,
+            +0.3419589947928900f,
 
-        +0.0000000000000000f,
-        -0.1903584501504231f,
-        -0.1451906347823569f,
-        +0.0000000000000000f
-    };
+            +0.0000000000000000f,
+            -0.0854897486982225f,
+            -0.0315888188312782f,
+            +0.0000000000000000f
+        };
+    )
 
     void lanczos_resample_3x2(float *dst, const float *src, size_t count)
     {
@@ -340,10 +330,9 @@ namespace neon_d32
             __ASM_EMIT("1:")
             // Even cycle: prepare
             __ASM_EMIT("vld1.32         {d0}, [%[src]]!")           // d0 = s1 s2
-            __ASM_EMIT("vmov            d1, d0")                    // q0 = s1 s2 s1 s2
             __ASM_EMIT("vldm            %[dst], {q8-q11}")
-            __ASM_EMIT("vmov            q1, q0")                    // q1 = s1 s2 s1 s2
-            __ASM_EMIT("vtrn.32         q0, q1")                    // q0 = s1 s1 s1 s1, q1 = s2 s2 s2 s2
+            __ASM_EMIT("vdup.32         q1, d0[1]")                 // q1 = s2 s2 s2 s2
+            __ASM_EMIT("vdup.32         q0, d0[0]")                 // q0 = s1 s1 s1 s1
             __ASM_EMIT("vmla.f32        q8, q2, q0")
             __ASM_EMIT("vmla.f32        q9, q3, q0")
             __ASM_EMIT("vmla.f32        q10, q4, q0")
@@ -377,61 +366,63 @@ namespace neon_d32
         );
     }
 
-    static const float lanczos_kernel_3x3[] __lsp_aligned16 =
-    {
-        // Middle part
-        +0.8251285571768335f,
-        +1.0000000000000000f,
-        +0.8251285571768335f,
-        +0.4097746665281485f,
+    IF_ARCH_ARM(
+        static const float lanczos_kernel_3x3[] __lsp_aligned16 =
+        {
+            // Middle part
+            +0.8103009258121772f,
+            +1.0000000000000000f,
+            +0.8103009258121772f,
+            +0.3807169003008463f,
 
-        +0.0000000000000000f,
-        -0.1993645686793863f,
-        -0.1562250559899557f,
-        +0.0000000000000000f,
+            +0.0000000000000000f,
+            -0.1458230329384726f,
+            -0.0933267410806225f,
+            +0.0000000000000000f,
 
-        +0.1055060549370832f,
-        +0.0890793429479492f,
-        +0.0000000000000000f,
-        +0.0000000000000000f,
+            +0.0310789306368038f,
+            +0.0126609519658153f,
+            +0.0000000000000000f,
+            +0.0000000000000000f,
 
-        +0.0890793429479492f,
-        +0.1055060549370832f,
-        +0.0000000000000000f,
-        -0.1562250559899557f,
+            +0.0126609519658153f,
+            +0.0310789306368038f,
+            +0.0000000000000000f,
+            -0.0933267410806225f,
 
-        -0.1993645686793863f,
-        +0.0000000000000000f,
-        +0.4097746665281485f,
-        +0.8251285571768335f,
+            -0.1458230329384726f,
+            +0.0000000000000000f,
+            +0.3807169003008463f,
+            +0.8103009258121772f,
 
-        +1.0000000000000000f,
-        +0.8251285571768335f,
-        +0.4097746665281485f,
-        +0.0000000000000000f,
+            +1.0000000000000000f,
+            +0.8103009258121772f,
+            +0.3807169003008463f,
+            +0.0000000000000000f,
 
-        // Head part
-        +0.0000000000000000f,
-        +0.0890793429479492f,
-        +0.1055060549370832f,
-        +0.0000000000000000f,
+            // Head part
+            +0.0000000000000000f,
+            +0.0126609519658153f,
+            +0.0310789306368038f,
+            +0.0000000000000000f,
 
-        -0.1562250559899557f,
-        -0.1993645686793863f,
-        +0.0000000000000000f,
-        +0.4097746665281485f,
+            -0.0933267410806225f,
+            -0.1458230329384726f,
+            +0.0000000000000000f,
+            +0.3807169003008463f,
 
-        // Tail part
-        -0.1993645686793863f,
-        -0.1562250559899557f,
-        +0.0000000000000000f,
-        +0.1055060549370832f,
+            // Tail part
+            -0.1458230329384726f,
+            -0.0933267410806225f,
+            -0.0000000000000000f,
+            +0.0310789306368038f,
 
-        +0.0890793429479492f,
-        +0.0000000000000000f,
-        +0.0000000000000000f,
-        +0.0000000000000000f
-    };
+            +0.0126609519658153f,
+            +0.0000000000000000f,
+            +0.0000000000000000f,
+            +0.0000000000000000f
+        };
+    )
 
     void lanczos_resample_3x3(float *dst, const float *src, size_t count)
     {
@@ -449,11 +440,10 @@ namespace neon_d32
             // 2x blocks
             __ASM_EMIT("1:")
             __ASM_EMIT("vld1.32         {d0}, [%[src]]!")           // d0 = s1 s2
-            __ASM_EMIT("vmov            d1, d0")                    // q0 = s1 s2 s1 s2
             __ASM_EMIT("vldm            %[dst], {q2-q7}")
-            __ASM_EMIT("vmov            q1, q0")                    // q1 = s1 s2 s1 s2
+            __ASM_EMIT("vdup.32         q1, d0[1]")                 // q1 = s2 s2 s2 s2
+            __ASM_EMIT("vdup.32         q0, d0[0]")                 // q0 = s1 s1 s1 s1
             __ASM_EMIT("vld1.32         {q8-q9}, [%[kernel]]!")
-            __ASM_EMIT("vtrn.32         q0, q1")                    // q0 = s1 s1 s1 s1, q1 = s2 s2 s2 s2
 
             __ASM_EMIT("vmla.f32        q2, q8, q0")
             __ASM_EMIT("vmla.f32        q3, q9, q0")
@@ -498,28 +488,30 @@ namespace neon_d32
         );
     }
 
-    static const float lanczos_kernel_4x2[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        -0.1187292496637064f,
-        -0.2001458972657618f,
-        -0.1729186314209981f,
+    IF_ARCH_ARM(
+        static const float lanczos_kernel_4x2[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            -0.0179051851263444f,
+            -0.0636843520278618f,
+            -0.0847248039068907f,
 
-        +0.0000000000000000f,
-        +0.2957854651930789f,
-        +0.6325370350028462f,
-        +0.8988707620097378f,
+            +0.0000000000000000f,
+            +0.2353466775191407f,
+            +0.5731591682507563f,
+            +0.8773540711908775f,
 
-        +1.0000000000000000f,
-        +0.8988707620097378f,
-        +0.6325370350028462f,
-        +0.2957854651930789f,
+            +1.0000000000000000f,
+            +0.8773540711908775f,
+            +0.5731591682507563f,
+            +0.2353466775191407f,
 
-        +0.0000000000000000f,
-        -0.1729186314209981f,
-        -0.2001458972657618f,
-        -0.1187292496637064f
-    };
+            +0.0000000000000000f,
+            -0.0847248039068907f,
+            -0.0636843520278618f,
+            -0.0179051851263444f
+        };
+    )
 
     void lanczos_resample_4x2(float *dst, const float *src, size_t count)
     {
@@ -530,21 +522,18 @@ namespace neon_d32
         ARCH_ARM_ASM
         (
             // Prepare
-            __ASM_EMIT("vldm            %[kernel], {d24-d31}")
+            __ASM_EMIT("vldm            %[kernel], {q12-q15}")
             __ASM_EMIT("subs            %[count], $4")
             __ASM_EMIT("blo             2f")
 
             // 4x blocks
             __ASM_EMIT("1:")
-            __ASM_EMIT("vldm            %[src]!, {d0-d1}")
-            __ASM_EMIT("vmov            q1, q0")
-            __ASM_EMIT("vldm            %[dst], {d8-d21}")
-            __ASM_EMIT("vmov            q2, q0")
-            __ASM_EMIT("vmov            q3, q1")
-            __ASM_EMIT("vtrn.32         q0, q1")
-            __ASM_EMIT("vtrn.32         q2, q3")
-            __ASM_EMIT("vswp            d1, d4")
-            __ASM_EMIT("vswp            d3, d6")
+            __ASM_EMIT("vldm            %[src]!, {q0}")
+            __ASM_EMIT("vldm            %[dst], {q4-q10}")
+            __ASM_EMIT("vdup.32         q3, d1[1]")
+            __ASM_EMIT("vdup.32         q2, d1[0]")
+            __ASM_EMIT("vdup.32         q1, d0[1]")
+            __ASM_EMIT("vdup.32         q0, d0[0]")
 
             __ASM_EMIT("vmla.f32        q4, q12, q0")
             __ASM_EMIT("vmla.f32        q5, q13, q0")
@@ -562,7 +551,7 @@ namespace neon_d32
             __ASM_EMIT("vmla.f32        q8, q13, q3")
             __ASM_EMIT("vmla.f32        q9, q14, q3")
             __ASM_EMIT("vmla.f32        q10, q15, q3")
-            __ASM_EMIT("vstm            %[dst], {d8-d21}")
+            __ASM_EMIT("vstm            %[dst], {q4-q10}")
             __ASM_EMIT("subs            %[count], $4")
             __ASM_EMIT("add             %[dst], $0x40")
             __ASM_EMIT("bhs             1b")
@@ -573,10 +562,9 @@ namespace neon_d32
             __ASM_EMIT("blt             4f")
 
             __ASM_EMIT("vldm            %[src]!, {d0}")
-            __ASM_EMIT("vmov            d1, d0")
-            __ASM_EMIT("vldm            %[dst], {d8-d17}")
-            __ASM_EMIT("vmov            q1, q0")
-            __ASM_EMIT("vtrn.32         q0, q1")
+            __ASM_EMIT("vldm            %[dst], {q4-q8}")
+            __ASM_EMIT("vdup.32         q1, d0[1]")
+            __ASM_EMIT("vdup.32         q0, d0[0]")
 
             __ASM_EMIT("vmla.f32        q4, q12, q0")
             __ASM_EMIT("vmla.f32        q5, q13, q0")
@@ -586,7 +574,7 @@ namespace neon_d32
             __ASM_EMIT("vmla.f32        q6, q13, q1")
             __ASM_EMIT("vmla.f32        q7, q14, q1")
             __ASM_EMIT("vmla.f32        q8, q15, q1")
-            __ASM_EMIT("vstm            %[dst], {d8-d17}")
+            __ASM_EMIT("vstm            %[dst], {q4-q8}")
             __ASM_EMIT("sub             %[count], $2")
             __ASM_EMIT("add             %[dst], $0x20")
 
@@ -613,38 +601,40 @@ namespace neon_d32
         );
     }
 
-    static const float lanczos_kernel_4x3[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        +0.0749573750027847f,
-        +0.1184264228024920f,
-        +0.0943496807888575f,
+    IF_ARCH_ARM(
+        static const float lanczos_kernel_4x3[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            +0.0073559260471942f,
+            +0.0243170840741611f,
+            +0.0300210914495816f,
 
-        +0.0000000000000000f,
-        -0.1241641556633335f,
-        -0.2067943377468982f,
-        -0.1768665375130828f,
+            +0.0000000000000000f,
+            -0.0677913359005429f,
+            -0.1350949115231170f,
+            -0.1328710183650640f,
 
-        +0.0000000000000000f,
-        +0.2981808178680967f,
-        +0.6348032780190205f,
-        +0.8996736756130924f,
+            +0.0000000000000000f,
+            +0.2701898230462341f,
+            +0.6079271018540265f,
+            +0.8900670517104946f,
 
-        +1.0000000000000000f,
-        +0.8996736756130924f,
-        +0.6348032780190205f,
-        +0.2981808178680967f,
+            +1.0000000000000000f,
+            +0.8900670517104946f,
+            +0.6079271018540265f,
+            +0.2701898230462341f,
 
-        +0.0000000000000000f,
-        -0.1768665375130828f,
-        -0.2067943377468982f,
-        -0.1241641556633335f,
+            +0.0000000000000000f,
+            -0.1328710183650640f,
+            -0.1350949115231170f,
+            -0.0677913359005429f,
 
-        +0.0000000000000000f,
-        +0.0943496807888575f,
-        +0.1184264228024920f,
-        +0.0749573750027847f,
-    };
+            +0.0000000000000000f,
+            +0.0300210914495816f,
+            +0.0243170840741611f,
+            +0.0073559260471942f
+        };
+    )
 
     void lanczos_resample_4x3(float *dst, const float *src, size_t count)
     {
@@ -662,13 +652,10 @@ namespace neon_d32
             // 4x blocks
             __ASM_EMIT("1:")
             __ASM_EMIT("vldm            %[src]!, {q0}")
-            __ASM_EMIT("vmov            q1, q0")
-            __ASM_EMIT("vmov            q2, q0")
-            __ASM_EMIT("vmov            q3, q1")
-            __ASM_EMIT("vtrn.32         q0, q1")
-            __ASM_EMIT("vtrn.32         q2, q3")
-            __ASM_EMIT("vswp            d1, d4")
-            __ASM_EMIT("vswp            d3, d6")
+            __ASM_EMIT("vdup.32         q3, d1[1]")
+            __ASM_EMIT("vdup.32         q2, d1[0]")
+            __ASM_EMIT("vdup.32         q1, d0[1]")
+            __ASM_EMIT("vdup.32         q0, d0[0]")
 
             // Part 1 - process 10 multiplications
             __ASM_EMIT("vldm            %[dst], {q4-q7}")
@@ -713,9 +700,8 @@ namespace neon_d32
             __ASM_EMIT("blt             4f")
 
             __ASM_EMIT("vldm            %[src]!, {d0}")
-            __ASM_EMIT("vmov            d1, d0")
-            __ASM_EMIT("vmov            q1, q0")
-            __ASM_EMIT("vtrn.32         q0, q1")
+            __ASM_EMIT("vdup.32         q1, d0[1]")
+            __ASM_EMIT("vdup.32         q0, d0[0]")
 
             // Part 1 - 7 multiplications
             __ASM_EMIT("vldm            %[dst], {q4-q7}")
@@ -764,38 +750,40 @@ namespace neon_d32
         );
     }
 
-    static const float lanczos_kernel_6x2[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        -0.0835168749111343f,
-        -0.1602001404590001f,
-        -0.2067943377468982f,
+    IF_ARCH_ARM(
+        static const float lanczos_kernel_6x2[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            -0.0078021377848166f,
+            -0.0315888188312782f,
+            -0.0636843520278618f,
 
-        -0.2025752314530442f,
-        -0.1343073344351001f,
-        +0.0000000000000000f,
-        +0.1894744904080251f,
+            -0.0854897486982225f,
+            -0.0719035699814534f,
+            +0.0000000000000000f,
+            +0.1409309971636486f,
 
-        +0.4114005618526707f,
-        +0.6348032780190205f,
-        +0.8259440903283795f,
-        +0.9546266788436470f,
+            +0.3419589947928900f,
+            +0.5731591682507563f,
+            +0.7897204707819555f,
+            +0.9440586719628122f,
 
-        +1.0000000000000000f,
-        +0.9546266788436470f,
-        +0.8259440903283795f,
-        +0.6348032780190205f,
+            +1.0000000000000000f,
+            +0.9440586719628122f,
+            +0.7897204707819555f,
+            +0.5731591682507563f,
 
-        +0.4114005618526707f,
-        +0.1894744904080251f,
-        +0.0000000000000000f,
-        -0.1343073344351001f,
+            +0.3419589947928900f,
+            +0.1409309971636486f,
+            +0.0000000000000000f,
+            -0.0719035699814534f,
 
-        -0.2025752314530442f,
-        -0.2067943377468982f,
-        -0.1602001404590001f,
-        -0.0835168749111343f
-    };
+            -0.0854897486982225f,
+            -0.0636843520278618f,
+            -0.0315888188312782f,
+            -0.0078021377848166f
+        };
+    )
 
     void lanczos_resample_6x2(float *dst, const float *src, size_t count)
     {
@@ -810,8 +798,7 @@ namespace neon_d32
             __ASM_EMIT("vldm            %[kernel], {q8-q13}")
 
             __ASM_EMIT("1:")
-            __ASM_EMIT("vldm            %[src]!, {s0}")
-            __ASM_EMIT("vdup.32         q0, d0[0]")
+            __ASM_EMIT("vld1.32         {d0[], d1[]}, [%[src]]!")
             __ASM_EMIT("vldm            %[dst], {q2-q7}")
             __ASM_EMIT("vmla.f32        q2, q8, q0")
             __ASM_EMIT("vmla.f32        q3, q9, q0")
@@ -834,53 +821,55 @@ namespace neon_d32
         );
     }
 
-    static const float lanczos_kernel_6x3[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        +0.0539107649662794f,
-        +0.0996822843396933f,
-        +0.1233221256242928f,
+    IF_ARCH_ARM(
+        static const float lanczos_kernel_6x3[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            +0.0032875048460955f,
+            +0.0126609519658153f,
+            +0.0243170840741611f,
 
-        +0.1149032936217488f,
-        +0.0717178866781924f,
-        -0.0000000000000000f,
-        -0.0853379770956540f,
+            +0.0310789306368038f,
+            +0.0248005479513036f,
+            -0.0000000000000000f,
+            -0.0424907562338176f,
 
-        -0.1630759564500400f,
-        -0.2097908159917360f,
-        -0.2048873332640742f,
-        -0.1354777817419537f,
+            -0.0933267410806225f,
+            -0.1350949115231170f,
+            -0.1458230329384726f,
+            -0.1049261531488149f,
 
-        +0.0000000000000000f,
-        +0.1903132912211352f,
-        +0.4125642785884168f,
-        +0.6358120571822271f,
+            +0.0000000000000000f,
+            +0.1676517041508127f,
+            +0.3807169003008463f,
+            +0.6079271018540265f,
 
-        +0.8265269098929101f,
-        +0.9547949937819170f,
-        +1.0000000000000000f,
-        +0.9547949937819170f,
+            +0.8103009258121772f,
+            +0.9500889005216107f,
+            +1.0000000000000000f,
+            +0.9500889005216107f,
 
-        +0.8265269098929101f,
-        +0.6358120571822271f,
-        +0.4125642785884168f,
-        +0.1903132912211352f,
+            +0.8103009258121772f,
+            +0.6079271018540265f,
+            +0.3807169003008463f,
+            +0.1676517041508127f,
 
-        +0.0000000000000000f,
-        -0.1354777817419537f,
-        -0.2048873332640742f,
-        -0.2097908159917360f,
+            +0.0000000000000000f,
+            -0.1049261531488149f,
+            -0.1458230329384726f,
+            -0.1350949115231170f,
 
-        -0.1630759564500400f,
-        -0.0853379770956540f,
-        -0.0000000000000000f,
-        +0.0717178866781924f,
+            -0.0933267410806225f,
+            -0.0424907562338176f,
+            -0.0000000000000000f,
+            +0.0248005479513036f,
 
-        +0.1149032936217488f,
-        +0.1233221256242928f,
-        +0.0996822843396933f,
-        +0.0539107649662794f
-    };
+            +0.0310789306368038f,
+            +0.0243170840741611f,
+            +0.0126609519658153f,
+            +0.0032875048460955f
+        };
+    )
 
     void lanczos_resample_6x3(float *dst, const float *src, size_t count)
     {
@@ -896,9 +885,9 @@ namespace neon_d32
             __ASM_EMIT("add             %[k1], %[k0], $0x30")
             __ASM_EMIT("vldm            %[k1], {q11-q13}")
             __ASM_EMIT("add             %[k1], $0x30")
+            // 1x blocks
             __ASM_EMIT("1:")
-            __ASM_EMIT("vldm            %[src]!, {s0}")
-            __ASM_EMIT("vdup.32         q0, d0[0]")
+            __ASM_EMIT("vld1.32         {d0[], d1[]}, [%[src]]!")
             __ASM_EMIT("vldm            %[k0], {q8-q10}")
             __ASM_EMIT("vldm            %[d0], {q2-q7}")
             __ASM_EMIT("add             %[d1], %[d0], $0x60")
@@ -930,48 +919,50 @@ namespace neon_d32
         );
     }
 
-    static const float lanczos_kernel_8x2[] __lsp_aligned16 =
-    {
-        -0.0000000000000000f,
-        -0.0635086982303136f,
-        -0.1261005767591211f,
-        -0.1779172640206227f,
+    IF_ARCH_ARM(
+        static const float lanczos_kernel_8x2[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            -0.0043033145538298f,
+            -0.0179051851263444f,
+            -0.0393892611124141f,
 
-        -0.2091519082144436f,
-        -0.2112875836092082f,
-        -0.1782608868029105f,
-        -0.1073988642554405f,
+            -0.0636843520278618f,
+            -0.0823353965569232f,
+            -0.0847248039068907f,
+            -0.0600950644541902f,
 
-        +0.0000000000000000f,
-        +0.1385297654836717f,
-        +0.2990219260297742f,
-        +0.4693478599070060f,
+            +0.0000000000000000f,
+            +0.0993408208324369f,
+            +0.2353466775191407f,
+            +0.3985033193355084f,
 
-        +0.6355976112274050f,
-        +0.7835048889119777f,
-        +0.8999547969760605f,
-        +0.9743975231790596f,
+            +0.5731591682507563f,
+            +0.7396427919997760f,
+            +0.8773540711908775f,
+            +0.9682457746117045f,
 
-        +1.0000000000000000f,
-        +0.9743975231790596f,
-        +0.8999547969760605f,
-        +0.7835048889119777f,
+            +1.0000000000000000f,
+            +0.9682457746117045f,
+            +0.8773540711908775f,
+            +0.7396427919997760f,
 
-        +0.6355976112274050f,
-        +0.4693478599070060f,
-        +0.2990219260297742f,
-        +0.1385297654836717f,
+            +0.5731591682507563f,
+            +0.3985033193355084f,
+            +0.2353466775191407f,
+            +0.0993408208324369f,
 
-        +0.0000000000000000f,
-        -0.1073988642554405f,
-        -0.1782608868029105f,
-        -0.2112875836092082f,
+            +0.0000000000000000f,
+            -0.0600950644541902f,
+            -0.0847248039068907f,
+            -0.0823353965569232f,
 
-        -0.2091519082144436f,
-        -0.1779172640206227f,
-        -0.1261005767591211f,
-        -0.0635086982303136f
-    };
+            -0.0636843520278618f,
+            -0.0393892611124141f,
+            -0.0179051851263444f,
+            -0.0043033145538298f
+        };
+    )
 
     void lanczos_resample_8x2(float *dst, const float *src, size_t count)
     {
@@ -989,16 +980,13 @@ namespace neon_d32
             // x4 blocks
             __ASM_EMIT("1:")
             __ASM_EMIT("vld1.32         {q0}, [%[src]]!")
-            __ASM_EMIT("vmov            q1, q0")
             __ASM_EMIT("add             %[d1], %[d0], $0x40")
-            __ASM_EMIT("vmov            q2, q0")
             __ASM_EMIT("add             %[d2], %[d0], $0x80")
-            __ASM_EMIT("vmov            q3, q1")
             __ASM_EMIT("add             %[d3], %[d0], $0xc0")
-            __ASM_EMIT("vtrn.32         q0, q1")
-            __ASM_EMIT("vtrn.32         q2, q3")
-            __ASM_EMIT("vswp            d1, d4")
-            __ASM_EMIT("vswp            d3, d6") // q2 = s0, q3 = s1, q4 = s2, q5 = s3
+            __ASM_EMIT("vdup.32         q3, d1[1]")
+            __ASM_EMIT("vdup.32         q2, d1[0]")
+            __ASM_EMIT("vdup.32         q1, d0[1]")
+            __ASM_EMIT("vdup.32         q0, d0[0]")
             // Part 1
             __ASM_EMIT("vldm            %[d0], {q4-q7}")
             __ASM_EMIT("vmla.f32        q4, q8, q0")
@@ -1053,10 +1041,9 @@ namespace neon_d32
             __ASM_EMIT("blt             4f")
 
             __ASM_EMIT("vldm            %[src]!, {d0}")
-            __ASM_EMIT("vmov            d1, d0")
             __ASM_EMIT("add             %[d1], %[d0], $0x50")
-            __ASM_EMIT("vmov            q1, q0")
-            __ASM_EMIT("vtrn.32         q0, q1")
+            __ASM_EMIT("vdup.32         q1, d0[1]")
+            __ASM_EMIT("vdup.32         q0, d0[0]")
             // Part 1
             __ASM_EMIT("vldm            %[d0], {q2-q6}")
             __ASM_EMIT("vmla.f32        q2, q8, q0")
@@ -1115,68 +1102,70 @@ namespace neon_d32
         );
     }
 
-    static const float lanczos_kernel_8x3[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        +0.0413762983728270f,
-        +0.0800907151667399f,
-        +0.1098388956286788f,
+    IF_ARCH_ARM(
+        static const float lanczos_kernel_8x3[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            +0.0018368899607481f,
+            +0.0073559260471942f,
+            +0.0155961678435580f,
 
-        +0.1250635280404858f,
-        +0.1218381732799711f,
-        +0.0985951550643596f,
-        +0.0565868891962354f,
+            +0.0243170840741611f,
+            +0.0303079634725070f,
+            +0.0300210914495816f,
+            +0.0204366616947175f,
 
-        -0.0000000000000000f,
-        -0.0643160644975587f,
-        -0.1274947024165923f,
-        -0.1796106608775988f,
+            +0.0000000000000000f,
+            -0.0305684889733737f,
+            -0.0677913359005429f,
+            -0.1054383717904384f,
 
-        -0.2108456783342821f,
-        -0.2127234580145891f,
-        -0.1792608653618989f,
-        -0.1078863334836789f,
+            -0.1350949115231170f,
+            -0.1472651639056537f,
+            -0.1328710183650640f,
+            -0.0849124693704824f,
 
-        +0.0000000000000000f,
-        +0.1389094368005209f,
-        +0.2996235873365793f,
-        +0.4700032639087741f,
+            +0.0000000000000000f,
+            +0.1205345965259870f,
+            +0.2701898230462341f,
+            +0.4376469925430009f,
 
-        +0.6361653568810439f,
-        +0.7838984051945062f,
-        +0.9001556302107272f,
-        +0.9744518753544421f,
+            +0.6079271018540265f,
+            +0.7642122243343417f,
+            +0.8900670517104946f,
+            +0.9717147892357163f,
 
-        +1.0000000000000000f,
-        +0.9744518753544421f,
-        +0.9001556302107272f,
-        +0.7838984051945062f,
+            +1.0000000000000000f,
+            +0.9717147892357163f,
+            +0.8900670517104946f,
+            +0.7642122243343417f,
 
-        +0.6361653568810439f,
-        +0.4700032639087741f,
-        +0.2996235873365793f,
-        +0.1389094368005209f,
+            +0.6079271018540265f,
+            +0.4376469925430009f,
+            +0.2701898230462341f,
+            +0.1205345965259870f,
 
-        +0.0000000000000000f,
-        -0.1078863334836789f,
-        -0.1792608653618989f,
-        -0.2127234580145891f,
+            +0.0000000000000000f,
+            -0.0849124693704824f,
+            -0.1328710183650640f,
+            -0.1472651639056537f,
 
-        -0.2108456783342821f,
-        -0.1796106608775988f,
-        -0.1274947024165923f,
-        -0.0643160644975587f,
+            -0.1350949115231170f,
+            -0.1054383717904384f,
+            -0.0677913359005429f,
+            -0.0305684889733737f,
 
-        -0.0000000000000000f,
-        +0.0565868891962354f,
-        +0.0985951550643596f,
-        +0.1218381732799711f,
+            +0.0000000000000000f,
+            +0.0204366616947175f,
+            +0.0300210914495816f,
+            +0.0303079634725070f,
 
-        +0.1250635280404858f,
-        +0.1098388956286788f,
-        +0.0800907151667399f,
-        +0.0413762983728270f
-    };
+            +0.0243170840741611f,
+            +0.0155961678435580f,
+            +0.0073559260471942f,
+            +0.0018368899607481f
+        };
+    )
 
     void lanczos_resample_8x3(float *dst, const float *src, size_t count)
     {
@@ -1193,17 +1182,14 @@ namespace neon_d32
             // x4 blocks
             __ASM_EMIT("1:")
             __ASM_EMIT("vld1.32         {q0}, [%[src]]!")
-            __ASM_EMIT("vmov            q1, q0")
             __ASM_EMIT("vldm            %[k0], {q10-q15}")
-            __ASM_EMIT("vmov            q2, q0")
             __ASM_EMIT("add             %[k1], %[k0], $0x60")
-            __ASM_EMIT("vmov            q3, q1")
             __ASM_EMIT("add             %[d1], %[d0], $0x60")
             __ASM_EMIT("add             %[d2], %[d0], $0xc0")
-            __ASM_EMIT("vtrn.32         q0, q1")
-            __ASM_EMIT("vtrn.32         q2, q3")
-            __ASM_EMIT("vswp            d1, d4")
-            __ASM_EMIT("vswp            d3, d6") // q2 = s0, q3 = s1, q4 = s2, q5 = s3
+            __ASM_EMIT("vdup.32         q3, d1[1]")
+            __ASM_EMIT("vdup.32         q2, d1[0]")
+            __ASM_EMIT("vdup.32         q1, d0[1]")
+            __ASM_EMIT("vdup.32         q0, d0[0]")
 
             // Part 1
             __ASM_EMIT("vldm            %[d0], {q4-q9}")
@@ -1274,12 +1260,11 @@ namespace neon_d32
             __ASM_EMIT("blt             4f")
 
             __ASM_EMIT("vldm            %[src]!, {d0}")
-            __ASM_EMIT("vmov            d1, d0")
             __ASM_EMIT("add             %[d1], %[d0], $0x70")
-            __ASM_EMIT("vmov            q1, q0")
             __ASM_EMIT("vldm            %[k0], {q9-q15}")
-            __ASM_EMIT("vtrn.32         q0, q1")
             __ASM_EMIT("add             %[k1], %[k0], $0x50")
+            __ASM_EMIT("vdup.32         q1, d0[1]")
+            __ASM_EMIT("vdup.32         q0, d0[0]")
             // Part 1
             __ASM_EMIT("vldm            %[d0], {q2-q8}")
             __ASM_EMIT("vmla.f32        q2, q9, q0")
