@@ -432,6 +432,219 @@ namespace avx
 
     #undef FMOP_VV_CORE
 
+    #define FMA_OFF(a, b)   a
+    #define FMA_ON(a, b)    b
+
+    #define FMOD_VV_CORE(DST, SRC1, SRC2, SRC3, OP_SEL, FMA_SEL) \
+        __ASM_EMIT("xor                 %[off], %[off]") \
+        /* x16 blocks */ \
+        __ASM_EMIT32("subl              $16, %[count]") \
+        __ASM_EMIT64("sub               $16, %[count]") \
+        __ASM_EMIT("jb                  2f") \
+        __ASM_EMIT("1:") \
+        __ASM_EMIT("vmovups             0x00(%[" SRC3 "], %[off]), %%ymm0") \
+        __ASM_EMIT("vmovups             0x20(%[" SRC3 "], %[off]), %%ymm1") \
+        __ASM_EMIT("vmovups             0x00(%[" SRC1 "], %[off]), " OP_SEL("%%ymm2", "%%ymm6")) \
+        __ASM_EMIT("vmovups             0x20(%[" SRC1 "], %[off]), " OP_SEL("%%ymm3", "%%ymm7")) \
+        __ASM_EMIT("vmulps              0x00(%[" SRC2 "], %[off]), %%ymm0, " OP_SEL("%%ymm6", "%%ymm2")) \
+        __ASM_EMIT("vmulps              0x20(%[" SRC2 "], %[off]), %%ymm1, " OP_SEL("%%ymm7", "%%ymm3")) \
+        __ASM_EMIT("vdivps              %%ymm6, %%ymm2, %%ymm4") \
+        __ASM_EMIT("vdivps              %%ymm7, %%ymm3, %%ymm5") \
+        __ASM_EMIT("vcvttps2dq          %%ymm4, %%ymm4") \
+        __ASM_EMIT("vcvttps2dq          %%ymm5, %%ymm5") \
+        __ASM_EMIT("vcvtdq2ps           %%ymm4, %%ymm4") \
+        __ASM_EMIT("vcvtdq2ps           %%ymm5, %%ymm5") \
+        __ASM_EMIT(FMA_SEL("vmulps      %%ymm6, %%ymm4, %%ymm4", "")) \
+        __ASM_EMIT(FMA_SEL("vmulps      %%ymm7, %%ymm5, %%ymm5", "")) \
+        __ASM_EMIT(FMA_SEL("vsubps      %%ymm4, %%ymm2, %%ymm2", "vfnmadd231ps  %%ymm6, %%ymm4, %%ymm2")) \
+        __ASM_EMIT(FMA_SEL("vsubps      %%ymm5, %%ymm3, %%ymm3", "vfnmadd231ps  %%ymm7, %%ymm5, %%ymm3")) \
+        __ASM_EMIT("vmovups             %%ymm2, 0x00(%[" DST "], %[off])") \
+        __ASM_EMIT("vmovups             %%ymm3, 0x20(%[" DST "], %[off])") \
+        __ASM_EMIT("add                 $0x40, %[off]") \
+        __ASM_EMIT32("subl              $16, %[count]") \
+        __ASM_EMIT64("sub               $16, %[count]") \
+        __ASM_EMIT("jae                 1b") \
+        __ASM_EMIT("2:") \
+        /* x8 block */ \
+        __ASM_EMIT32("addl              $8, %[count]") \
+        __ASM_EMIT64("add               $8, %[count]") \
+        __ASM_EMIT("jl                  4f") \
+        __ASM_EMIT("vmovups             0x00(%[" SRC3 "], %[off]), %%xmm0") \
+        __ASM_EMIT("vmovups             0x10(%[" SRC3 "], %[off]), %%xmm1") \
+        __ASM_EMIT("vmovups             0x00(%[" SRC1 "], %[off]), " OP_SEL("%%xmm2", "%%xmm6")) \
+        __ASM_EMIT("vmovups             0x10(%[" SRC1 "], %[off]), " OP_SEL("%%xmm3", "%%xmm7")) \
+        __ASM_EMIT("vmulps              0x00(%[" SRC2 "], %[off]), %%xmm0, " OP_SEL("%%xmm6", "%%xmm2")) \
+        __ASM_EMIT("vmulps              0x10(%[" SRC2 "], %[off]), %%xmm1, " OP_SEL("%%xmm7", "%%xmm3")) \
+        __ASM_EMIT("vdivps              %%xmm6, %%xmm2, %%xmm4") \
+        __ASM_EMIT("vdivps              %%xmm7, %%xmm3, %%xmm5") \
+        __ASM_EMIT("vcvttps2dq          %%xmm4, %%xmm4") \
+        __ASM_EMIT("vcvttps2dq          %%xmm5, %%xmm5") \
+        __ASM_EMIT("vcvtdq2ps           %%xmm4, %%xmm4") \
+        __ASM_EMIT("vcvtdq2ps           %%xmm5, %%xmm5") \
+        __ASM_EMIT(FMA_SEL("vmulps      %%xmm6, %%xmm4, %%xmm4", "")) \
+        __ASM_EMIT(FMA_SEL("vmulps      %%xmm7, %%xmm5, %%xmm5", "")) \
+        __ASM_EMIT(FMA_SEL("vsubps      %%xmm4, %%xmm2, %%xmm2", "vfnmadd231ps  %%xmm6, %%xmm4, %%xmm2")) \
+        __ASM_EMIT(FMA_SEL("vsubps      %%xmm5, %%xmm3, %%xmm3", "vfnmadd231ps  %%xmm7, %%xmm5, %%xmm3")) \
+        __ASM_EMIT("vmovups             %%xmm2, 0x00(%[" DST "], %[off])") \
+        __ASM_EMIT("vmovups             %%xmm3, 0x10(%[" DST "], %[off])") \
+        __ASM_EMIT32("subl              $8, %[count]") \
+        __ASM_EMIT64("sub               $8, %[count]") \
+        __ASM_EMIT("add                 $0x20, %[off]") \
+        __ASM_EMIT("4:") \
+        /* x4 block */ \
+        __ASM_EMIT32("addl              $4, %[count]") \
+        __ASM_EMIT64("add               $4, %[count]") \
+        __ASM_EMIT("jl                  6f") \
+        __ASM_EMIT("vmovups             0x00(%[" SRC3 "], %[off]), %%xmm0") \
+        __ASM_EMIT("vmovups             0x00(%[" SRC1 "], %[off]), " OP_SEL("%%xmm2", "%%xmm6")) \
+        __ASM_EMIT("vmulps              0x00(%[" SRC2 "], %[off]), %%xmm0, " OP_SEL("%%xmm6", "%%xmm2")) \
+        __ASM_EMIT("vdivps              %%xmm6, %%xmm2, %%xmm4") \
+        __ASM_EMIT("vcvttps2dq          %%xmm4, %%xmm4") \
+        __ASM_EMIT("vcvtdq2ps           %%xmm4, %%xmm4") \
+        __ASM_EMIT(FMA_SEL("vmulps      %%xmm6, %%xmm4, %%xmm4", "")) \
+        __ASM_EMIT(FMA_SEL("vsubps      %%xmm4, %%xmm2, %%xmm2", "vfnmadd231ps  %%xmm6, %%xmm4, %%xmm2")) \
+        __ASM_EMIT("vmovups             %%xmm2, 0x00(%[" DST "], %[off])") \
+        __ASM_EMIT32("subl              $4, %[count]") \
+        __ASM_EMIT64("sub               $4, %[count]") \
+        __ASM_EMIT("add                 $0x10, %[off]") \
+        __ASM_EMIT("6:") \
+        /* x1 blocks */ \
+        __ASM_EMIT32("addl              $3, %[count]") \
+        __ASM_EMIT64("add               $3, %[count]") \
+        __ASM_EMIT("jl                  8f") \
+        __ASM_EMIT("7:") \
+        __ASM_EMIT("vmovss              0x00(%[" SRC3 "], %[off]), %%xmm0") \
+        __ASM_EMIT("vmovss              0x00(%[" SRC1 "], %[off]), " OP_SEL("%%xmm2", "%%xmm6")) \
+        __ASM_EMIT("vmulss              0x00(%[" SRC2 "], %[off]), %%xmm0, " OP_SEL("%%xmm6", "%%xmm2")) \
+        __ASM_EMIT("vdivss              %%xmm6, %%xmm2, %%xmm4") \
+        __ASM_EMIT("vcvttps2dq          %%xmm4, %%xmm4") \
+        __ASM_EMIT("vcvtdq2ps           %%xmm4, %%xmm4") \
+        __ASM_EMIT(FMA_SEL("vmulss      %%xmm6, %%xmm4, %%xmm4", "")) \
+        __ASM_EMIT(FMA_SEL("vsubss      %%xmm4, %%xmm2, %%xmm2", "vfnmadd231ss  %%xmm6, %%xmm4, %%xmm2")) \
+        __ASM_EMIT("vmovss              %%xmm2, 0x00(%[" DST "], %[off])") \
+        __ASM_EMIT("add                 $0x04, %[off]") \
+        __ASM_EMIT32("decl              %[count]") \
+        __ASM_EMIT64("dec               %[count]") \
+        __ASM_EMIT("jge                 7b") \
+        __ASM_EMIT("8:")
+
+    void fmmod3(float *dst, const float *a, const float *b, size_t count)
+    {
+        IF_ARCH_X86(size_t off);
+        ARCH_X86_ASM
+        (
+            FMOD_VV_CORE("dst", "dst", "a", "b", OP_DSEL, FMA_OFF)
+            : [off] "=&r" (off), [count] "+r" (count)
+            : [dst] "r"(dst), [a] "r" (a), [b] "r" (b)
+            : "cc", "memory",
+              "%xmm0", "%xmm1", "%xmm2", "%xmm3",
+              "%xmm4", "%xmm5", "%xmm6", "%xmm7"
+        );
+    }
+
+    void fmmod3_fma3(float *dst, const float *a, const float *b, size_t count)
+    {
+        IF_ARCH_X86(size_t off);
+        ARCH_X86_ASM
+        (
+            FMOD_VV_CORE("dst", "dst", "a", "b", OP_DSEL, FMA_OFF)
+            : [off] "=&r" (off), [count] "+r" (count)
+            : [dst] "r"(dst), [a] "r" (a), [b] "r" (b)
+            : "cc", "memory",
+              "%xmm0", "%xmm1", "%xmm2", "%xmm3",
+              "%xmm4", "%xmm5", "%xmm6", "%xmm7"
+        );
+    }
+
+    void fmrmod3(float *dst, const float *a, const float *b, size_t count)
+    {
+        IF_ARCH_X86(size_t off);
+        ARCH_X86_ASM
+        (
+            FMOD_VV_CORE("dst", "dst", "a", "b", OP_RSEL, FMA_OFF)
+            : [off] "=&r" (off), [count] "+r" (count)
+            : [dst] "r"(dst), [a] "r" (a), [b] "r" (b)
+            : "cc", "memory",
+              "%xmm0", "%xmm1", "%xmm2", "%xmm3",
+              "%xmm4", "%xmm5", "%xmm6", "%xmm7"
+        );
+    }
+
+    void fmrmod3_fma3(float *dst, const float *a, const float *b, size_t count)
+    {
+        IF_ARCH_X86(size_t off);
+        ARCH_X86_ASM
+        (
+            FMOD_VV_CORE("dst", "dst", "a", "b", OP_RSEL, FMA_OFF)
+            : [off] "=&r" (off), [count] "+r" (count)
+            : [dst] "r"(dst), [a] "r" (a), [b] "r" (b)
+            : "cc", "memory",
+              "%xmm0", "%xmm1", "%xmm2", "%xmm3",
+              "%xmm4", "%xmm5", "%xmm6", "%xmm7"
+        );
+    }
+
+    void fmmod4(float *dst, const float *a, const float *b, const float *c, size_t count)
+    {
+        IF_ARCH_X86(size_t off);
+        ARCH_X86_ASM
+        (
+            FMOD_VV_CORE("dst", "a", "b", "c", OP_DSEL, FMA_OFF)
+            : [off] "=&r" (off), [count] __ASM_ARG_RW(count)
+            : [dst] "r"(dst), [a] "r" (a), [b] "r" (b), [c] "r" (c)
+            : "cc", "memory",
+              "%xmm0", "%xmm1", "%xmm2", "%xmm3",
+              "%xmm4", "%xmm5", "%xmm6", "%xmm7"
+        );
+    }
+
+    void fmmod4_fma3(float *dst, const float *a, const float *b, const float *c, size_t count)
+    {
+        IF_ARCH_X86(size_t off);
+        ARCH_X86_ASM
+        (
+            FMOD_VV_CORE("dst", "a", "b", "c", OP_DSEL, FMA_ON)
+            : [off] "=&r" (off), [count] __ASM_ARG_RW(count)
+            : [dst] "r"(dst), [a] "r" (a), [b] "r" (b), [c] "r" (c)
+            : "cc", "memory",
+              "%xmm0", "%xmm1", "%xmm2", "%xmm3",
+              "%xmm4", "%xmm5", "%xmm6", "%xmm7"
+        );
+    }
+
+    void fmrmod4(float *dst, const float *a, const float *b, const float *c, size_t count)
+    {
+        IF_ARCH_X86(size_t off);
+        ARCH_X86_ASM
+        (
+            FMOD_VV_CORE("dst", "a", "b", "c", OP_RSEL, FMA_OFF)
+            : [off] "=&r" (off), [count] __ASM_ARG_RW(count)
+            : [dst] "r"(dst), [a] "r" (a), [b] "r" (b), [c] "r" (c)
+            : "cc", "memory",
+              "%xmm0", "%xmm1", "%xmm2", "%xmm3",
+              "%xmm4", "%xmm5", "%xmm6", "%xmm7"
+        );
+    }
+
+    void fmrmod4_fma3(float *dst, const float *a, const float *b, const float *c, size_t count)
+    {
+        IF_ARCH_X86(size_t off);
+        ARCH_X86_ASM
+        (
+            FMOD_VV_CORE("dst", "a", "b", "c", OP_RSEL, FMA_ON)
+            : [off] "=&r" (off), [count] __ASM_ARG_RW(count)
+            : [dst] "r"(dst), [a] "r" (a), [b] "r" (b), [c] "r" (c)
+            : "cc", "memory",
+              "%xmm0", "%xmm1", "%xmm2", "%xmm3",
+              "%xmm4", "%xmm5", "%xmm6", "%xmm7"
+        );
+    }
+
+    #undef FMOD_VV_CORE
+
+    #undef FMA_OFF
+    #undef FMA_ON
+
     #undef OP_DSEL
     #undef OP_RSEL
 }
