@@ -14,360 +14,361 @@
 
 namespace sse
 {
-    static const float lanczos_2x2[] __lsp_aligned16 =
-    {
-        0.6203830132406946f, // k0
-        -0.1664152316035080f // k1
-    };
+    IF_ARCH_X86(
+        static const float lanczos_2x2[] __lsp_aligned16 =
+        {
+            +0.5731591682507563f,   // k0
+            -0.0636843520278618f    // k1
+        };
 
-    // Lanczos kernel 2x3: 3 SSE registers
-    static const float lanczos_2x3[] __lsp_aligned16 =
-    {
-        0.6293724479752082f, // k0
-        -0.1910530560835854f, // k1
-        0.0939539981090991f, // k2
-    };
+        // Lanczos kernel 2x3: 3 SSE registers
+        static const float lanczos_2x3[] __lsp_aligned16 =
+        {
+            +0.6079271018540265f,   // k0
+            -0.1350949115231170f,   // k1
+            +0.0243170840741611f    // k2
+        };
 
-    // Lanczos kernel 3x2: 6 SSE registers
-    static const float lanczos_kernel_3x2[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        -0.1451906347823569f,
-        -0.1903584501504231f,
-        +0.0000000000000000f,
+        // Lanczos kernel 3x2: 6 SSE registers
+        static const float lanczos_kernel_3x2[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            -0.0315888188312782f,
+            -0.0854897486982225f,
+            +0.0000000000000000f,
 
-        +0.4051504629060886f,
-        +0.8228011237053413f,
-        +1.0000000000000000f,
-        +0.8228011237053413f,
+            +0.3419589947928900f,
+            +0.7897204707819555f,
+            +1.0000000000000000f,
+            +0.7897204707819555f,
 
-        +0.4051504629060886f,
-        +0.0000000000000000f,
-        -0.1903584501504231f,
-        -0.1451906347823569f,
+            +0.3419589947928900f,
+            +0.0000000000000000f,
+            -0.0854897486982225f,
+            -0.0315888188312782f,
 
-        // Shifted by 1 left
-        -0.1451906347823569f,
-        -0.1903584501504231f,
-        +0.0000000000000000f,
-        +0.4051504629060886f,
+            // Shifted by 1 left
+            -0.0315888188312782f,
+            -0.0854897486982225f,
+            +0.0000000000000000f,
+            +0.3419589947928900f,
 
-        +0.8228011237053413f,
-        +1.0000000000000000f,
-        +0.8228011237053413f,
-        +0.4051504629060886f,
+            +0.7897204707819555f,
+            +1.0000000000000000f,
+            +0.7897204707819555f,
+            +0.3419589947928900f,
 
-        +0.0000000000000000f,
-        -0.1903584501504231f,
-        -0.1451906347823569f,
-        +0.0000000000000000f
-    };
+            +0.0000000000000000f,
+            -0.0854897486982225f,
+            -0.0315888188312782f,
+            +0.0000000000000000f
+        };
 
 
-    // Lanczos kernel 3x3: 9 SSE registers
-    static const float lanczos_kernel_3x3[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        +0.0890793429479492f,
-        +0.1055060549370832f,
-        +0.0000000000000000f,
+        // Lanczos kernel 3x3: 9 SSE registers
+        static const float lanczos_kernel_3x3[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            +0.0126609519658153f,
+            +0.0310789306368038f,
+            +0.0000000000000000f,
 
-        -0.1562250559899557f,
-        -0.1993645686793863f,
-        +0.0000000000000000f,
-        +0.4097746665281485f,
+            -0.0933267410806225f,
+            -0.1458230329384726f,
+            +0.0000000000000000f,
+            +0.3807169003008463f,
 
-        +0.8251285571768335f,
-        +1.0000000000000000f,
-        +0.8251285571768335f,
-        +0.4097746665281485f,
+            +0.8103009258121772f,
+            +1.0000000000000000f,
+            +0.8103009258121772f,
+            +0.3807169003008463f,
 
-        +0.0000000000000000f,
-        -0.1993645686793863f,
-        -0.1562250559899557f,
-        +0.0000000000000000f,
+            +0.0000000000000000f,
+            -0.1458230329384726f,
+            -0.0933267410806225f,
+            +0.0000000000000000f,
 
-        +0.1055060549370832f,
-        +0.0890793429479492f,
-        +0.0000000000000000f,
-        +0.0000000000000000f,
+            +0.0310789306368038f,
+            +0.0126609519658153f,
+            +0.0000000000000000f,
+            +0.0000000000000000f,
 
-        // Shifted by 1 left
-        +0.0890793429479492f,
-        +0.1055060549370832f,
-        +0.0000000000000000f,
-        -0.1562250559899557f,
+            // Shifted by 1 left
+            +0.0126609519658153f,
+            +0.0310789306368038f,
+            +0.0000000000000000f,
+            -0.0933267410806225f,
 
-        -0.1993645686793863f,
-        +0.0000000000000000f,
-        +0.4097746665281485f,
-        +0.8251285571768335f,
+            -0.1458230329384726f,
+            +0.0000000000000000f,
+            +0.3807169003008463f,
+            +0.8103009258121772f,
 
-        +1.0000000000000000f,
-        +0.8251285571768335f,
-        +0.4097746665281485f,
-        +0.0000000000000000f,
+            +1.0000000000000000f,
+            +0.8103009258121772f,
+            +0.3807169003008463f,
+            +0.0000000000000000f,
 
-        -0.1993645686793863f,
-        -0.1562250559899557f,
-        +0.0000000000000000f,
-        +0.1055060549370832f,
+            -0.1458230329384726f,
+            -0.0933267410806225f,
+            -0.0000000000000000f,
+            +0.0310789306368038f,
 
-        +0.0890793429479492f,
-        +0.0000000000000000f,
-        +0.0000000000000000f,
-        +0.0000000000000000f
-    };
+            +0.0126609519658153f,
+            +0.0000000000000000f,
+            +0.0000000000000000f,
+            +0.0000000000000000f
+        };
 
-    // Lanczos kernel 4x2: 4 SSE registers
-    static const float lanczos_kernel_4x2[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        -0.1187292496637064f,
-        -0.2001458972657618f,
-        -0.1729186314209981f,
+        // Lanczos kernel 4x2: 4 SSE registers
+        static const float lanczos_kernel_4x2[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            -0.0179051851263444f,
+            -0.0636843520278618f,
+            -0.0847248039068907f,
 
-        +0.0000000000000000f,
-        +0.2957854651930789f,
-        +0.6325370350028462f,
-        +0.8988707620097378f,
+            +0.0000000000000000f,
+            +0.2353466775191407f,
+            +0.5731591682507563f,
+            +0.8773540711908775f,
 
-        +1.0000000000000000f,
-        +0.8988707620097378f,
-        +0.6325370350028462f,
-        +0.2957854651930789f,
+            +1.0000000000000000f,
+            +0.8773540711908775f,
+            +0.5731591682507563f,
+            +0.2353466775191407f,
 
-        +0.0000000000000000f,
-        -0.1729186314209981f,
-        -0.2001458972657618f,
-        -0.1187292496637064f
-    };
+            +0.0000000000000000f,
+            -0.0847248039068907f,
+            -0.0636843520278618f,
+            -0.0179051851263444f
+        };
 
-    // Lanczos kernel 4x3: 6 SSE registers
-    static const float lanczos_kernel_4x3[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        +0.0749573750027847f,
-        +0.1184264228024920f,
-        +0.0943496807888575f,
+        // Lanczos kernel 4x3: 6 SSE registers
+        static const float lanczos_kernel_4x3[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            +0.0073559260471942f,
+            +0.0243170840741611f,
+            +0.0300210914495816f,
 
-        +0.0000000000000000f,
-        -0.1241641556633335f,
-        -0.2067943377468982f,
-        -0.1768665375130828f,
+            +0.0000000000000000f,
+            -0.0677913359005429f,
+            -0.1350949115231170f,
+            -0.1328710183650640f,
 
-        +0.0000000000000000f,
-        +0.2981808178680967f,
-        +0.6348032780190205f,
-        +0.8996736756130924f,
+            +0.0000000000000000f,
+            +0.2701898230462341f,
+            +0.6079271018540265f,
+            +0.8900670517104946f,
 
-        +1.0000000000000000f,
-        +0.8996736756130924f,
-        +0.6348032780190205f,
-        +0.2981808178680967f,
+            +1.0000000000000000f,
+            +0.8900670517104946f,
+            +0.6079271018540265f,
+            +0.2701898230462341f,
 
-        +0.0000000000000000f,
-        -0.1768665375130828f,
-        -0.2067943377468982f,
-        -0.1241641556633335f,
+            +0.0000000000000000f,
+            -0.1328710183650640f,
+            -0.1350949115231170f,
+            -0.0677913359005429f,
 
-        +0.0000000000000000f,
-        +0.0943496807888575f,
-        +0.1184264228024920f,
-        +0.0749573750027847f,
-    };
+            +0.0000000000000000f,
+            +0.0300210914495816f,
+            +0.0243170840741611f,
+            +0.0073559260471942f
+        };
 
-    // Lanczos kernel 6x2: 6 SSE registers
-    static const float lanczos_kernel_6x2[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        -0.0835168749111343f,
-        -0.1602001404590001f,
-        -0.2067943377468982f,
+        // Lanczos kernel 6x2: 6 SSE registers
+        static const float lanczos_kernel_6x2[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            -0.0078021377848166f,
+            -0.0315888188312782f,
+            -0.0636843520278618f,
 
-        -0.2025752314530442f,
-        -0.1343073344351001f,
-        +0.0000000000000000f,
-        +0.1894744904080251f,
+            -0.0854897486982225f,
+            -0.0719035699814534f,
+            +0.0000000000000000f,
+            +0.1409309971636486f,
 
-        +0.4114005618526707f,
-        +0.6348032780190205f,
-        +0.8259440903283795f,
-        +0.9546266788436470f,
+            +0.3419589947928900f,
+            +0.5731591682507563f,
+            +0.7897204707819555f,
+            +0.9440586719628122f,
 
-        +1.0000000000000000f,
-        +0.9546266788436470f,
-        +0.8259440903283795f,
-        +0.6348032780190205f,
+            +1.0000000000000000f,
+            +0.9440586719628122f,
+            +0.7897204707819555f,
+            +0.5731591682507563f,
 
-        +0.4114005618526707f,
-        +0.1894744904080251f,
-        +0.0000000000000000f,
-        -0.1343073344351001f,
+            +0.3419589947928900f,
+            +0.1409309971636486f,
+            +0.0000000000000000f,
+            -0.0719035699814534f,
 
-        -0.2025752314530442f,
-        -0.2067943377468982f,
-        -0.1602001404590001f,
-        -0.0835168749111343f
-    };
+            -0.0854897486982225f,
+            -0.0636843520278618f,
+            -0.0315888188312782f,
+            -0.0078021377848166f
+        };
 
-    // Lanczos kernel 6x3: 9 SSE registers
-    static const float lanczos_kernel_6x3[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        +0.0539107649662794f,
-        +0.0996822843396933f,
-        +0.1233221256242928f,
+        // Lanczos kernel 6x3: 9 SSE registers
+        static const float lanczos_kernel_6x3[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            +0.0032875048460955f,
+            +0.0126609519658153f,
+            +0.0243170840741611f,
 
-        +0.1149032936217488f,
-        +0.0717178866781924f,
-        -0.0000000000000000f,
-        -0.0853379770956540f,
+            +0.0310789306368038f,
+            +0.0248005479513036f,
+            -0.0000000000000000f,
+            -0.0424907562338176f,
 
-        -0.1630759564500400f,
-        -0.2097908159917360f,
-        -0.2048873332640742f,
-        -0.1354777817419537f,
+            -0.0933267410806225f,
+            -0.1350949115231170f,
+            -0.1458230329384726f,
+            -0.1049261531488149f,
 
-        +0.0000000000000000f,
-        +0.1903132912211352f,
-        +0.4125642785884168f,
-        +0.6358120571822271f,
+            +0.0000000000000000f,
+            +0.1676517041508127f,
+            +0.3807169003008463f,
+            +0.6079271018540265f,
 
-        +0.8265269098929101f,
-        +0.9547949937819170f,
-        +1.0000000000000000f,
-        +0.9547949937819170f,
+            +0.8103009258121772f,
+            +0.9500889005216107f,
+            +1.0000000000000000f,
+            +0.9500889005216107f,
 
-        +0.8265269098929101f,
-        +0.6358120571822271f,
-        +0.4125642785884168f,
-        +0.1903132912211352f,
+            +0.8103009258121772f,
+            +0.6079271018540265f,
+            +0.3807169003008463f,
+            +0.1676517041508127f,
 
-        +0.0000000000000000f,
-        -0.1354777817419537f,
-        -0.2048873332640742f,
-        -0.2097908159917360f,
+            +0.0000000000000000f,
+            -0.1049261531488149f,
+            -0.1458230329384726f,
+            -0.1350949115231170f,
 
-        -0.1630759564500400f,
-        -0.0853379770956540f,
-        -0.0000000000000000f,
-        +0.0717178866781924f,
+            -0.0933267410806225f,
+            -0.0424907562338176f,
+            -0.0000000000000000f,
+            +0.0248005479513036f,
 
-        +0.1149032936217488f,
-        +0.1233221256242928f,
-        +0.0996822843396933f,
-        +0.0539107649662794f
-    };
+            +0.0310789306368038f,
+            +0.0243170840741611f,
+            +0.0126609519658153f,
+            +0.0032875048460955f
+        };
 
-    // Lanczos kernel 8x2: 8 SSE registers
-    static const float lanczos_kernel_8x2[] __lsp_aligned16 =
-    {
-        -0.0000000000000000f,
-        -0.0635086982303136f,
-        -0.1261005767591211f,
-        -0.1779172640206227f,
+        // Lanczos kernel 8x2: 8 SSE registers
+        static const float lanczos_kernel_8x2[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            -0.0043033145538298f,
+            -0.0179051851263444f,
+            -0.0393892611124141f,
 
-        -0.2091519082144436f,
-        -0.2112875836092082f,
-        -0.1782608868029105f,
-        -0.1073988642554405f,
+            -0.0636843520278618f,
+            -0.0823353965569232f,
+            -0.0847248039068907f,
+            -0.0600950644541902f,
 
-        +0.0000000000000000f,
-        +0.1385297654836717f,
-        +0.2990219260297742f,
-        +0.4693478599070060f,
+            +0.0000000000000000f,
+            +0.0993408208324369f,
+            +0.2353466775191407f,
+            +0.3985033193355084f,
 
-        +0.6355976112274050f,
-        +0.7835048889119777f,
-        +0.8999547969760605f,
-        +0.9743975231790596f,
+            +0.5731591682507563f,
+            +0.7396427919997760f,
+            +0.8773540711908775f,
+            +0.9682457746117045f,
 
-        +1.0000000000000000f,
-        +0.9743975231790596f,
-        +0.8999547969760605f,
-        +0.7835048889119777f,
+            +1.0000000000000000f,
+            +0.9682457746117045f,
+            +0.8773540711908775f,
+            +0.7396427919997760f,
 
-        +0.6355976112274050f,
-        +0.4693478599070060f,
-        +0.2990219260297742f,
-        +0.1385297654836717f,
+            +0.5731591682507563f,
+            +0.3985033193355084f,
+            +0.2353466775191407f,
+            +0.0993408208324369f,
 
-        +0.0000000000000000f,
-        -0.1073988642554405f,
-        -0.1782608868029105f,
-        -0.2112875836092082f,
+            +0.0000000000000000f,
+            -0.0600950644541902f,
+            -0.0847248039068907f,
+            -0.0823353965569232f,
 
-        -0.2091519082144436f,
-        -0.1779172640206227f,
-        -0.1261005767591211f,
-        -0.0635086982303136f
-    };
+            -0.0636843520278618f,
+            -0.0393892611124141f,
+            -0.0179051851263444f,
+            -0.0043033145538298f
+        };
 
-    // Lanczos kernel 8x3: 12 SSE registers
-    static const float lanczos_kernel_8x3[] __lsp_aligned16 =
-    {
-        +0.0000000000000000f,
-        +0.0413762983728270f,
-        +0.0800907151667399f,
-        +0.1098388956286788f,
+        // Lanczos kernel 8x3: 12 SSE registers
+        static const float lanczos_kernel_8x3[] __lsp_aligned16 =
+        {
+            +0.0000000000000000f,
+            +0.0018368899607481f,
+            +0.0073559260471942f,
+            +0.0155961678435580f,
 
-        +0.1250635280404858f,
-        +0.1218381732799711f,
-        +0.0985951550643596f,
-        +0.0565868891962354f,
+            +0.0243170840741611f,
+            +0.0303079634725070f,
+            +0.0300210914495816f,
+            +0.0204366616947175f,
 
-        -0.0000000000000000f,
-        -0.0643160644975587f,
-        -0.1274947024165923f,
-        -0.1796106608775988f,
+            +0.0000000000000000f,
+            -0.0305684889733737f,
+            -0.0677913359005429f,
+            -0.1054383717904384f,
 
-        -0.2108456783342821f,
-        -0.2127234580145891f,
-        -0.1792608653618989f,
-        -0.1078863334836789f,
+            -0.1350949115231170f,
+            -0.1472651639056537f,
+            -0.1328710183650640f,
+            -0.0849124693704824f,
 
-        +0.0000000000000000f,
-        +0.1389094368005209f,
-        +0.2996235873365793f,
-        +0.4700032639087741f,
+            +0.0000000000000000f,
+            +0.1205345965259870f,
+            +0.2701898230462341f,
+            +0.4376469925430009f,
 
-        +0.6361653568810439f,
-        +0.7838984051945062f,
-        +0.9001556302107272f,
-        +0.9744518753544421f,
+            +0.6079271018540265f,
+            +0.7642122243343417f,
+            +0.8900670517104946f,
+            +0.9717147892357163f,
 
-        +1.0000000000000000f,
-        +0.9744518753544421f,
-        +0.9001556302107272f,
-        +0.7838984051945062f,
+            +1.0000000000000000f,
+            +0.9717147892357163f,
+            +0.8900670517104946f,
+            +0.7642122243343417f,
 
-        +0.6361653568810439f,
-        +0.4700032639087741f,
-        +0.2996235873365793f,
-        +0.1389094368005209f,
+            +0.6079271018540265f,
+            +0.4376469925430009f,
+            +0.2701898230462341f,
+            +0.1205345965259870f,
 
-        +0.0000000000000000f,
-        -0.1078863334836789f,
-        -0.1792608653618989f,
-        -0.2127234580145891f,
+            +0.0000000000000000f,
+            -0.0849124693704824f,
+            -0.1328710183650640f,
+            -0.1472651639056537f,
 
-        -0.2108456783342821f,
-        -0.1796106608775988f,
-        -0.1274947024165923f,
-        -0.0643160644975587f,
+            -0.1350949115231170f,
+            -0.1054383717904384f,
+            -0.0677913359005429f,
+            -0.0305684889733737f,
 
-        -0.0000000000000000f,
-        +0.0565868891962354f,
-        +0.0985951550643596f,
-        +0.1218381732799711f,
-
-        +0.1250635280404858f,
-        +0.1098388956286788f,
-        +0.0800907151667399f,
-        +0.0413762983728270f
-    };
+            +0.0000000000000000f,
+            +0.0204366616947175f,
+            +0.0300210914495816f,
+            +0.0303079634725070f,
 
+            +0.0243170840741611f,
+            +0.0155961678435580f,
+            +0.0073559260471942f,
+            +0.0018368899607481f
+        };
+    )
 
     void lanczos_resample_2x2(float *dst, const float *src, size_t count)
     {
