@@ -654,6 +654,93 @@ namespace sse2
 
 #undef HSLA_TRANSPOSE
 
+    void rgba32_to_bgra32(void *dst, const void *src, size_t count)
+    {
+        size_t off;
+
+        ARCH_X86_ASM
+        (
+            __ASM_EMIT("movdqa      %[MASK], %%xmm6")                   // xmm6 = 00 ff 00 ff
+            __ASM_EMIT("xor         %[off], %[off]")                    // off  = 0
+            __ASM_EMIT("movdqa      %%xmm6, %%xmm7")                    // xmm7 = 00 ff 00 ff
+            __ASM_EMIT("pslld       $8, %%xmm6")                        // xmm6 = ff 00 ff 00
+
+            // 8-element blocks
+            __ASM_EMIT("sub         $8, %[count]")
+            __ASM_EMIT("jb          2f")
+            __ASM_EMIT("1:")
+            __ASM_EMIT("movdqu      0x00(%[src], %[off]), %%xmm0")      // xmm0 = A1 R1 G1 B1
+            __ASM_EMIT("movdqu      0x10(%[src], %[off]), %%xmm1")      // xmm1 = A2 R2 G2 B2
+            __ASM_EMIT("movdqa      %%xmm0, %%xmm2")                    // xmm2 = A1 R1 G1 B1
+            __ASM_EMIT("movdqa      %%xmm1, %%xmm3")                    // xmm3 = A2 R2 G2 B2
+            __ASM_EMIT("pand        %%xmm7, %%xmm0")                    // xmm0 = 00 R1 00 B1
+            __ASM_EMIT("pand        %%xmm6, %%xmm2")                    // xmm2 = A1 00 G1 00
+            __ASM_EMIT("pand        %%xmm7, %%xmm1")                    // xmm1 = 00 R2 00 B2
+            __ASM_EMIT("pand        %%xmm6, %%xmm3")                    // xmm3 = A2 00 G2 00
+            __ASM_EMIT("movdqa      %%xmm0, %%xmm4")                    // xmm4 = A1 00 G1 00
+            __ASM_EMIT("movdqa      %%xmm1, %%xmm5")                    // xmm5 = A2 00 G2 00
+            __ASM_EMIT("pslld       $16, %%xmm0")                       // xmm0 = 00 B1 00 00
+            __ASM_EMIT("pslld       $16, %%xmm1")                       // xmm1 = 00 B2 00 00
+            __ASM_EMIT("psrld       $16, %%xmm4")                       // xmm4 = 00 00 00 R1
+            __ASM_EMIT("psrld       $16, %%xmm5")                       // xmm5 = 00 00 00 R2
+            __ASM_EMIT("orpd        %%xmm2, %%xmm0")                    // xmm0 = A1 B1 G1 00
+            __ASM_EMIT("orpd        %%xmm3, %%xmm1")                    // xmm1 = A2 B2 G2 00
+            __ASM_EMIT("orpd        %%xmm4, %%xmm0")                    // xmm0 = A1 B1 G1 R1
+            __ASM_EMIT("orpd        %%xmm5, %%xmm1")                    // xmm1 = A2 B2 G2 R2
+            __ASM_EMIT("movdqu      %%xmm0, 0x00(%[dst], %[off])")
+            __ASM_EMIT("movdqu      %%xmm1, 0x10(%[dst], %[off])")
+            __ASM_EMIT("add         $0x20, %[off]")
+            __ASM_EMIT("sub         $8, %[count]")
+            __ASM_EMIT("jae         1b")
+
+            // 4-element block
+            __ASM_EMIT("2:")
+            __ASM_EMIT("add         $4, %[count]")
+            __ASM_EMIT("jl          4f")
+            __ASM_EMIT("movdqu      0x00(%[src], %[off]), %%xmm0")      // xmm0 = A1 R1 G1 B1
+            __ASM_EMIT("movdqa      %%xmm0, %%xmm2")                    // xmm2 = A1 R1 G1 B1
+            __ASM_EMIT("pand        %%xmm7, %%xmm0")                    // xmm0 = 00 R1 00 B1
+            __ASM_EMIT("pand        %%xmm6, %%xmm2")                    // xmm2 = A1 00 G1 00
+            __ASM_EMIT("movdqa      %%xmm0, %%xmm4")                    // xmm4 = A1 00 G1 00
+            __ASM_EMIT("pslld       $16, %%xmm0")                       // xmm0 = 00 B1 00 00
+            __ASM_EMIT("psrld       $16, %%xmm4")                       // xmm4 = 00 00 00 R1
+            __ASM_EMIT("orpd        %%xmm2, %%xmm0")                    // xmm0 = A1 B1 G1 00
+            __ASM_EMIT("orpd        %%xmm4, %%xmm0")                    // xmm0 = A1 B1 G1 R1
+            __ASM_EMIT("movdqu      %%xmm0, 0x00(%[dst], %[off])")
+            __ASM_EMIT("add         $0x10, %[off]")
+            __ASM_EMIT("sub         $4, %[count]")
+
+            // Tail
+            __ASM_EMIT("4:")
+            __ASM_EMIT("add         $3, %[count]")
+            __ASM_EMIT("jl          6f")
+            __ASM_EMIT("5:")
+            __ASM_EMIT("movd        0x00(%[src], %[off]), %%xmm0")      // xmm0 = AA RR GG BB
+            __ASM_EMIT("movdqa      %%xmm0, %%xmm1")                    // xmm1 = AA RR GG BB
+            __ASM_EMIT("pand        %%xmm7, %%xmm0")                    // xmm0 = 00 RR 00 BB
+            __ASM_EMIT("pand        %%xmm6, %%xmm1")                    // xmm1 = AA 00 GG 00
+            __ASM_EMIT("movdqa      %%xmm0, %%xmm2")                    // xmm2 = 00 RR 00 BB
+            __ASM_EMIT("pslld       $16, %%xmm0")                       // xmm0 = 00 BB 00 00
+            __ASM_EMIT("psrld       $16, %%xmm2")                       // xmm2 = 00 00 00 RR
+            __ASM_EMIT("orpd        %%xmm1, %%xmm0")                    // xmm0 = AA 00 GG RR
+            __ASM_EMIT("orpd        %%xmm2, %%xmm0")                    // xmm0 = AA BB GG RR
+            __ASM_EMIT("movd        %%xmm0, 0x00(%[dst], %[off])")
+            __ASM_EMIT("add         $4, %[off]")
+            __ASM_EMIT("dec         %[count]")
+            __ASM_EMIT("jge         5b")
+
+            // End
+            __ASM_EMIT("6:")
+
+            : [dst] "+r"(dst), [src] "+r"(src), [count] "+r" (count),
+              [off] "=&r" (off)
+            : [MASK] "m" (X_CMASK)
+            : "cc", "memory",
+              "%xmm0", "%xmm1", "%xmm2", "%xmm3",
+              "%xmm4", "%xmm5", "%xmm6", "%xmm7"
+        );
+    }
+
 }
 
 #endif /* INCLUDE_DSP_ARCH_X86_SSE2_GRAPHICS_H_ */
