@@ -23,9 +23,9 @@
                 b0 + b1*s + b2*s^2
 
      Normalized biquad filter:
-               a0 + a1*z^-1 + a2*z^-2
+               b0 + b1*z^-1 + b2*z^-2
        h[z] = ────────────────────────
-               1 - b1*z^-1 - b2*z^-2
+               1 - a1*z^-1 - a2*z^-2
 
      The structure of biquad filter:
 
@@ -34,7 +34,7 @@
        d0, d1 = filter memory
 
         s   ┌─────┐     ┌───┐                s'
-       ──┬─►│x a0 │────►│ + │──────────────┬────►
+       ──┬─►│x b0 │────►│ + │──────────────┬────►
          │  └─────┘     └───┘              │
          │                ▲                │
          │                │                │
@@ -44,7 +44,7 @@
          │                ▲                │
          │                │                │
          │  ┌─────┐     ┌───┐     ┌─────┐  │
-         ├─►│x a1 │────►│ + │◄────│x b1 │◄─┤
+         ├─►│x b1 │────►│ + │◄────│x a1 │◄─┤
          │  └─────┘     └───┘     └─────┘  │
          │                ▲                │
          │                │                │
@@ -54,7 +54,7 @@
          │                ▲                │
          │                │                │
          │  ┌─────┐     ┌───┐     ┌─────┐  │
-         └─►│x a2 │────►│ + │◄────│x b2 │◄─┘
+         └─►│x b2 │────►│ + │◄────│x a2 │◄─┘
             └─────┘     └───┘     └─────┘
 
     Static filters consist of biquad filter banks organized in optimal
@@ -73,15 +73,15 @@
 
       Index      +0x00     +0x01     +0x02     +0x03
               ┌─────────┬─────────┬─────────┬─────────┐
-       +0x00  │ f[0].a0 │ f[1].a0 │ f[2].a0 │ f[3].a0 │
+       +0x00  │ f[0].b0 │ f[1].b0 │ f[2].b0 │ f[3].b0 │
               ├─────────┼─────────┼─────────┼─────────┤
-       +0x04  │ f[0].a1 │ f[1].a1 │ f[2].a1 │ f[3].a1 │
+       +0x04  │ f[0].b1 │ f[1].b1 │ f[2].b1 │ f[3].b1 │
               ├─────────┼─────────┼─────────┼─────────┤
-       +0x08  │ f[0].a2 │ f[1].a2 │ f[2].a2 │ f[3].a2 │
+       +0x08  │ f[0].b2 │ f[1].b2 │ f[2].b2 │ f[3].b2 │
               ├─────────┼─────────┼─────────┼─────────┤
-       +0x0c  │ f[0].b0 │ f[1].b0 │ f[2].b0 │ f[3].b0 │
+       +0x0c  │ f[0].a0 │ f[1].a0 │ f[2].a0 │ f[3].a0 │
               ├─────────┼─────────┼─────────┼─────────┤
-       +0x10  │ f[0].b1 │ f[1].b1 │ f[2].b1 │ f[3].b1 │
+       +0x10  │ f[0].a1 │ f[1].a1 │ f[2].a1 │ f[3].a1 │
               └─────────┴─────────┴─────────┴─────────┘
 
     Dynamic filters consist of shifted biquad filter banks. For example,
@@ -175,164 +175,85 @@
 
 */
 
-// These constants should be redefined if structure of biquad_t changes
-#define BIQUAD_X8_A0_OFF        0x40
-#define BIQUAD_X8_A0_SOFF       "0x40"
-#define BIQUAD_X8_I0_OFF        0x50
-#define BIQUAD_X8_I0_SOFF       "0x50"
-#define BIQUAD_X8_A1_OFF        0x60
-#define BIQUAD_X8_A1_SOFF       "0x60"
-#define BIQUAD_X8_I1_OFF        0x70
-#define BIQUAD_X8_I1_SOFF       "0x70"
-#define BIQUAD_X8_A2_OFF        0x80
-#define BIQUAD_X8_A2_SOFF       "0x80"
-#define BIQUAD_X8_I2_OFF        0x90
-#define BIQUAD_X8_I2_SOFF       "0x90"
-#define BIQUAD_X8_B1_OFF        0xa0
-#define BIQUAD_X8_B1_SOFF       "0xa0"
-#define BIQUAD_X8_J1_OFF        0xb0
-#define BIQUAD_X8_J1_SOFF       "0xb0"
-#define BIQUAD_X8_B2_OFF        0xc0
-#define BIQUAD_X8_B2_SOFF       "0xc0"
-#define BIQUAD_X8_J2_OFF        0xd0
-#define BIQUAD_X8_J2_SOFF       "0xd0"
-
-#define BIQUAD_X4_A0_OFF        BIQUAD_X8_A0_OFF
-#define BIQUAD_X4_A0_SOFF       BIQUAD_X8_A0_SOFF
-#define BIQUAD_X4_A1_OFF        BIQUAD_X8_I0_OFF
-#define BIQUAD_X4_A1_SOFF       BIQUAD_X8_I0_SOFF
-#define BIQUAD_X4_A2_OFF        BIQUAD_X8_A1_OFF
-#define BIQUAD_X4_A2_SOFF       BIQUAD_X8_A1_SOFF
-#define BIQUAD_X4_B1_OFF        BIQUAD_X8_I1_OFF
-#define BIQUAD_X4_B1_SOFF       BIQUAD_X8_I1_SOFF
-#define BIQUAD_X4_B2_OFF        BIQUAD_X8_A2_OFF
-#define BIQUAD_X4_B2_SOFF       BIQUAD_X8_A2_SOFF
-
-#define BIQUAD_X2_A_OFF         BIQUAD_X4_A0_OFF
-#define BIQUAD_X2_A_SOFF        BIQUAD_X4_A0_SOFF
-#define BIQUAD_X2_I_OFF         BIQUAD_X4_A1_OFF
-#define BIQUAD_X2_I_SOFF        BIQUAD_X4_A1_SOFF
-#define BIQUAD_X2_B_OFF         BIQUAD_X4_A2_OFF
-#define BIQUAD_X2_B_SOFF        BIQUAD_X4_A2_SOFF
-#define BIQUAD_X2_J_OFF         BIQUAD_X4_B1_OFF
-#define BIQUAD_X2_J_SOFF        BIQUAD_X4_B1_SOFF
-
-#define BIQUAD_X1_A_OFF         BIQUAD_X4_A0_OFF
-#define BIQUAD_X1_A_SOFF        BIQUAD_X4_A0_SOFF
-#define BIQUAD_X1_B_OFF         BIQUAD_X4_A1_OFF
-#define BIQUAD_X1_B_SOFF        BIQUAD_X4_A1_SOFF
-
-#define BIQUAD_D0_OFF           0x00
-#define BIQUAD_D0_SOFF          "0x00"
-#define BIQUAD_D1_OFF           0x10
-#define BIQUAD_D1_SOFF          "0x10"
-#define BIQUAD_D2_OFF           0x20
-#define BIQUAD_D2_SOFF          "0x20"
-#define BIQUAD_D3_OFF           0x30
-#define BIQUAD_D3_SOFF          "0x30"
-
-#define DYN_BIQUAD_X8_A0_OFF    0x00
-#define DYN_BIQUAD_X8_A0_SOFF   "0x00"
-#define DYN_BIQUAD_X8_I0_OFF    0x10
-#define DYN_BIQUAD_X8_I0_SOFF   "0x10"
-#define DYN_BIQUAD_X8_A1_OFF    0x20
-#define DYN_BIQUAD_X8_A1_SOFF   "0x20"
-#define DYN_BIQUAD_X8_I1_OFF    0x30
-#define DYN_BIQUAD_X8_I1_SOFF   "0x30"
-#define DYN_BIQUAD_X8_A2_OFF    0x40
-#define DYN_BIQUAD_X8_A2_SOFF   "0x40"
-#define DYN_BIQUAD_X8_I2_OFF    0x50
-#define DYN_BIQUAD_X8_I2_SOFF   "0x50"
-#define DYN_BIQUAD_X8_B1_OFF    0x60
-#define DYN_BIQUAD_X8_B1_SOFF   "0x60"
-#define DYN_BIQUAD_X8_J1_OFF    0x70
-#define DYN_BIQUAD_X8_J1_SOFF   "0x70"
-#define DYN_BIQUAD_X8_B2_OFF    0x80
-#define DYN_BIQUAD_X8_B2_SOFF   "0x80"
-#define DYN_BIQUAD_X8_J2_OFF    0x90
-#define DYN_BIQUAD_X8_J2_SOFF   "0x90"
-#define DYN_BIQUAD_X8_SIZE      0xa0
-#define DYN_BIQUAD_X8_SSIZE     "0xa0"
-#define DYN_BIQUAD_X8_X4SIZE    0x280
-#define DYN_BIQUAD_X8_X4SSIZE   "0x280"
-
-#define DYN_BIQUAD_X4_A0_OFF    DYN_BIQUAD_X8_A0_OFF
-#define DYN_BIQUAD_X4_A0_SOFF   DYN_BIQUAD_X8_A0_SOFF
-#define DYN_BIQUAD_X4_A1_OFF    DYN_BIQUAD_X8_I0_OFF
-#define DYN_BIQUAD_X4_A1_SOFF   DYN_BIQUAD_X8_I0_SOFF
-#define DYN_BIQUAD_X4_A2_OFF    DYN_BIQUAD_X8_A1_OFF
-#define DYN_BIQUAD_X4_A2_SOFF   DYN_BIQUAD_X8_A1_SOFF
-#define DYN_BIQUAD_X4_B1_OFF    DYN_BIQUAD_X8_I1_OFF
-#define DYN_BIQUAD_X4_B1_SOFF   DYN_BIQUAD_X8_I1_SOFF
-#define DYN_BIQUAD_X4_B2_OFF    DYN_BIQUAD_X8_A2_OFF
-#define DYN_BIQUAD_X4_B2_SOFF   DYN_BIQUAD_X8_A2_SOFF
-#define DYN_BIQUAD_X4_SIZE      0x50
-#define DYN_BIQUAD_X4_SSIZE     "0x50"
-
-#define DYN_BIQUAD_X2_A_OFF     DYN_BIQUAD_X4_A0_OFF
-#define DYN_BIQUAD_X2_A_SOFF    DYN_BIQUAD_X4_A0_SOFF
-#define DYN_BIQUAD_X2_I_OFF     DYN_BIQUAD_X4_A1_OFF
-#define DYN_BIQUAD_X2_I_SOFF    DYN_BIQUAD_X4_A1_SOFF
-#define DYN_BIQUAD_X2_B_OFF     DYN_BIQUAD_X4_A2_OFF
-#define DYN_BIQUAD_X2_B_SOFF    DYN_BIQUAD_X4_A2_SOFF
-#define DYN_BIQUAD_X2_J_OFF     DYN_BIQUAD_X4_B1_OFF
-#define DYN_BIQUAD_X2_J_SOFF    DYN_BIQUAD_X4_B1_SOFF
-#define DYN_BIQUAD_X2_SIZE      0x40
-#define DYN_BIQUAD_X2_SSIZE     "0x40"
-
-#define DYN_BIQUAD_X1_A_OFF     DYN_BIQUAD_X2_A_OFF
-#define DYN_BIQUAD_X1_A_SOFF    DYN_BIQUAD_X2_A_SOFF
-#define DYN_BIQUAD_X1_B_OFF     DYN_BIQUAD_X2_I_OFF
-#define DYN_BIQUAD_X1_B_SOFF    DYN_BIQUAD_X2_I_SOFF
-#define DYN_BIQUAD_X1_SIZE      0x20
-#define DYN_BIQUAD_X1_SSIZE     "0x20"
-
-#define BIQUAD_D_ITEMS      16
-#define BIQUAD_ALIGN        0x40
+/**
+ * These constants define the offset of filter constants relative to the memory in biquad_t structure,
+ * filter alignment and maximum number of memory elements
+ */
+#define BIQUAD_XN_OFF           0x40
+#define BIQUAD_XN_SOFF          "0x40"
+#define BIQUAD_ALIGN            0x40
+#define BIQUAD_D_ITEMS          16
 
 #pragma pack(push, 1)
 
-// Analog filter cascade
+/**
+ * Analog filter cascade transfer function:
+ *
+ *              t0 + t1*s + t2*s^2
+ *     H[s] = ──────────────────────
+ *              b0 + b1*s + b2*s^2
+ */
 typedef struct f_cascade_t
 {
     float       t[4];       // Top part of polynom (zeros): T[p] = t[0] + t[1] * p + t[2] * p^2
     float       b[4];       // Bottom part of polynom (poles): B[p] = b[0] + b[1] * p + b[2] * p^2
 } f_cascade_t;
 
-// Biquad filter banks
+/**
+ * Biquad filter bank for 1 digital biquad filter
+ * Non-used elements should be filled with zeros
+ */
 typedef struct biquad_x1_t
 {
-    float   a[4];  //  a0 a0 a1 a2
-    float   b[4];  //  b1 b2 0  0
+    float   b0, b1, b2;     //  b0 b1 b2
+    float   a1, a2;         //  a1 a2
+    float   p0, p1, p2;     //  padding (not used), SHOULD be zero
 } biquad_x1_t;
 
+/**
+ * Biquad filter bank for 2 digital biquad filters
+ * Non-used elements should be filled with zeros
+ */
 typedef struct biquad_x2_t
 {
-    float   a[8];  //  a0 a0 a1 a2 i0 i1 i2 i3
-    float   b[8];  //  b1 b2 0  0  j1 j2 0  0
+    float   b0[2];
+    float   b1[2];
+    float   b2[2];
+    float   a1[2];
+    float   a2[2];
+    float   p[2];           // padding (not used), SHOULD be zero
 } biquad_x2_t;
 
+/**
+ * Biquad filter bank for 4 digital biquad filters
+ */
 typedef struct biquad_x4_t
 {
-    float   a0[4];
-    float   a1[4];
-    float   a2[4];
+    float   b0[4];
     float   b1[4];
     float   b2[4];
+    float   a1[4];
+    float   a2[4];
 } biquad_x4_t;
 
+/**
+ * Biquad filter bank for 8 digital biquad filters
+ */
 typedef struct biquad_x8_t
 {
-    float   a0[8];
-    float   a1[8];
-    float   a2[8];
+    float   b0[8];
     float   b1[8];
     float   b2[8];
+    float   a1[8];
+    float   a2[8];
 } biquad_x8_t;
 
-// This is main filter structure with memory
-// It should be aligned at least to 16-byte boundary
-// For best purposes it should be aligned to 64-byte boundary
+/**
+ * This is main filter structure with memory elements
+ * It should be aligned at least to 16-byte boundary due to
+ * alignment restrictions of some different hardware architectures
+ * For best purpose it should be aligned to 64-byte boundary
+ */
 typedef struct biquad_t
 {
     float   d[BIQUAD_D_ITEMS];
@@ -434,6 +355,47 @@ namespace dsp
      */
     extern void (* dyn_biquad_process_x8)(float *dst, const float *src, float *d, size_t count, const biquad_x8_t *f);
 
+    //---------------------------------------------------------------------------------------
+    // Transfer function calculation
+    //---------------------------------------------------------------------------------------
+    /**
+     * Compute filter transfer function, computes complex dst = H(f)
+     * @param re destination to store transfer function (real value)
+     * @param im destination to store transfer function (imaginary value)
+     * @param c filter cascade
+     * @param freq normalized frequency array
+     * @param count size of frequency array
+     */
+    extern void (* filter_transfer_calc_ri)(float *re, float *im, const f_cascade_t *c, const float *freq, size_t count);
+
+    /**
+     * Apply filter transfer function, computes complex dst = dst * H(f)
+     * @param re destination to apply transfer function (real value)
+     * @param im destination to apply transfer function (imaginary value)
+     * @param c filter cascade
+     * @param freq normalized frequency array
+     * @param count size of frequency array
+     */
+    extern void (* filter_transfer_apply_ri)(float *re, float *im, const f_cascade_t *c, const float *freq, size_t count);
+
+    /**
+     * Compute filter transfer function, computes complex dst = H(f)
+     * @param dst destination to store transfer function (packed complex value)
+     * @param c filter cascade
+     * @param freq normalized frequency array
+     * @param count size of frequency array
+     */
+    extern void (* filter_transfer_calc_pc)(float *dst, const f_cascade_t *c, const float *freq, size_t count);
+
+    /**
+     * Apply filter transfer function, computes complex dst = dst * H(f)
+     * @param dst destination to apply transfer function (packed complex value)
+     * @param c filter cascade
+     * @param freq normalized frequency array
+     * @param count size of frequency array
+     */
+    extern void (* filter_transfer_apply_pc)(float *dst, const f_cascade_t *c, const float *freq, size_t count);
+
     //---------------------------------------------------------------------------------------
     // Bilinear transformation of dynamic filters
     //---------------------------------------------------------------------------------------
