@@ -32,8 +32,13 @@
 #define LSP_LATENCY_METER_BASE          LSP_LADSPA_BASE + 69
 #define LSP_MB_COMPRESSOR_BASE          LSP_LADSPA_BASE + 70
 #define LSP_PROFILER_BASE               LSP_LADSPA_BASE + 78
-#define LSP_REVERB_BUILDER_BASE         LSP_LADSPA_BASE + 80
-#define LSP_NONLINEAR_CONVOLVER_BASE    LSP_LADSPA_BASE + 82
+#define LSP_ROOM_BUILDER_BASE           LSP_LADSPA_BASE + 80
+#define LSP_MB_EXPANDER_BASE            LSP_LADSPA_BASE + 82
+#define LSP_MB_GATE_BASE                LSP_LADSPA_BASE + 90
+#define LSP_LOUD_COMP_BASE              LSP_LADSPA_BASE + 98
+#define LSP_SURGE_FILTER_BASE           LSP_LADSPA_BASE + 100
+
+#define LSP_NONLINEAR_CONVOLVER_BASE    LSP_LADSPA_BASE + 1000
 
 
 // Plugin metadata definitions
@@ -58,6 +63,10 @@
 #include <metadata/mb_compressor.h>
 #include <metadata/profiler.h>
 #include <metadata/room_builder.h>
+#include <metadata/mb_expander.h>
+#include <metadata/mb_gate.h>
+#include <metadata/loud_comp.h>
+#include <metadata/surge_filter.h>
 
 #include <metadata/nonlinear_convolver.h>
 
