@@ -15,7 +15,6 @@
 #include <core/util/Sidechain.h>
 #include <core/util/Delay.h>
 #include <core/util/MeterGraph.h>
-#include <core/util/MeterGraph.h>
 #include <core/util/Analyzer.h>
 #include <core/filters/DynamicFilters.h>
 #include <core/filters/Equalizer.h>
@@ -50,7 +49,6 @@ namespace lsp
                 Filter          sPassFilter;        // Passing filter for 'classic' mode
                 Filter          sRejFilter;         // Rejection filter for 'classic' mode
                 Filter          sAllFilter;         // All-pass filter for phase compensation
-//                Equalizer       sRejFilter;         // Rejection equalizer for 'classic' mode
                 Delay           sDelay;             // Delay for lookahead purpose
 
                 float          *vTr;                // Transfer function
@@ -63,14 +61,12 @@ namespace lsp
                 float           fFreqHCF;           // Cutoff frequency for low-pass filter
                 float           fFreqLCF;           // Cutoff frequency for high-pass filter
                 float           fMakeup;            // Makeup gain
-                float           fEnvLevel;          // Envelope level
                 float           fGainLevel;         // Gain adjustment level
                 size_t          nLookahead;         // Lookahead amount
 
                 bool            bEnabled;           // Enabled flag
                 bool            bCustHCF;           // Custom frequency for high-cut filter
                 bool            bCustLCF;           // Custom frequency for low-cut filter
-//                bool            bFFT;               // Enable FFT for channel
                 bool            bMute;              // Mute channel
                 bool            bSolo;              // Solo channel
                 bool            bExtSc;             // External sidechain
@@ -88,8 +84,6 @@ namespace lsp
                 IPort          *pScLcfFreq;         // Sidechain low-cut frequency
                 IPort          *pScHcfFreq;         // Sidechain hi-cut frequency
                 IPort          *pScFreqChart;       // Sidechain band frequency chart
-//                IPort          *pScFftOn;           // Sidechain FFT enable
-//                IPort          *pScFftChart;        // Sidechain FFT graph
 
                 IPort          *pMode;              // Compressor mode
                 IPort          *pEnable;            // Enable compressor
@@ -101,6 +95,7 @@ namespace lsp
                 IPort          *pRelTime;           // Release time
                 IPort          *pRatio;             // Ratio
                 IPort          *pKnee;              // Knee
+                IPort          *pBThresh;           // Boost threshold
                 IPort          *pMakeup;            // Makeup gain
                 IPort          *pFreqEnd;           // Frequency range end
                 IPort          *pCurveGraph;        // Compressor curve graph
@@ -142,10 +137,8 @@ namespace lsp
 
                 size_t          nAnInChannel;       // Analyzer channel used for input signal analysis
                 size_t          nAnOutChannel;      // Analyzer channel used for output signal analysis
-//                size_t          nSfFttChannel;      // Sidechain FFT channel
                 bool            bInFft;             // Input signal FFT enabled
                 bool            bOutFft;            // Output signal FFT enabled
-//                bool            bScFft;             // Sidechain FFT enabled
 
                 IPort          *pIn;                // Input
                 IPort          *pOut;               // Output
@@ -154,7 +147,6 @@ namespace lsp
                 IPort          *pFftInSw;           // Pre-processing FFT analysis control port
                 IPort          *pFftOut;            // Post-processing FFT analysis data
                 IPort          *pFftOutSw;          // Post-processing FFT analysis controlport
-//                IPort          *pScFftSw;           // Sidechain FFT enable switch
                 IPort          *pAmpGraph;          // Compressor's amplitude graph
                 IPort          *pInLvl;             // Input level meter
                 IPort          *pOutLvl;            // Output level meter
