@@ -34,9 +34,10 @@
 #include <plugins/room_builder.h>
 #include <plugins/mb_expander.h>
 #include <plugins/mb_gate.h>
-
+#include <plugins/loud_comp.h>
 #include <plugins/nonlinear_convolver.h>
 
 #include <plugins/experimental.h>
+#include <plugins/surge_filter.h>
 
 #endif /* PLUGINS_PLUGINS_H_ */
