@@ -25,12 +25,19 @@ namespace lsp
                 bool                bLog;
                 bool                bLogSet;
                 bool                bCyclingSet;
+                float               fDefaultValue;
+                bool                bBalanceSet;
+                float               fBalance;
                 CtlPort            *pPort;
 
             protected:
-                static status_t    slot_change(LSPWidget *sender, void *ptr, void *data);
-                void        submit_value();
-                void        commit_value(float value);
+                static status_t     slot_change(LSPWidget *sender, void *ptr, void *data);
+                static status_t     slot_dbl_click(LSPWidget *sender, void *ptr, void *data);
+
+            protected:
+                void                submit_value();
+                void                set_default_value();
+                void                commit_value(float value);
 
             public:
                 explicit CtlKnob(CtlRegistry *src, LSPKnob *widget);
@@ -38,13 +45,13 @@ namespace lsp
 
             public:
 
-                virtual void init();
+                virtual void        init();
 
-                virtual void set(widget_attribute_t att, const char *value);
+                virtual void        set(widget_attribute_t att, const char *value);
 
-                virtual void notify(CtlPort *port);
+                virtual void        notify(CtlPort *port);
 
-                virtual void end();
+                virtual void        end();
         };
     
     } /* namespace ctl */
