@@ -13,6 +13,7 @@
 #include <data/cvector.h>
 #include <data/cstorage.h>
 #include <core/LSPString.h>
+#include <core/i18n/IDictionary.h>
 
 #include <ui/ws/ws.h>
 
@@ -153,6 +154,7 @@ namespace lsp
             LSPSLOT_CHANGE,             //!< LSPSLOT_CHANGE Triggered only when value(s) stored by the widget is(are) changed
             LSPSLOT_DESTROY,            //!< LSPSLOT_DESTROY Triggered when widget associated with slot is destroyed
             LSPSLOT_RESIZE,             //!< LSPSLOT_RESIZE Triggered when the geometry of widget has been changed
+            LSPSLOT_RESIZE_PARENT,      //!< LSPSLOT_RESIZE_PARENT Triggered when the parent widget has been resized
             LSPSLOT_CLOSE,              //!< LSPSLOT_CLOSE Triggered when the window is requested to close
             LSPSLOT_HSCROLL,            //!< LSPSLOT_HSCROLL Triggered when the horizontal scrolling is applied
             LSPSLOT_VSCROLL,            //!< LSPSLOT_VSCROLL Triggered when the vertical scrolling is applied
@@ -209,6 +211,7 @@ namespace lsp
 #include <ui/tk/sys/LSPSlotSet.h>
 #include <ui/tk/sys/LSPStyle.h>
 #include <ui/tk/sys/LSPColor.h>
+#include <ui/tk/sys/LSPLocalString.h>
 #include <ui/tk/sys/LSPFloat.h>
 #include <ui/tk/sys/LSPTheme.h>
 #include <ui/tk/sys/LSPDisplay.h>
@@ -230,7 +233,9 @@ namespace lsp
 #include <ui/tk/util/LSPTextDataSource.h>
 #include <ui/tk/util/LSPTextDataSink.h>
 #include <ui/tk/util/LSPUrlSink.h>
+#include <ui/tk/util/LSPFileFilterItem.h>
 #include <ui/tk/util/LSPFileFilter.h>
+#include <ui/tk/util/LSPStyleTrigger.h>
 
 // Widget basics
 #include <ui/tk/basic/LSPWidget.h>
@@ -238,6 +243,7 @@ namespace lsp
 #include <ui/tk/basic/LSPWidgetContainer.h>
 
 // Basic widgets
+#include <ui/tk/widgets/LSPVoid.h>
 #include <ui/tk/widgets/LSPWindow.h>
 #include <ui/tk/widgets/LSPBox.h>
 #include <ui/tk/widgets/LSPGrid.h>
