@@ -27,6 +27,16 @@ namespace lsp
             public:
                 static const w_class_t    metadata;
 
+            protected:
+                class Title: public LSPLocalString
+                {
+                    public:
+                        inline Title(LSPWidget *widget): LSPLocalString(widget) {}
+
+                    protected:
+                        virtual void        sync();
+                };
+
             protected:
                 INativeWindow      *pWindow;
                 void               *pNativeHandle;
@@ -46,9 +56,9 @@ namespace lsp
                 float               nVertScale;
                 float               nHorScale;
                 size_t              nBorder;
-                LSPString           sCaption;
                 LSPWindowActions    sActions;
                 LSPColor            sBorder;
+                Title               sTitle;
                 window_poilicy_t    enPolicy;
 
             //---------------------------------------------------------------------------------
@@ -120,9 +130,8 @@ namespace lsp
 
                 inline bool size_request_pending() const { return bSizeRequest; }
 
-                inline const char *title() const { return sCaption.get_native(); }
-
-                inline status_t get_title(LSPString *dst) const { return dst->set(&sCaption) ? STATUS_OK : STATUS_NO_MEM; };
+                inline LSPLocalString *title()                  { return &sTitle; }
+                inline const LSPLocalString *title() const      { return &sTitle; }
 
                 inline window_poilicy_t policy() const          { return enPolicy; }
 
@@ -138,10 +147,6 @@ namespace lsp
             public:
                 virtual void        query_resize();
 
-                virtual status_t    set_title(const char *caption);
-
-                virtual status_t    set_title(const LSPString *value);
-
                 /** Render window's content to surface
                  *
                  * @param s surface to perform rendering
@@ -245,7 +250,9 @@ namespace lsp
 
                 status_t point_child(LSPWidget *focus);
 
-                status_t grab_events();
+                status_t                grab_events(grab_t grab);
+
+                status_t                ungrab_events();
 
                 void                    set_border(size_t border);
 
@@ -256,6 +263,12 @@ namespace lsp
                 void                    set_vscale(float value);
                 void                    set_hscale(float value);
 
+                status_t                set_class(const char *instance, const char *wclass);
+                status_t                set_class(const LSPString *instance, const LSPString *wclass);
+
+                status_t                set_role(const char *role);
+                status_t                set_role(const LSPString *role);
+
             //---------------------------------------------------------------------------------
             // Event handling
             public:
