@@ -36,8 +36,11 @@ namespace lsp
 
         class IGradient
         {
+            private:
+                IGradient & operator = (const IGradient &);
+
             public:
-                IGradient();
+                explicit IGradient();
                 virtual ~IGradient();
 
             public:
@@ -56,6 +59,9 @@ namespace lsp
 
         class Font
         {
+            private:
+                Font & operator = (const Font &);
+
             private:
                 enum flags_t
                 {
@@ -69,11 +75,11 @@ namespace lsp
                 int         nFlags;
 
             public:
-                Font();
-                Font(const char *name);
-                Font(const char *name, float size);
-                Font(float size);
-                Font(const Font *s);
+                explicit Font();
+                explicit Font(const char *name);
+                explicit Font(const char *name, float size);
+                explicit Font(float size);
+                explicit Font(const Font *s);
 
                 ~Font();
 
@@ -100,6 +106,9 @@ namespace lsp
          */
         class ISurface
         {
+            private:
+                ISurface & operator = (const ISurface &);
+
             protected:
                 size_t          nWidth;
                 size_t          nHeight;
@@ -111,7 +120,7 @@ namespace lsp
                 ISurface(size_t width, size_t height, surface_type_t type);
 
             public:
-                ISurface();
+                explicit ISurface();
                 virtual ~ISurface();
 
             public:
