@@ -1079,5 +1079,70 @@ namespace lsp
                 fixup_path();
             return res;
         }
+
+        ssize_t Path::fmt(const char *fmt...)
+        {
+            va_list list;
+            va_start(list, fmt);
+            ssize_t res = sPath.vfmt_utf8(fmt, list);
+            va_end(list);
+            if (res > 0)
+                fixup_path();
+            return res;
+        }
+
+        ssize_t Path::fmt(const LSPString *fmt...)
+        {
+            va_list list;
+            va_start(list, fmt);
+            ssize_t res = sPath.vfmt_utf8(fmt->get_utf8(), list);
+            va_end(list);
+            if (res > 0)
+                fixup_path();
+            return res;
+        }
+
+        ssize_t Path::vfmt(const char *fmt, va_list args)
+        {
+            ssize_t res = sPath.vfmt_utf8(fmt, args);
+            if (res > 0)
+                fixup_path();
+            return res;
+        }
+
+        ssize_t Path::vfmt(const LSPString *fmt, va_list args)
+        {
+            ssize_t res = sPath.vfmt_utf8(fmt->get_utf8(), args);
+            if (res > 0)
+                fixup_path();
+            return res;
+        }
+
+        status_t Path::append(const char *path)
+        {
+            if (path == NULL)
+                return STATUS_BAD_ARGUMENTS;
+            LSPString tmp;
+            if (!tmp.set_utf8(path))
+                return STATUS_NO_MEM;
+            return append(&tmp);
+        }
+
+        status_t Path::append(const LSPString *path)
+        {
+            if (path == NULL)
+                return STATUS_BAD_ARGUMENTS;
+            if (!sPath.append(path))
+                return STATUS_NO_MEM;
+            fixup_path();
+            return STATUS_OK;
+        }
+
+        status_t Path::append(const Path *path)
+        {
+            if (path == NULL)
+                return STATUS_BAD_ARGUMENTS;
+            return append(&path->sPath);
+        }
     }
 } /* namespace lsp */
