@@ -228,6 +228,49 @@ namespace lsp
             return homedir->set(&path);
         }
 
+        status_t get_user_config_path(LSPString *path)
+        {
+            if (path == NULL)
+                return STATUS_BAD_ARGUMENTS;
+            LSPString upath;
+
+#ifdef PLATFORM_WINDOWS
+            status_t res = get_env_var("LOCALAPPDATA", &upath);
+            if (res != STATUS_OK)
+            {
+                res = get_env_var("APPDATA", &upath);
+                if (res != STATUS_OK)
+                {
+                    res = get_home_directory(upath);
+                    if (res != STATUS_OK)
+                        return res;
+                    if (!upath.append_ascii(FILE_SEPARATOR_S ".config"))
+                        return STATUS_NO_MEM;
+                }
+            }
+#else
+            status_t res = get_env_var("HOME", &upath);
+            if (res != STATUS_OK)
+                return res;
+            if (!upath.append_ascii(FILE_SEPARATOR_S ".config"))
+                return STATUS_NO_MEM;
+#endif /* PLATFORM_WINDOWS */
+
+            path->swap(&upath);
+            return STATUS_OK;
+        }
+
+        status_t get_user_config_path(io::Path *path)
+        {
+            if (path == NULL)
+                return STATUS_BAD_ARGUMENTS;
+            LSPString upath;
+            status_t res = get_user_config_path(&upath);
+            if (res != STATUS_OK)
+                return res;
+            return path->set(&upath);
+        }
+
 #ifdef PLATFORM_WINDOWS
         void get_time(time_t *time)
         {
@@ -248,6 +291,43 @@ namespace lsp
             time->nanos     = t.tv_nsec;
         }
 #endif /* PLATFORM_WINDOWS */
+
+#ifdef PLATFORM_WINDOWS
+        status_t get_temporary_dir(LSPString *path)
+        {
+            if (get_env_var("TEMP", path) == STATUS_OK)
+                return STATUS_OK;
+            if (get_env_var("TMP", path) == STATUS_OK)
+                return STATUS_OK;
+            return (path->set_ascii("tmp")) ? STATUS_OK : STATUS_NO_MEM;
+        }
+
+        status_t get_temporary_dir(io::Path *path)
+        {
+            LSPString tmp;
+            if (get_env_var("TEMP", &tmp) == STATUS_OK)
+                return STATUS_OK;
+            if (get_env_var("TMP", &tmp) == STATUS_OK)
+                return STATUS_OK;
+            if (!tmp.set_ascii("tmp"))
+                return STATUS_NO_MEM;
+            return path->set(&tmp);
+        }
+#else
+        status_t get_temporary_dir(LSPString *path)
+        {
+            if (path == NULL)
+                return STATUS_BAD_ARGUMENTS;
+            return (path->set_ascii("/tmp")) ? STATUS_OK : STATUS_NO_MEM;
+        }
+
+        status_t get_temporary_dir(io::Path *path)
+        {
+            if (path == NULL)
+                return STATUS_BAD_ARGUMENTS;
+            return path->set("/tmp");
+        }
+#endif /* PLATFORM_WINDOWS */
     }
 }
 
