@@ -26,6 +26,16 @@ namespace lsp
     }
 
     Oversampler::Oversampler()
+    {
+        construct();
+    }
+
+    Oversampler::~Oversampler()
+    {
+        destroy();
+    }
+
+    void Oversampler::construct()
     {
         pCallback   = NULL;
         fUpBuffer   = NULL;
@@ -37,10 +47,6 @@ namespace lsp
         bData       = NULL;
         bFilter     = true;
     }
-    
-    Oversampler::~Oversampler()
-    {
-    }
 
     bool Oversampler::init()
     {
@@ -78,6 +84,7 @@ namespace lsp
             delete [] bData;
             fUpBuffer   = NULL;
             fDownBuffer = NULL;
+            bData       = NULL;
         }
         pCallback = NULL;
     }
@@ -710,4 +717,18 @@ namespace lsp
         return 0;
     }
 
+    void Oversampler::dump(IStateDumper *v) const
+    {
+        v->write("pCallback", pCallback);
+        v->write("fUpBuffer", fUpBuffer);
+        v->write("fDownBuffer", fDownBuffer);
+        v->write("nUpHead", nUpHead);
+        v->write("nMode", nMode);
+        v->write("nSampleRate", nSampleRate);
+        v->write("nUpdate", nUpdate);
+        v->write_object("sFilter", &sFilter);
+        v->write("bData", bData);
+        v->write("bFilter", bFilter);
+    }
+
 } /* namespace lsp */
