@@ -78,6 +78,10 @@ namespace lsp
 
             if (!c->sSC.init(channels, compressor_base_metadata::REACTIVITY_MAX))
                 return;
+            if (!c->sSCEq.init(2, 12))
+                return;
+            c->sSCEq.set_mode(EQM_IIR);
+            c->sSC.set_pre_equalizer(&c->sSCEq);
 
             c->vIn              = reinterpret_cast<float *>(ptr);
             ptr                += buf_size;
@@ -116,6 +120,10 @@ namespace lsp
             c->pScSource        = NULL;
             c->pScReactivity    = NULL;
             c->pScPreamp        = NULL;
+            c->pScHpfMode       = NULL;
+            c->pScHpfFreq       = NULL;
+            c->pScLpfMode       = NULL;
+            c->pScLpfFreq       = NULL;
 
             c->pMode            = NULL;
             c->pAttackLvl       = NULL;
@@ -124,6 +132,7 @@ namespace lsp
             c->pReleaseTime     = NULL;
             c->pRatio           = NULL;
             c->pKnee            = NULL;
+            c->pBThresh         = NULL;
             c->pMakeup          = NULL;
             c->pDryGain         = NULL;
             c->pWetGain         = NULL;
@@ -190,7 +199,6 @@ namespace lsp
             if ((i > 0) && (nMode == CM_STEREO))
             {
                 channel_t *sc       = &vChannels[0];
-                c->pSC              = sc->pSC;
                 c->pScType          = sc->pScType;
                 c->pScSource        = sc->pScSource;
                 c->pScLookahead     = sc->pScLookahead;
@@ -198,6 +206,10 @@ namespace lsp
                 c->pScListen        = sc->pScListen;
                 c->pScReactivity    = sc->pScReactivity;
                 c->pScPreamp        = sc->pScPreamp;
+                c->pScHpfMode       = sc->pScHpfMode;
+                c->pScHpfFreq       = sc->pScHpfFreq;
+                c->pScLpfMode       = sc->pScLpfMode;
+                c->pScLpfFreq       = sc->pScLpfFreq;
             }
             else
             {
@@ -218,6 +230,14 @@ namespace lsp
                 c->pScReactivity    =   vPorts[port_id++];
                 TRACE_PORT(vPorts[port_id]);
                 c->pScPreamp        =   vPorts[port_id++];
+                TRACE_PORT(vPorts[port_id]);
+                c->pScHpfMode       =   vPorts[port_id++];
+                TRACE_PORT(vPorts[port_id]);
+                c->pScHpfFreq       =   vPorts[port_id++];
+                TRACE_PORT(vPorts[port_id]);
+                c->pScLpfMode       =   vPorts[port_id++];
+                TRACE_PORT(vPorts[port_id]);
+                c->pScLpfFreq       =   vPorts[port_id++];
             }
         }
 
@@ -238,6 +258,7 @@ namespace lsp
                 c->pReleaseTime     = sc->pReleaseTime;
                 c->pRatio           = sc->pRatio;
                 c->pKnee            = sc->pKnee;
+                c->pBThresh         = sc->pBThresh;
                 c->pMakeup          = sc->pMakeup;
                 c->pDryGain         = sc->pDryGain;
                 c->pWetGain         = sc->pWetGain;
@@ -259,6 +280,8 @@ namespace lsp
                 TRACE_PORT(vPorts[port_id]);
                 c->pKnee            =   vPorts[port_id++];
                 TRACE_PORT(vPorts[port_id]);
+                c->pBThresh         =   vPorts[port_id++];
+                TRACE_PORT(vPorts[port_id]);
                 c->pMakeup          =   vPorts[port_id++];
                 TRACE_PORT(vPorts[port_id]);
                 c->pDryGain         =   vPorts[port_id++];
@@ -336,7 +359,9 @@ namespace lsp
             for (size_t i=0; i<channels; ++i)
             {
                 vChannels[i].sSC.destroy();
+                vChannels[i].sSCEq.destroy();
                 vChannels[i].sDelay.destroy();
+                vChannels[i].sCompDelay.destroy();
             }
 
             delete [] vChannels;
@@ -368,7 +393,9 @@ namespace lsp
             c->sBypass.init(sr);
             c->sComp.set_sample_rate(sr);
             c->sSC.set_sample_rate(sr);
+            c->sSCEq.set_sample_rate(sr);
             c->sDelay.init(max_delay);
+            c->sCompDelay.init(max_delay);
 
             for (size_t j=0; j<G_TOTAL; ++j)
                 c->sGraph[j].init(compressor_base_metadata::TIME_MESH_SIZE, samples_per_dot);
@@ -378,6 +405,7 @@ namespace lsp
 
     void compressor_base::update_settings()
     {
+        filter_params_t fp;
         size_t channels = (nMode == CM_MONO) ? 1 : 2;
         bool bypass     = pBypass->getValue() >= 0.5f;
 
@@ -387,6 +415,7 @@ namespace lsp
         bMSListen       = (pMSListen != NULL) ? pMSListen->getValue() >= 0.5f : false;
         fInGain         = pInGain->getValue();
         float out_gain  = pOutGain->getValue();
+        size_t latency  = 0;
 
         for (size_t i=0; i<channels; ++i)
         {
@@ -405,8 +434,31 @@ namespace lsp
             c->sSC.set_reactivity(c->pScReactivity->getValue());
             c->sSC.set_stereo_mode(((nMode == CM_MS) && (c->nScType != SCT_EXTERNAL)) ? SCSM_MIDSIDE : SCSM_STEREO);
 
-            // Update delay
-            c->sDelay.set_delay(millis_to_samples(fSampleRate, (c->pScLookahead != NULL) ? c->pScLookahead->getValue() : 0));
+            // Setup hi-pass filter for sidechain
+            size_t hp_slope = c->pScHpfMode->getValue() * 2;
+            fp.nType        = (hp_slope > 0) ? FLT_BT_BWC_HIPASS : FLT_NONE;
+            fp.fFreq        = c->pScHpfFreq->getValue();
+            fp.fFreq2       = fp.fFreq;
+            fp.fGain        = 1.0f;
+            fp.nSlope       = hp_slope;
+            fp.fQuality     = 0.0f;
+            c->sSCEq.set_params(0, &fp);
+
+            // Setup low-pass filter for sidechain
+            size_t lp_slope = c->pScLpfMode->getValue() * 2;
+            fp.nType        = (lp_slope > 0) ? FLT_BT_BWC_LOPASS : FLT_NONE;
+            fp.fFreq        = c->pScLpfFreq->getValue();
+            fp.fFreq2       = fp.fFreq;
+            fp.fGain        = 1.0f;
+            fp.nSlope       = lp_slope;
+            fp.fQuality     = 0.0f;
+            c->sSCEq.set_params(1, &fp);
+
+            // Update delay and estimate overall delay
+            size_t delay    = millis_to_samples(fSampleRate, (c->pScLookahead != NULL) ? c->pScLookahead->getValue() : 0);
+            c->sDelay.set_delay(delay);
+            if (delay > latency)
+                latency         = delay;
 
             // Update compressor settings
             float attack    = c->pAttackLvl->getValue();
@@ -418,6 +470,7 @@ namespace lsp
             c->sComp.set_timings(c->pAttackTime->getValue(), c->pReleaseTime->getValue());
             c->sComp.set_ratio(c->pRatio->getValue());
             c->sComp.set_knee(c->pKnee->getValue());
+            c->sComp.set_boost_threshold(c->pBThresh->getValue());
             c->sComp.set_mode((upward) ? CM_UPWARD : CM_DOWNWARD);
             if (c->pReleaseOut != NULL)
                 c->pReleaseOut->setValue(release);
@@ -439,6 +492,16 @@ namespace lsp
                 c->nSync           |= S_CURVE;
             }
         }
+
+        // Tune compensation delays
+        for (size_t i=0; i<channels; ++i)
+        {
+            channel_t *c    = &vChannels[i];
+            c->sCompDelay.set_delay(latency - c->sDelay.get_delay());
+        }
+
+        // Report latency
+        set_latency(latency);
     }
 
     void compressor_base::ui_activated()
@@ -505,7 +568,7 @@ namespace lsp
             if (c->nScType == SCT_FEED_BACK)
                 feedback |= (1 << i);
 
-//            dump_buffer("in_buf", in_buf[i], samples);
+//            lsp_dumpf("in_buf", "%g", in_buf[i], samples);
         }
 
         // Perform compression
@@ -643,8 +706,11 @@ namespace lsp
             {
                 channel_t *c        = &vChannels[i];
 
-                c->sDelay.process(c->vIn, c->vIn, to_process); // Add delay to original signal
-                dsp::mul3(c->vOut, c->vGain, c->vIn, to_process);
+                // Add delay to original signal and apply gain
+                c->sDelay.process(c->vOut, c->vIn, c->vGain, to_process);
+
+                // Apply latency compensation delay
+                c->sCompDelay.process(c->vOut, c->vOut, to_process);
 
                 // Process graph outputs
                 if ((i == 0) || (nMode != CM_STEREO))
@@ -904,6 +970,108 @@ namespace lsp
         return true;
     }
 
+    void compressor_base::dump(IStateDumper *v) const
+    {
+        size_t channels = (nMode == CM_MONO) ? 1 : 2;
+
+        v->write("nMode", nMode);
+        v->write("nChannels", channels);
+        v->write("bSidechain", bSidechain);
+
+        v->begin_array("vChannels", vChannels, channels);
+        for (size_t i=0; i<channels; ++i)
+        {
+            const channel_t *c = &vChannels[i];
+
+            v->begin_object(c, sizeof(channel_t));
+            {
+                v->write_object("sBypass", &c->sBypass);
+                v->write_object("sSC", &c->sSC);
+                v->write_object("sSCEq", &c->sSCEq);
+                v->write_object("sComp", &c->sComp);
+                v->write_object("sDelay", &c->sDelay);
+                v->write_object("sCompDelay", &c->sCompDelay);
+
+                v->begin_array("sGraph", c->sGraph, G_TOTAL);
+                for (size_t j=0; j<G_TOTAL; ++j)
+                    v->write_object(&c->sGraph[j]);
+                v->end_array();
+
+                v->write("vIn", c->vIn);
+                v->write("vOut", c->vOut);
+                v->write("vSc", c->vSc);
+                v->write("vEnv", c->vEnv);
+                v->write("vGain", c->vGain);
+                v->write("bScListen", c->bScListen);
+                v->write("nSync", c->nSync);
+                v->write("nScType", c->nScType);
+                v->write("fMakeup", c->fMakeup);
+                v->write("fFeedback", c->fFeedback);
+                v->write("fDryGain", c->fDryGain);
+                v->write("fWetGain", c->fWetGain);
+                v->write("fDotIn", c->fDotIn);
+                v->write("fDotOut", c->fDotOut);
+
+                v->write("pIn", c->pIn);
+                v->write("pOut", c->pOut);
+                v->write("pSC", c->pSC);
+                v->begin_array("pGraph", c->pGraph, G_TOTAL);
+                for (size_t j=0; j<G_TOTAL; ++j)
+                    v->write(c->pGraph[j]);
+                v->end_array();
+                v->begin_array("pMeter", c->pGraph, M_TOTAL);
+                for (size_t j=0; j<M_TOTAL; ++j)
+                    v->write(c->pMeter[j]);
+                v->end_array();
+
+                v->write("pScType", c->pScType);
+                v->write("pScMode", c->pScMode);
+                v->write("pScLookahead", c->pScLookahead);
+                v->write("pScListen", c->pScListen);
+                v->write("pScSource", c->pScSource);
+                v->write("pScReactivity", c->pScReactivity);
+                v->write("pScPreamp", c->pScPreamp);
+                v->write("pScHpfMode", c->pScHpfMode);
+                v->write("pScHpfFreq", c->pScHpfFreq);
+                v->write("pScLpfMode", c->pScLpfMode);
+                v->write("pScLpfFreq", c->pScLpfFreq);
+
+                v->write("pMode", c->pMode);
+                v->write("pAttackLvl", c->pAttackLvl);
+                v->write("pReleaseLvl", c->pReleaseLvl);
+                v->write("pAttackTime", c->pAttackTime);
+                v->write("pReleaseTime", c->pReleaseTime);
+                v->write("pRatio", c->pRatio);
+                v->write("pKnee", c->pKnee);
+                v->write("pBThresh", c->pBThresh);
+                v->write("pMakeup", c->pMakeup);
+
+                v->write("pDryGain", c->pDryGain);
+                v->write("pWetGain", c->pWetGain);
+                v->write("pCurve", c->pCurve);
+                v->write("pReleaseOut", c->pReleaseOut);
+            }
+            v->end_object();
+        }
+        v->end_array();
+
+        v->write("vCurve", vCurve);
+        v->write("vTime", vTime);
+        v->write("bPause", bPause);
+        v->write("bClear", bClear);
+        v->write("bMSListen", bMSListen);
+        v->write("fInGain", fInGain);
+        v->write("bUISync", bUISync);
+        v->write("pIDisplay", pIDisplay);
+        v->write("pBypass", pBypass);
+        v->write("pInGain", pInGain);
+        v->write("pOutGain", pOutGain);
+        v->write("pPause", pPause);
+        v->write("pClear", pClear);
+        v->write("pMSListen", pMSListen);
+        v->write("pData", pData);
+    }
+
     //-------------------------------------------------------------------------
     // Compressor derivatives
     compressor_mono::compressor_mono() : compressor_base(metadata, false, CM_MONO)
