@@ -59,6 +59,13 @@ static const int *glx_pb_config[] =
 //#define TRACK_GL_ERRORS { GLenum glErr; while ((glErr = glGetError()) != GL_NO_ERROR) lsp_error("GL ERROR code=%d", int(glErr)); }
 #define TRACK_GL_ERRORS
 
+namespace lsp
+{
+#ifdef LSP_TRACEFILE
+    FILE *log_fd = stderr;
+#endif /* LSP_TRACEFILE */
+}
+
 namespace lsp
 {
     void glx_backend_t::destroy(glx_backend_t *_this)
