@@ -22,6 +22,8 @@ namespace native
     void    fmmul_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
     void    fmdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
     void    fmrdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+    void    fmmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+    void    fmrmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
 }
 
 IF_ARCH_X86(
@@ -35,6 +37,12 @@ IF_ARCH_X86(
         void    fmrdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
     }
 
+    namespace sse2
+    {
+        void    fmmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmrmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+    }
+
     namespace avx
     {
         void    fmadd_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
@@ -43,10 +51,14 @@ IF_ARCH_X86(
         void    fmmul_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmrdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmrmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
 
         void    fmadd_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmsub_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmrsub_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmmod_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmrmod_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
     }
 
     namespace avx2
@@ -57,10 +69,14 @@ IF_ARCH_X86(
         void    fmmul_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmrdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmrmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
 
         void    fmadd_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmsub_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmrsub_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmmod_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmrmod_k4_fma3(float *dst, const float *src1, const float *src2, float k, size_t count);
     }
 )
 
@@ -73,6 +89,8 @@ IF_ARCH_ARM(
         void    fmmul_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmrdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmrmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
     }
 )
 
@@ -85,6 +103,8 @@ IF_ARCH_AARCH64(
         void    fmmul_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
         void    fmrdiv_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
+        void    fmrmod_k4(float *dst, const float *src1, const float *src2, float k, size_t count);
     }
 )
 
@@ -179,6 +199,26 @@ PTEST_BEGIN("dsp.pmath", fmop_k4, 5, 1000)
             IF_ARCH_X86(CALL(avx2::fmrdiv_k4));
             IF_ARCH_ARM(CALL(neon_d32::fmrdiv_k4));
             IF_ARCH_AARCH64(CALL(asimd::fmrdiv_k4));
+            PTEST_SEPARATOR;
+
+            CALL(native::fmmod_k4);
+            IF_ARCH_X86(CALL(sse2::fmmod_k4));
+            IF_ARCH_X86(CALL(avx::fmmod_k4));
+            IF_ARCH_X86(CALL(avx::fmmod_k4_fma3));
+            IF_ARCH_X86(CALL(avx2::fmmod_k4));
+            IF_ARCH_X86(CALL(avx2::fmmod_k4_fma3));
+            IF_ARCH_ARM(CALL(neon_d32::fmmod_k4));
+            IF_ARCH_AARCH64(CALL(asimd::fmmod_k4));
+            PTEST_SEPARATOR;
+
+            CALL(native::fmrmod_k4);
+            IF_ARCH_X86(CALL(sse2::fmrmod_k4));
+            IF_ARCH_X86(CALL(avx::fmrmod_k4));
+            IF_ARCH_X86(CALL(avx::fmrmod_k4_fma3));
+            IF_ARCH_X86(CALL(avx2::fmrmod_k4));
+            IF_ARCH_X86(CALL(avx2::fmrmod_k4_fma3));
+            IF_ARCH_ARM(CALL(neon_d32::fmrmod_k4));
+            IF_ARCH_AARCH64(CALL(asimd::fmrmod_k4));
             PTEST_SEPARATOR2;
         }
 
