@@ -16,10 +16,12 @@ namespace native
 {
     void    add_k2(float *dst, float k, size_t count);
     void    sub_k2(float *dst, float k, size_t count);
+    void    rsub_k2(float *dst, float k, size_t count);
     void    mul_k2(float *dst, float k, size_t count);
     void    div_k2(float *dst, float k, size_t count);
-    void    rsub_k2(float *dst, float k, size_t count);
     void    rdiv_k2(float *dst, float k, size_t count);
+    void    mod_k2(float *dst, float k, size_t count);
+    void    rmod_k2(float *dst, float k, size_t count);
 }
 
 IF_ARCH_X86(
@@ -27,30 +29,44 @@ IF_ARCH_X86(
     {
         void    add_k2(float *dst, float k, size_t count);
         void    sub_k2(float *dst, float k, size_t count);
+        void    rsub_k2(float *dst, float k, size_t count);
         void    mul_k2(float *dst, float k, size_t count);
         void    div_k2(float *dst, float k, size_t count);
-        void    rsub_k2(float *dst, float k, size_t count);
         void    rdiv_k2(float *dst, float k, size_t count);
     }
 
+    namespace sse2
+    {
+        void    mod_k2(float *dst, float k, size_t count);
+        void    rmod_k2(float *dst, float k, size_t count);
+    }
+
     namespace avx
     {
         void    add_k2(float *dst, float k, size_t count);
         void    sub_k2(float *dst, float k, size_t count);
+        void    rsub_k2(float *dst, float k, size_t count);
         void    mul_k2(float *dst, float k, size_t count);
         void    div_k2(float *dst, float k, size_t count);
-        void    rsub_k2(float *dst, float k, size_t count);
         void    rdiv_k2(float *dst, float k, size_t count);
+        void    mod_k2(float *dst, float k, size_t count);
+        void    rmod_k2(float *dst, float k, size_t count);
+        void    mod_k2_fma3(float *dst, float k, size_t count);
+        void    rmod_k2_fma3(float *dst, float k, size_t count);
     }
 
     namespace avx2
     {
         void    add_k2(float *dst, float k, size_t count);
         void    sub_k2(float *dst, float k, size_t count);
+        void    rsub_k2(float *dst, float k, size_t count);
         void    mul_k2(float *dst, float k, size_t count);
         void    div_k2(float *dst, float k, size_t count);
-        void    rsub_k2(float *dst, float k, size_t count);
         void    rdiv_k2(float *dst, float k, size_t count);
+        void    mod_k2(float *dst, float k, size_t count);
+        void    rmod_k2(float *dst, float k, size_t count);
+        void    mod_k2_fma3(float *dst, float k, size_t count);
+        void    rmod_k2_fma3(float *dst, float k, size_t count);
     }
 )
 
@@ -59,10 +75,12 @@ IF_ARCH_ARM(
     {
         void    add_k2(float *dst, float k, size_t count);
         void    sub_k2(float *dst, float k, size_t count);
+        void    rsub_k2(float *dst, float k, size_t count);
         void    mul_k2(float *dst, float k, size_t count);
         void    div_k2(float *dst, float k, size_t count);
-        void    rsub_k2(float *dst, float k, size_t count);
         void    rdiv_k2(float *dst, float k, size_t count);
+        void    mod_k2(float *dst, float k, size_t count);
+        void    rmod_k2(float *dst, float k, size_t count);
     }
 )
 
@@ -71,10 +89,12 @@ IF_ARCH_AARCH64(
     {
         void    add_k2(float *dst, float k, size_t count);
         void    sub_k2(float *dst, float k, size_t count);
+        void    rsub_k2(float *dst, float k, size_t count);
         void    mul_k2(float *dst, float k, size_t count);
         void    div_k2(float *dst, float k, size_t count);
-        void    rsub_k2(float *dst, float k, size_t count);
         void    rdiv_k2(float *dst, float k, size_t count);
+        void    mod_k2(float *dst, float k, size_t count);
+        void    rmod_k2(float *dst, float k, size_t count);
     }
 )
 
@@ -162,6 +182,26 @@ PTEST_BEGIN("dsp.pmath", op_k2, 5, 1000)
             IF_ARCH_X86(CALL(avx2::rdiv_k2));
             IF_ARCH_ARM(CALL(neon_d32::rdiv_k2));
             IF_ARCH_AARCH64(CALL(asimd::rdiv_k2));
+            PTEST_SEPARATOR;
+
+            CALL(native::mod_k2);
+            IF_ARCH_X86(CALL(sse2::mod_k2));
+            IF_ARCH_X86(CALL(avx::mod_k2));
+            IF_ARCH_X86(CALL(avx::mod_k2_fma3));
+            IF_ARCH_X86(CALL(avx2::mod_k2));
+            IF_ARCH_X86(CALL(avx2::mod_k2_fma3))
+            IF_ARCH_ARM(CALL(neon_d32::mod_k2));
+            IF_ARCH_AARCH64(CALL(asimd::mod_k2));
+            PTEST_SEPARATOR;
+
+            CALL(native::rmod_k2);
+            IF_ARCH_X86(CALL(sse2::rmod_k2));
+            IF_ARCH_X86(CALL(avx::rmod_k2));
+            IF_ARCH_X86(CALL(avx::rmod_k2_fma3));
+            IF_ARCH_X86(CALL(avx2::rmod_k2));
+            IF_ARCH_X86(CALL(avx2::rmod_k2_fma3));
+            IF_ARCH_ARM(CALL(neon_d32::rmod_k2));
+            IF_ARCH_AARCH64(CALL(asimd::rmod_k2));
             PTEST_SEPARATOR2;
         }
 
