@@ -31,6 +31,7 @@ UTEST_BEGIN("core.files.json", serializer)
                     "true,"
                     "null,"
                     "\"multiline\\n\\rstring\","
+                    "\"_\\uD834\\uDD1E_\","
                     "{},"
                     "[]"
                 "],"
@@ -40,6 +41,7 @@ UTEST_BEGIN("core.files.json", serializer)
                     "\"bool\":false,"
                     "\"null\":null,"
                     "\"string\":\"test\\n\","
+                    "\"special\":\"_\\uD834\\uDD1E_\","
                     "\"object\":{},"
                     "\"array\":[]"
                 "}"
@@ -69,6 +71,7 @@ UTEST_BEGIN("core.files.json", serializer)
                 UTEST_ASSERT(s.write_null() == STATUS_OK);
                 UTEST_ASSERT(s.write_string("multiline\n\rstring") == STATUS_OK);
                 UTEST_ASSERT(s.write_comment("comment") == STATUS_INVALID_VALUE);
+                UTEST_ASSERT(s.write_string("_\xf0\x9d\x84\x9e_") == STATUS_OK);
 
                 UTEST_ASSERT(s.start_object() == STATUS_OK);
                 UTEST_ASSERT(s.end_object() == STATUS_OK);
@@ -98,6 +101,9 @@ UTEST_BEGIN("core.files.json", serializer)
 
                 UTEST_ASSERT(s.write_comment("comment") == STATUS_INVALID_VALUE);
 
+                UTEST_ASSERT(s.write_property("special") == STATUS_OK);
+                UTEST_ASSERT(s.write_string("_\xf0\x9d\x84\x9e_") == STATUS_OK);
+
                 UTEST_ASSERT(s.write_property("object") == STATUS_OK);
                 UTEST_ASSERT(s.start_object() == STATUS_OK);
                 UTEST_ASSERT(s.end_object() == STATUS_OK);
@@ -126,8 +132,8 @@ UTEST_BEGIN("core.files.json", serializer)
         Serializer s;
 
         const char *data =
-            "/*c1\\U000A*/{"
-                "/*/\\U002Ac2*\\U002F*/"
+            "/*c1\\u000A*/{"
+                "/*/\\u002Ac2*\\u002F*/"
                 "array:["
                     "123,/*q0*/"
                     "-0x123,"
@@ -135,6 +141,7 @@ UTEST_BEGIN("core.files.json", serializer)
                     "true,"
                     "null,"
                     "\"multiline\\n\\rstring\"/*c3*/,/*q1*/"
+                    "\"_\\uD834\\uDD1E_\","
                     "{}/*q2*/,"
                     "[]/*q3*/"
                 "],"
@@ -145,6 +152,7 @@ UTEST_BEGIN("core.files.json", serializer)
                     "bool:false,"
                     "\"null\":null,"
                     "string:\"test\\n\"/*comment*/,"
+                    "special:\"_\\uD834\\uDD1E_\","
                     "object:{/*q10*/},/*q9*/"
                     "array:[/*q12*/]/*q11*/,"
                 "}/*c6*/"
@@ -189,6 +197,9 @@ UTEST_BEGIN("core.files.json", serializer)
                 UTEST_ASSERT(s.write_comma() == STATUS_OK);
                 UTEST_ASSERT(s.write_comment("q1") == STATUS_OK);
 
+                UTEST_ASSERT(s.write_string("_\xf0\x9d\x84\x9e_") == STATUS_OK);
+                UTEST_ASSERT(s.write_comma() == STATUS_OK)
+
                 UTEST_ASSERT(s.start_object() == STATUS_OK);
                 UTEST_ASSERT(s.end_object() == STATUS_OK);
                 UTEST_ASSERT(s.write_comment("q2") == STATUS_OK);
@@ -224,9 +235,11 @@ UTEST_BEGIN("core.files.json", serializer)
 
                 UTEST_ASSERT(s.write_property("string") == STATUS_OK);
                 UTEST_ASSERT(s.write_string("test\n") == STATUS_OK);
-
                 UTEST_ASSERT(s.write_comment("comment") == STATUS_OK);
 
+                UTEST_ASSERT(s.write_property("special") == STATUS_OK);
+                UTEST_ASSERT(s.write_string("_\xf0\x9d\x84\x9e_") == STATUS_OK);
+
                 UTEST_ASSERT(s.write_property("object") == STATUS_OK);
                 UTEST_ASSERT(s.start_object() == STATUS_OK);
                 UTEST_ASSERT(s.write_comma() == STATUS_INVALID_VALUE);
@@ -267,7 +280,7 @@ UTEST_BEGIN("core.files.json", serializer)
             " * c1\n"
             " */\n"
             "{\n"
-            "    /*/\\U002A array *\\U002F*/\n"
+            "    /*/\\u002A array *\\u002F*/\n"
             "    array: [\n"
             "        123, /* integer */\n"
             "        -0x123, /* hex */\n"
@@ -275,6 +288,7 @@ UTEST_BEGIN("core.files.json", serializer)
             "        true, /* boolean */\n"
             "        null, /* null string */\n"
             "        \"multiline\\n\\rstring\", /* multiline string */\n"
+            "        \"_\\uD834\\uDD1E_\",\n"
             "        {}, /* empty object */\n"
             "        [], /* empty array */\n"
             "    ], /* end of array */\n"
@@ -285,6 +299,7 @@ UTEST_BEGIN("core.files.json", serializer)
             "        \"double\": -Infinity,\n"
             "        bool: false,\n"
             "        \"null\": null,\n"
+            "        special: \"_\\uD834\\uDD1E_\",\n"
             "        string: /* key */ \"test\\n\" /* value */ /*comment*/,\n"
             "        object: /* key */ {} /* value */,\n"
             "        array: /* key */ [] /* value */\n"
@@ -343,6 +358,9 @@ UTEST_BEGIN("core.files.json", serializer)
                 UTEST_ASSERT(s.write_comma() == STATUS_OK);
                 UTEST_ASSERT(s.write_comment(" multiline string ") == STATUS_OK);
 
+                UTEST_ASSERT(s.write_string("_\xf0\x9d\x84\x9e_") == STATUS_OK);
+                UTEST_ASSERT(s.write_comma() == STATUS_OK);
+
                 UTEST_ASSERT(s.start_object() == STATUS_OK);
                 UTEST_ASSERT(s.end_object() == STATUS_OK);
                 UTEST_ASSERT(s.write_comma() == STATUS_OK);
@@ -378,6 +396,9 @@ UTEST_BEGIN("core.files.json", serializer)
                 UTEST_ASSERT(s.write_property("null") == STATUS_OK);
                 UTEST_ASSERT(s.write_null() == STATUS_OK);
 
+                UTEST_ASSERT(s.write_property("special") == STATUS_OK);
+                UTEST_ASSERT(s.write_string("_\xf0\x9d\x84\x9e_") == STATUS_OK);
+
                 UTEST_ASSERT(s.write_property("string") == STATUS_OK);
                 UTEST_ASSERT(s.write_comment(" key ") == STATUS_OK);
                 UTEST_ASSERT(s.write_string("test\n") == STATUS_OK);
