@@ -63,6 +63,7 @@ namespace lsp
             "vbox",
             "vgrid",
             "viewer3d",
+            "void",
             "vsbar",
             "vsbox",
             "vsep",
@@ -98,6 +99,8 @@ namespace lsp
             "detailed",
             "distance_id",
             "duration_id",
+            "dx",
+            "dy",
             "editable",
             "embed",
             "expand",
