@@ -79,6 +79,7 @@ namespace lsp
             if (id >= 0) id = sSlots.add(LSPSLOT_SHOW, slot_show, self());
             if (id >= 0) id = sSlots.add(LSPSLOT_DESTROY, slot_destroy, self());
             if (id >= 0) id = sSlots.add(LSPSLOT_RESIZE, slot_resize, self());
+            if (id >= 0) id = sSlots.add(LSPSLOT_RESIZE_PARENT, slot_resize_parent, self());
             if (id >= 0) id = sSlots.add(LSPSLOT_DRAG_REQUEST, slot_drag_request, self());
 
             return (id >= 0) ? STATUS_OK : -id;
@@ -265,6 +266,16 @@ namespace lsp
             return _this->on_resize(ev);
         }
 
+        status_t LSPWidget::slot_resize_parent(LSPWidget *sender, void *ptr, void *data)
+        {
+            if ((ptr == NULL) || (data == NULL))
+                return STATUS_BAD_ARGUMENTS;
+
+            LSPWidget *_this  = static_cast<LSPWidget *>(ptr);
+            realize_t *ev   = static_cast<realize_t *>(data);
+            return _this->on_resize_parent(ev);
+        }
+
         status_t LSPWidget::slot_focus_in(LSPWidget *sender, void *ptr, void *data)
         {
             if ((ptr == NULL) || (data == NULL))
@@ -587,9 +598,10 @@ namespace lsp
                 (sSize.nHeight == r->nHeight))
                 return;
 
-            // Update size and execute slot
+            // Execute slot and update size
+            realize_t xr = *r;
+            sSlots.execute(LSPSLOT_RESIZE, this, &xr);
             sSize       = *r;
-            sSlots.execute(LSPSLOT_RESIZE, this, &sSize);
         }
 
         void LSPWidget::size_request(size_request_t *r)
@@ -733,6 +745,11 @@ namespace lsp
             return STATUS_OK;
         }
 
+        status_t LSPWidget::on_resize_parent(const realize_t *r)
+        {
+            return STATUS_OK;
+        }
+
         status_t LSPWidget::on_hide()
         {
             return STATUS_OK;
