@@ -8,6 +8,8 @@
 #include <ui/ws/ws.h>
 #include <ui/tk/tk.h>
 
+#include <core/i18n/Dictionary.h>
+
 #if defined(USE_X11_DISPLAY)
     #include <ui/ws/x11/ws.h>
 #else
@@ -20,6 +22,7 @@ namespace lsp
     {
         LSPDisplay::LSPDisplay()
         {
+            pDictionary     = NULL;
             pDisplay        = NULL;
         }
 
@@ -69,6 +72,13 @@ namespace lsp
                 delete pDisplay;
                 pDisplay = NULL;
             }
+
+            // Destroy dictionary
+            if (pDictionary != NULL)
+            {
+                delete pDictionary;
+                pDictionary = NULL;
+            }
         }
 
         status_t LSPDisplay::main_task_handler(ws::timestamp_t time, void *arg)
@@ -145,20 +155,32 @@ namespace lsp
             if (dpy == NULL)
                 return STATUS_BAD_ARGUMENTS;
 
+            // Initialize dictionary
+            Dictionary *dict = new Dictionary();
+            if (dict == NULL)
+                return STATUS_NO_MEM;
+
             // Initialize display
-            pDisplay    = dpy;
+            pDisplay        = dpy;
             pDisplay->set_main_callback(main_task_handler, this);
 
             // Create slots
-            LSPSlot *slot = sSlots.add(LSPSLOT_DESTROY);
+            LSPSlot *slot   = sSlots.add(LSPSLOT_DESTROY);
             if (slot == NULL)
+            {
+                delete dict;
                 return STATUS_NO_MEM;
+            }
             slot = sSlots.add(LSPSLOT_RESIZE);
             if (slot == NULL)
+            {
+                delete dict;
                 return STATUS_NO_MEM;
+            }
 
             // Initialize theme
             sTheme.init(this);
+            pDictionary     = dict;
 
             return STATUS_OK;
         }
@@ -190,7 +212,7 @@ namespace lsp
 
         ISurface *LSPDisplay::create_surface(size_t width, size_t height)
         {
-            return (pDisplay != NULL) ? pDisplay->createSurface(width, height) : NULL;
+            return (pDisplay != NULL) ? pDisplay->create_surface(width, height) : NULL;
         }
 
         void LSPDisplay::sync()
@@ -353,27 +375,27 @@ namespace lsp
 
         status_t LSPDisplay::get_clipboard(size_t id, IDataSink *sink)
         {
-            return pDisplay->getClipboard(id, sink);
+            return pDisplay->get_clipboard(id, sink);
         }
 
         status_t LSPDisplay::set_clipboard(size_t id, IDataSource *src)
         {
-            return pDisplay->setClipboard(id, src);
+            return pDisplay->set_clipboard(id, src);
         }
 
         status_t LSPDisplay::reject_drag()
         {
-            return pDisplay->rejectDrag();
+            return pDisplay->reject_drag();
         }
 
         status_t LSPDisplay::accept_drag(IDataSink *sink, drag_t action, bool internal, const realize_t *r)
         {
-            return pDisplay->acceptDrag(sink, action, internal, r);
+            return pDisplay->accept_drag(sink, action, internal, r);
         }
 
         const char * const *LSPDisplay::get_drag_mime_types()
         {
-            return pDisplay->getDragContentTypes();
+            return pDisplay->get_drag_ctypes();
         }
 
         status_t LSPDisplay::queue_destroy(LSPWidget *widget)
