@@ -26,7 +26,6 @@ namespace lsp
             nButtons    = 0;
 
             fValue      = 0.5f;
-            fDflValue   = 0.5f;
             fStep       = 0.01f;
             fTinyStep   = 0.001f;
             fMin        = 0.0f;
@@ -129,15 +128,6 @@ namespace lsp
             query_draw();
         }
 
-        void LSPKnob::set_default_value(float value)
-        {
-            value       = limit_value(value);
-            if (fDflValue == value)
-                return;
-
-            fDflValue   = value;
-        }
-
         void LSPKnob::set_step(float value)
         {
             fStep       = value;
@@ -374,17 +364,17 @@ namespace lsp
             return STATUS_OK;
         }
 
-        status_t LSPKnob::on_mouse_dbl_click(const ws_event_t *e)
-        {
-//            lsp_trace("x=%d, y=%d, state=%x, code=%x", int(e->nLeft), int(e->nTop), int(e->nState), int(e->nCode));
-            if (check_mouse_over(e->nLeft, e->nTop) == S_NONE)
-                return STATUS_OK;
-
-            set_value(fDflValue);
-            sSlots.execute(LSPSLOT_CHANGE, this);
-
-            return STATUS_OK;
-        }
+//        status_t LSPKnob::on_mouse_dbl_click(const ws_event_t *e)
+//        {
+////            lsp_trace("x=%d, y=%d, state=%x, code=%x", int(e->nLeft), int(e->nTop), int(e->nState), int(e->nCode));
+//            if (check_mouse_over(e->nLeft, e->nTop) == S_NONE)
+//                return STATUS_OK;
+//
+//            set_value(fDflValue);
+//            sSlots.execute(LSPSLOT_CHANGE, this);
+//
+//            return STATUS_OK;
+//        }
 
         void LSPKnob::draw(ISurface *s)
         {
