@@ -28,6 +28,7 @@
 #include <utils/resource_gen/xml.h>
 #include <utils/resource_gen/scene3d.h>
 #include <utils/resource_gen/presets.h>
+#include <utils/resource_gen/json.h>
 
 namespace lsp
 {
@@ -48,9 +49,10 @@ namespace lsp
             printf("Preprocessing resource file %s\n", resource->path);
             switch (resource->type)
             {
-                case RESOURCE_XML: return preprocess_xml_resource(resource, sdict);
-                case RESOURCE_3D_SCENE: return preprocess_3d_scene(resource, sdict, fdict);
-                case RESOURCE_PRESET: return preprocess_preset(resource, sdict, fdict);
+                case resource::RESOURCE_XML: return preprocess_xml_resource(resource, sdict);
+                case resource::RESOURCE_3D_SCENE: return preprocess_3d_scene(resource, sdict, fdict);
+                case resource::RESOURCE_PRESET: return preprocess_preset(resource, sdict, fdict);
+                case resource::RESOURCE_JSON: return preprocess_json(resource, sdict, fdict);
                 default: break;
             }
 
@@ -66,9 +68,10 @@ namespace lsp
             printf("Serializing resource file %s\n", resource->path);
             switch (resource->type)
             {
-                case RESOURCE_XML: return serialize_xml_resource(out, resource, dict);
-                case RESOURCE_3D_SCENE: return serialize_3d_scene(out, resource, dict, fdict);
-                case RESOURCE_PRESET: return serialize_preset(out, resource, dict, fdict);
+                case resource::RESOURCE_XML: return serialize_xml_resource(out, resource, dict);
+                case resource::RESOURCE_3D_SCENE: return serialize_3d_scene(out, resource, dict, fdict);
+                case resource::RESOURCE_PRESET: return serialize_preset(out, resource, dict, fdict);
+                case resource::RESOURCE_JSON: return serialize_json(out, resource, dict, fdict);
                 default: break;
             }
 
@@ -130,11 +133,11 @@ namespace lsp
                     continue;
 
                 // Check file type
-                int rtype = RESOURCE_UNKNOWN;
+                int rtype = resource::RESOURCE_UNKNOWN;
                 if (fattr.type == io::fattr_t::FT_REGULAR)
                 {
                     rtype = get_resource_type(ipath.get_utf8());
-                    if (rtype == RESOURCE_UNKNOWN)
+                    if (rtype == resource::RESOURCE_UNKNOWN)
                         continue;
                 }
                 else if (fattr.type != io::fattr_t::FT_DIRECTORY)
@@ -277,6 +280,8 @@ namespace lsp
             fprintf(out,    "// Resource definition\n");
             fprintf(out,    "namespace lsp\n");
             fprintf(out,    "{\n");
+            fprintf(out,    "namespace resource\n");
+            fprintf(out,    "{\n");
 
             // Convert XML files into CPP code
             cvector<xml_word_t> sdict;
@@ -326,7 +331,7 @@ namespace lsp
             // Write footer
             if (result == STATUS_OK)
             {
-                fprintf(out,    "\textern const resource_t builtin_resources[] =\n");
+                fprintf(out,    "\tconst resource_t builtin_resources[] =\n");
                 fprintf(out,    "\t{\n");
 
                 for (size_t i=0; i<resources.size(); ++i)
@@ -335,10 +340,11 @@ namespace lsp
                     fprintf(out,    "\t\t{ \"%s\", builtin_resource%s, %d },\n", res->id, res->hex, res->type);
                 }
 
-                fprintf(out,    "\t\t{ NULL, NULL, %d }\n", RESOURCE_UNKNOWN);
+                fprintf(out,    "\t\t{ NULL, NULL, %d }\n", resource::RESOURCE_UNKNOWN);
                 fprintf(out,    "\t};\n\n");
 
-                fprintf(out,    "}\n"); // End of namespace
+                fprintf(out,    "} /* namespace resource */\n"); // End of namespace
+                fprintf(out,    "} /* namespace lsp */\n"); // End of namespace
             }
 
             // Free dictionary
